@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id:  qt/display.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/display.h   3.3.7   edited Aug 31 2005 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -54,9 +54,10 @@ private:
 class Screen : public QFrame {
     Q_OBJECT
 public:
-    enum { MaxSamples = 500, Range = 100, FrameWidth = 3 };
+    enum { FrameWidth = 3 };
     Screen( QWidget *parent=0, const char *name=0 );
-
+    ~Screen();
+    
     void animate();
 
 public slots:
@@ -64,9 +65,10 @@ public slots:
 
 protected:
     virtual void drawContents( QPainter * );
+    virtual void resizeEvent( QResizeEvent * );
 
 private:
-    int yval[ MaxSamples ];
+    int *yval;
     int pos0; // buffer pointer for x == 0
     int t0;   // time parameter at x == 0
     int step;
@@ -74,7 +76,7 @@ private:
 
 class Curve : public QFrame {
     Q_OBJECT
-    enum { FrameWidth = 3, Radius = 100 };
+    enum { FrameWidth = 3 };
 public:
     Curve( QWidget *parent=0, const char *name=0 );
 
