@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id:  qt/textedit.cpp   3.0.3   edited Oct 12 12:18 $
+** $Id: qt/textedit.cpp   3.3.7   edited Aug 31 2005 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -49,16 +49,16 @@ void TextEdit::setupFileActions()
     menuBar()->insertItem( tr( "&File" ), menu );
 
     QAction *a;
-    a = new QAction( tr( "New" ), QPixmap( "textdrawing/filenew.xpm" ), tr( "&New..." ), CTRL + Key_N, this, "fileNew" );
+    a = new QAction( tr( "New" ), QPixmap( "textdrawing/filenew.png" ), tr( "&New..." ), CTRL + Key_N, this, "fileNew" );
     connect( a, SIGNAL( activated() ), this, SLOT( fileNew() ) );
     a->addTo( tb );
     a->addTo( menu );
-    a = new QAction( tr( "Open" ), QPixmap( "textdrawing/fileopen.xpm" ), tr( "&Open..." ), CTRL + Key_O, this, "fileOpen" );
+    a = new QAction( tr( "Open" ), QPixmap( "textdrawing/fileopen.png" ), tr( "&Open..." ), CTRL + Key_O, this, "fileOpen" );
     connect( a, SIGNAL( activated() ), this, SLOT( fileOpen() ) );
     a->addTo( tb );
     a->addTo( menu );
     menu->insertSeparator();
-    a = new QAction( tr( "Save" ), QPixmap( "textdrawing/filesave.xpm" ), tr( "&Save..." ), CTRL + Key_S, this, "fileSave" );
+    a = new QAction( tr( "Save" ), QPixmap( "textdrawing/filesave.png" ), tr( "&Save..." ), CTRL + Key_S, this, "fileSave" );
     connect( a, SIGNAL( activated() ), this, SLOT( fileSave() ) );
     a->addTo( tb );
     a->addTo( menu );
@@ -66,7 +66,7 @@ void TextEdit::setupFileActions()
     connect( a, SIGNAL( activated() ), this, SLOT( fileSaveAs() ) );
     a->addTo( menu );
     menu->insertSeparator();
-    a = new QAction( tr( "Print" ), QPixmap( "textdrawing/fileprint.xpm" ), tr( "&Print..." ), CTRL + Key_P, this, "filePrint" );
+    a = new QAction( tr( "Print" ), QPixmap( "textdrawing/print.png" ), tr( "&Print..." ), CTRL + Key_P, this, "filePrint" );
     connect( a, SIGNAL( activated() ), this, SLOT( filePrint() ) );
     a->addTo( tb );
     a->addTo( menu );
@@ -82,24 +82,24 @@ void TextEdit::setupEditActions()
     menuBar()->insertItem( tr( "&Edit" ), menu );
 
     QAction *a;
-    a = new QAction( tr( "Undo" ), QPixmap( "textdrawing/editundo.xpm" ), tr( "&Undo" ), CTRL + Key_Z, this, "editUndo" );
+    a = new QAction( tr( "Undo" ), QPixmap( "textdrawing/undo.png" ), tr( "&Undo" ), CTRL + Key_Z, this, "editUndo" );
     connect( a, SIGNAL( activated() ), this, SLOT( editUndo() ) );
     a->addTo( tb );
     a->addTo( menu );
-    a = new QAction( tr( "Redo" ), QPixmap( "textdrawing/editredo.xpm" ), tr( "&Redo" ), CTRL + Key_Y, this, "editRedo" );
+    a = new QAction( tr( "Redo" ), QPixmap( "textdrawing/redo.png" ), tr( "&Redo" ), CTRL + Key_Y, this, "editRedo" );
     connect( a, SIGNAL( activated() ), this, SLOT( editRedo() ) );
     a->addTo( tb );
     a->addTo( menu );
     menu->insertSeparator();
-    a = new QAction( tr( "Cut" ), QPixmap( "textdrawing/editcut.xpm" ), tr( "&Cut" ), CTRL + Key_X, this, "editCut" );
+    a = new QAction( tr( "Cut" ), QPixmap( "textdrawing/editcut.png" ), tr( "&Cut" ), CTRL + Key_X, this, "editCut" );
     connect( a, SIGNAL( activated() ), this, SLOT( editCut() ) );
     a->addTo( tb );
     a->addTo( menu );
-    a = new QAction( tr( "Copy" ), QPixmap( "textdrawing/editcopy.xpm" ), tr( "C&opy" ), CTRL + Key_C, this, "editCopy" );
-    connect( a, SIGNAL( activated() ), this, SLOT( editUndo() ) );
+    a = new QAction( tr( "Copy" ), QPixmap( "textdrawing/editcopy.png" ), tr( "C&opy" ), CTRL + Key_C, this, "editCopy" );
+    connect( a, SIGNAL( activated() ), this, SLOT( editCopy() ) );
     a->addTo( tb );
     a->addTo( menu );
-    a = new QAction( tr( "Paste" ), QPixmap( "textdrawing/editpaste.xpm" ), tr( "&Paste" ), CTRL + Key_V, this, "editPaste" );
+    a = new QAction( tr( "Paste" ), QPixmap( "textdrawing/editpaste.png" ), tr( "&Paste" ), CTRL + Key_V, this, "editPaste" );
     connect( a, SIGNAL( activated() ), this, SLOT( editPaste() ) );
     a->addTo( tb );
     a->addTo( menu );
@@ -138,17 +138,17 @@ void TextEdit::setupTextActions()
 	     this, SLOT( textSize( const QString & ) ) );
     comboSize->lineEdit()->setText( QString::number( QApplication::font().pointSize() ) );
 
-    actionTextBold = new QAction( tr( "Bold" ), QPixmap( "textdrawing/textbold.xpm" ), tr( "&Bold" ), CTRL + Key_B, this, "textBold" );
+    actionTextBold = new QAction( tr( "Bold" ), QPixmap( "textdrawing/textbold.png" ), tr( "&Bold" ), CTRL + Key_B, this, "textBold" );
     connect( actionTextBold, SIGNAL( activated() ), this, SLOT( textBold() ) );
     actionTextBold->addTo( tb );
     actionTextBold->addTo( menu );
     actionTextBold->setToggleAction( TRUE );
-    actionTextItalic = new QAction( tr( "Italic" ), QPixmap( "textdrawing/textitalic.xpm" ), tr( "&Italic" ), CTRL + Key_I, this, "textItalic" );
+    actionTextItalic = new QAction( tr( "Italic" ), QPixmap( "textdrawing/textitalic.png" ), tr( "&Italic" ), CTRL + Key_I, this, "textItalic" );
     connect( actionTextItalic, SIGNAL( activated() ), this, SLOT( textItalic() ) );
     actionTextItalic->addTo( tb );
     actionTextItalic->addTo( menu );
     actionTextItalic->setToggleAction( TRUE );
-    actionTextUnderline = new QAction( tr( "Underline" ), QPixmap( "textdrawing/textunder.xpm" ), tr( "&Underline" ), CTRL + Key_U, this, "textUnderline" );
+    actionTextUnderline = new QAction( tr( "Underline" ), QPixmap( "textdrawing/textunderline.png" ), tr( "&Underline" ), CTRL + Key_U, this, "textUnderline" );
     connect( actionTextUnderline, SIGNAL( activated() ), this, SLOT( textUnderline() ) );
     actionTextUnderline->addTo( tb );
     actionTextUnderline->addTo( menu );
@@ -159,19 +159,19 @@ void TextEdit::setupTextActions()
     grp->setExclusive( TRUE );
     connect( grp, SIGNAL( selected( QAction* ) ), this, SLOT( textAlign( QAction* ) ) );
 
-    actionAlignLeft = new QAction( tr( "Left" ), QPixmap( "textdrawing/textleft.xpm" ), tr( "&Left" ), CTRL + Key_L, grp, "textLeft" );
+    actionAlignLeft = new QAction( tr( "Left" ), QPixmap( "textdrawing/textleft.png" ), tr( "&Left" ), CTRL + Key_L, grp, "textLeft" );
     actionAlignLeft->addTo( tb );
     actionAlignLeft->addTo( menu );
     actionAlignLeft->setToggleAction( TRUE );
-    actionAlignCenter = new QAction( tr( "Center" ), QPixmap( "textdrawing/textcenter.xpm" ), tr( "C&enter" ), CTRL + Key_M, grp, "textCenter" );
+    actionAlignCenter = new QAction( tr( "Center" ), QPixmap( "textdrawing/textcenter.png" ), tr( "C&enter" ), CTRL + Key_M, grp, "textCenter" );
     actionAlignCenter->addTo( tb );
     actionAlignCenter->addTo( menu );
     actionAlignCenter->setToggleAction( TRUE );
-    actionAlignRight = new QAction( tr( "Right" ), QPixmap( "textdrawing/textright.xpm" ), tr( "&Right" ), CTRL + Key_R, grp, "textRight" );
+    actionAlignRight = new QAction( tr( "Right" ), QPixmap( "textdrawing/textright.png" ), tr( "&Right" ), CTRL + Key_R, grp, "textRight" );
     actionAlignRight->addTo( tb );
     actionAlignRight->addTo( menu );
     actionAlignRight->setToggleAction( TRUE );
-    actionAlignJustify = new QAction( tr( "Justify" ), QPixmap( "textdrawing/textjustify.xpm" ), tr( "&Justify" ), CTRL + Key_J, grp, "textjustify" );
+    actionAlignJustify = new QAction( tr( "Justify" ), QPixmap( "textdrawing/textjustify.png" ), tr( "&Justify" ), CTRL + Key_J, grp, "textjustify" );
     actionAlignJustify->addTo( tb );
     actionAlignJustify->addTo( menu );
     actionAlignJustify->setToggleAction( TRUE );
@@ -422,7 +422,7 @@ void TextEdit::textColor()
 	return;
     currentEditor()->setColor( col );
     QPixmap pix( 16, 16 );
-    pix.fill( black );
+    pix.fill( col );
     actionTextColor->setIconSet( pix );
 }
 
