@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id: qt/main.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/main.cpp   3.3.7   edited Aug 31 2005 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -46,11 +46,11 @@ public:
 		    const QString& mid2=QString::null,
 		    const QString& mid3=QString::null)
     {
-	addButton(ok.isNull() ? tr("OK") : ok, 1);
+	addButton(ok.isNull() ? QObject::tr("OK") : ok, 1);
 	if ( !mid1.isNull() ) addButton(mid1,2);
 	if ( !mid2.isNull() ) addButton(mid2,3);
 	if ( !mid3.isNull() ) addButton(mid3,4);
-	addButton(cancel.isNull() ? tr("Cancel") : cancel, 0);
+	addButton(cancel.isNull() ? QObject::tr("Cancel") : cancel, 0);
     }
 
     void addButton( const QString& text, int result )
@@ -100,7 +100,7 @@ int main( int argc, char** argv )
     QApplication app( argc, argv );
 
     const char* qm[]=
-	{ "cs", "de", "el", "en", "eo", "fr", "it", "jp", "ko", "no", "ru", "zh", 0 };
+	{ "ar", "cs", "de", "el", "en", "eo", "fr", "it", "jp", "ko", "no", "ru", "zh", 0 };
 
 #if defined(Q_OS_UNIX)
     srand( getpid() << 2 );
