@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id:  qt/main.cpp   3.0.5   edited Nov 6 2001 $
+** $Id: qt/main.cpp   3.3.7   edited Aug 31 2005 $
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -9,25 +9,16 @@
 *****************************************************************************/
 
 #include <qapplication.h>
-#include <qnetwork.h>
-#include <qsplitter.h>
-
 #include "ftpmainwindow.h"
 
 int main( int argc, char **argv )
 {
     QApplication a( argc, argv );
 
-    // call this to register the built-in network protocols, e.g. FTP
-    // and HTTP.
-    qInitNetworkProtocols();
-
     FtpMainWindow m;
     a.setMainWidget( &m );
-    QValueList<int> sizes;
-    sizes << 300 << 70 << 300;
-    m.mainSplitter()->setSizes( sizes );
-    m.resize( 800, 600 );
     m.show();
+    a.processEvents();
+    m.connectToHost();
     return a.exec();
 }
