@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id:  qt/qtetrixb.cpp   3.0.5   edited Mar 2 1999 $
+** $Id: qt/qtetrixb.cpp   3.3.7   edited Aug 31 2005 $
 **
-** Copyright (C) 1992-1998 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -211,20 +211,7 @@ void QTetrixBoard::keyPressEvent( QKeyEvent *e )
 {
     if ( noGame || isPaused || waitingAfterLine )
         return;
-    int k = e->key();
-#ifdef Q_OS_QNX4
-    if(k == Key_T)
-	k = Key_Left;
-    else if(k == Key_S)
-	k = Key_Right;
-    else if(k == Key_D)
-	k = Key_Space;
-    else if(k == Key_P)
-        k = Key_Up;
-    else if(k == Key_I)
-	k = Key_Down;
-#endif	
-    switch( k ) {
+    switch( e->key() ) {
 	case Key_Left :
 	    moveLeft();
 	    break;
