@@ -1,7 +1,7 @@
 /****************************************************************************
-** $Id: qt/widgets.cpp   3.2.3   edited May 26 16:16 $
+** $Id: qt/widgets.cpp   3.3.7   edited Aug 31 2005 $
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of an example program for Qt.  This example
 ** program may be used, distributed and modified without limitation.
@@ -182,8 +182,7 @@ WidgetView::WidgetView( QWidget *parent, const char *name )
     id = popup->insertItem( openIcon, "&Open...", this, SLOT( open() ) );
 
     popup->insertSeparator();
-    popup->insertItem( "&Quit", qApp, SLOT(quit()), CTRL+Key_Q );
-
+    popup->insertItem( "Quit", qApp, SLOT(quit()), CTRL+Key_Q );
 
     textStylePopup = popup = new QPopupMenu( this );
     menuBar()->insertItem( "&Edit", popup );
