@@ -1,3 +1,3 @@
-#error "/tmp/qt-3.0-reggie-28534/qt/extensions/nsplugin/src/jritypes.h must be provided by Netscape"
+#error "/home/harald/tmp/qt-3.3.7-harald-4508/qt/extensions/nsplugin/src/jritypes.h must be provided by Netscape"
 #error "This file is just a placeholder. Please see the documentation"
 #error "to learn how to obtain the real file"
