@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qapplication.h   3.0.3   edited Dec 7 10:36 $
+** $Id: qt/qapplication.h   3.3.7   edited Aug 31 2005 $
 **
 ** Definition of QApplication class
 **
 ** Created : 931107
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -47,32 +47,28 @@
 #include "qstringlist.h"
 #endif // QT_H
 
-#ifndef QT_NO_REMOTE
-#include "quuid.h"
-#endif //QT_NO_REMOTE
-
 class QSessionManager;
 class QStyle;
 class QTranslator;
+class QEventLoop;
 #if defined(Q_WS_QWS)
 class QWSDecoration;
 #endif
 
-template <class type> class QPtrList;
+#ifdef QT_THREAD_SUPPORT
+class QMutex;
+#endif // QT_THREAD_SUPPORT
+
 
 class QApplication;
 extern Q_EXPORT QApplication *qApp;		// global application object
 
-#if defined(QT_THREAD_SUPPORT)
-class QMutex;
-#endif
-
 
 class Q_EXPORT QApplication : public QObject
 {
     Q_OBJECT
 public:
-    QApplication( int &argc, char **argv);
+    QApplication( int &argc, char **argv );
     QApplication( int &argc, char **argv, bool GUIenabled );
     enum Type { Tty, GuiClient, GuiServer };
     QApplication( int &argc, char **argv, Type );
@@ -139,6 +135,8 @@ public:
     static QWidget  *widgetAt( int x, int y, bool child=FALSE );
     static QWidget  *widgetAt( const QPoint &, bool child=FALSE );
 
+    static QEventLoop *eventLoop();
+
     int		     exec();
     void	     processEvents();
     void	     processEvents( int maxtime );
@@ -168,8 +166,10 @@ public:
     static void	     beep();
 
 #ifndef QT_NO_TRANSLATION
+# ifndef QT_NO_TEXTCODEC
     void	     setDefaultCodec( QTextCodec * );
     QTextCodec*	     defaultCodec() const;
+# endif
     void	     installTranslator( QTranslator * );
     void	     removeTranslator( QTranslator * );
 #endif
@@ -178,6 +178,10 @@ public:
 				const char * key,
 				const char * comment = 0,
 				Encoding encoding = DefaultCodec ) const;
+#ifndef QT_NO_DIR
+    QString   applicationDirPath();
+    QString   applicationFilePath();
+#endif
 #ifndef QT_NO_PALETTE
     // obsolete functions
     static void      setWinStyleHighlightColor( const QColor &c ) {
@@ -205,11 +209,11 @@ public:
     static QSize     globalStrut();
 
 #ifndef QT_NO_COMPONENT
-    static void      setLibraryPaths(const QStringList &);
+    static void      setLibraryPaths( const QStringList & );
     static QStringList libraryPaths();
-    static void      addLibraryPath(const QString &);
-    static void      removeLibraryPath(const QString &);
-#endif //QT_NO_COMPONENT
+    static void      addLibraryPath( const QString & );
+    static void      removeLibraryPath( const QString & );
+#endif // QT_NO_COMPONENT
     static void setStartDragTime( int ms );
     static int startDragTime();
     static void setStartDragDistance( int l );
@@ -224,7 +228,7 @@ public:
     static void	    setEffectEnabled( Qt::UIEffect, bool enable = TRUE );
 
 #if defined(Q_WS_MAC)
-    virtual bool     macEventFilter( EventRef );
+    virtual bool     macEventFilter( EventHandlerCallRef, EventRef );
 #endif
 #if defined(Q_WS_WIN)
     virtual bool     winEventFilter( MSG * );
@@ -238,14 +242,23 @@ public:
     virtual bool     qwsEventFilter( QWSEvent * );
     int              qwsProcessEvent( QWSEvent* );
     void             qwsSetCustomColors( QRgb *colortable, int start, int numColors );
+/*!
+  \internal
+    Returns true if the process is GUI server
+*/
+    bool          qwsIsGUIServer();
 #ifndef QT_NO_QWS_MANAGER
     static QWSDecoration &qwsDecoration();
     static void      qwsSetDecoration( QWSDecoration *);
 #endif
 #endif
 
-#if defined(Q_WS_WIN)
+#if defined(Q_OS_WIN32) || defined(Q_OS_CYGWIN)
     static WindowsVersion winVersion();
+#elif defined(Q_OS_MAC)
+    static MacintoshVersion macVersion();
+#endif
+#if defined(Q_WS_WIN)
     void	     winFocus( QWidget *, bool );
     static void	     winMouseButtonUp();
 #endif
@@ -253,7 +266,8 @@ public:
 #ifndef QT_NO_SESSIONMANAGER
     // session management
     bool	     isSessionRestored() const;
-    QString	     sessionId() const;
+    QString 	sessionId() const;
+    QString 	sessionKey() const;
     virtual void     commitData( QSessionManager& sm );
     virtual void     saveState( QSessionManager& sm );
 #endif
@@ -278,34 +292,45 @@ signals:
 public slots:
     void	     quit();
     void	     closeAllWindows();
+    void	     aboutQt();
+
+#if defined(Q_WS_QWS)
+protected:
+    void setArgs(int, char **);
+#endif
+
+protected:
+    bool event(QEvent *);
 
 private:
-    void	     construct( int &argc, char **argv, Type );
-    bool	     processNextEvent( bool );
-    void	     initialize( int, char ** );
-    void	     init_precmdline();
-    void	     process_cmdline( int* argcptr, char ** argv );
-    bool	     internalNotify( QObject *, QEvent * );
+    void construct( int &argc, char **argv, Type );
+    void initialize( int, char ** );
+    void init_precmdline();
+    void process_cmdline( int* argcptr, char ** argv );
+    bool internalNotify( QObject *, QEvent * );
 #if defined(Q_WS_QWS)
     static QWidget *findChildWidget( const QWidget *p, const QPoint &pos );
     static QWidget *findWidget( const QObjectList&, const QPoint &, bool rec );
 #endif
 
 #if defined(Q_WS_MAC)
-    bool	     do_mouse_down(Point *);
+    bool do_mouse_down(Point *, bool *);
     static QMAC_PASCAL OSStatus globalEventProcessor(EventHandlerCallRef,  EventRef, void *);
+    static QMAC_PASCAL OSStatus globalAppleEventProcessor(const AppleEvent *, AppleEvent *, long);
     static QMAC_PASCAL void qt_context_timer_callbk(EventLoopTimerRef, void *);
     static QMAC_PASCAL void qt_select_timer_callbk(EventLoopTimerRef, void *);
     static bool qt_mac_apply_settings();
     friend class QMacInputMethod;
+    friend QMAC_PASCAL OSStatus qt_window_event(EventHandlerCallRef, EventRef, void *);
+    friend void qt_mac_update_os_settings();
     friend bool qt_set_socket_handler( int, int, QObject *, bool);
     friend void qt_mac_destroy_widget(QWidget *);
     friend void qt_init(int *, char **, QApplication::Type);
 #endif
 
-#if defined(QT_THREAD_SUPPORT)
-    static QMutex * qt_mutex;
-#endif
+#ifdef QT_THREAD_SUPPORT
+    static QMutex   *qt_mutex;
+#endif // QT_THREAD_SUPPORT
 
     int		     app_argc;
     char	   **app_argv;
@@ -320,6 +345,7 @@ private:
 #ifndef QT_NO_CURSOR
     static QCursor  *app_cursor;
 #endif
+    static QEventLoop* eventloop;
     static int	     app_tracking;
     static bool	     is_app_running;
     static bool	     is_app_closing;
@@ -332,6 +358,7 @@ private:
     static int	     cursor_flash_time;
     static int	     mouse_double_click_time;
     static int	     wheel_scroll_lines;
+    static int	     composedUnicode; // Value, meta-composed character
 
     static bool	     animate_ui;
     static bool	     animate_menu;
@@ -339,11 +366,15 @@ private:
     static bool	     animate_combo;
     static bool	     fade_menu;
     static bool	     fade_tooltip;
+    static bool	     animate_toolbox;
+    static bool	     widgetCount; // Coupled with -widgetcount switch
+    static bool	     metaComposeUnicode;
 
-    QPtrList<QTranslator> *translators;
+    QValueList<QTranslator*> *translators;
 #ifndef QT_NO_SESSIONMANAGER
     QSessionManager *session_manager;
     QString	     session_id;
+    static QString* session_key;
     bool	     is_session_restored;
 #endif
 #if defined(Q_WS_X11) && !defined (QT_NO_STYLE )
@@ -351,9 +382,9 @@ private:
 #endif
 
     static QSize     app_strut;
-
+#ifndef QT_NO_COMPONENT
     static QStringList *app_libpaths;
-
+#endif
     static QAsciiDict<QPalette> *app_palettes;
     static QAsciiDict<QFont>    *app_fonts;
 
@@ -368,19 +399,18 @@ private:
 
     friend class QWidget;
     friend class QETWidget;
+    friend class QDialog;
+    friend class QAccelManager;
     friend class QEvent;
+    friend class QTranslator;
+    friend class QEventLoop;
     friend Q_EXPORT void qt_ucm_initialize( QApplication * );
 #if defined(Q_WS_WIN)
     friend bool qt_sendSpontaneousEvent( QObject*, QEvent* );
 #endif
-
-#ifndef QT_NO_REMOTE
-public:
-    void setEnableRemoteControl(bool enable, const QUuid appId = QUuid());
-    bool remoteControlEnabled() const;
-    QUuid applicationId() const;
-#endif //QT_NO_REMOTE
-
+#if defined(Q_WS_QWS)
+    friend class QInputContext;
+#endif
 private: // Disabled copy constructor and operator=
 #if defined(Q_DISABLE_COPY)
     QApplication( const QApplication & );
@@ -397,6 +427,15 @@ inline char **QApplication::argv() const
 {
     return app_argv;
 }
+
+#if defined(Q_WS_QWS)
+inline void QApplication::setArgs(int c, char **v)
+{
+        app_argc = c;
+	    app_argv = v;
+}
+#endif
+
 #ifndef QT_NO_CURSOR
 inline QCursor *QApplication::overrideCursor()
 {
@@ -442,6 +481,11 @@ inline QString QApplication::sessionId() const
 {
     return session_id;
 }
+
+inline QString QApplication::sessionKey() const
+{
+    return session_key ? *session_key : QString::null;
+}
 #endif
 inline QSize QApplication::globalStrut()
 {
@@ -449,7 +493,7 @@ inline QSize QApplication::globalStrut()
 }
 
 inline bool QApplication::sendEvent( QObject *receiver, QEvent *event )
-{ return qApp ? qApp->notify( receiver, event ) : FALSE; }
+{  if ( event ) event->spont = FALSE; return qApp ? qApp->notify( receiver, event ) : FALSE; }
 
 inline bool QApplication::sendSpontaneousEvent( QObject *receiver, QEvent *event )
 { if ( event ) event->spont = TRUE; return qApp ? qApp->notify( receiver, event ) : FALSE; }
