@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qasyncimageio.h   3.2.1   edited May 13 09:08 $
+** $Id: qt/qasyncimageio.h   3.3.7   edited Jan 30 2006 $
 **
 ** Definition of asynchronous image/movie loading classes
 **
 ** Created : 970617
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -44,6 +44,9 @@
 
 #ifndef QT_NO_ASYNC_IMAGE_IO
 
+#if __GNUC__ - 0 > 3
+#pragma GCC system_header
+#endif
 
 class Q_EXPORT QImageConsumer {
 public:
