@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qcolor.h   3.2.3   edited May 13 09:08 $
+** $Id: qt/qcolor.h   3.3.7   edited Aug 31 2005 $
 **
 ** Definition of QColor class
 **
 ** Created : 940112
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -177,7 +177,11 @@ inline QColor::QColor()
 { d.d32.argb = Invalid; d.d32.pix = Dirt; }
 
 inline QColor::QColor( int r, int g, int b )
-{ setRgb( r, g, b ); }
+{
+    d.d32.argb = Invalid;
+    d.d32.pix = Dirt;
+    setRgb( r, g, b );
+}
 
 inline QRgb QColor::rgb() const
 { return d.argb; }
