@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qevent.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qevent.h   3.3.7   edited Aug 31 2005 $
 **
 ** Definition of event classes
 **
 ** Created : 931029
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -51,10 +51,12 @@ class Q_EXPORT QEvent: public Qt		// event base class
 public:
     enum Type {
 
-	// NOTE: if you get a strange compiler error on the line with "None",
-	//       it's probably because you're trying to include X11, which
-	//	 has a mess of #defines in it.  Put the messy X11 includes
-	//	 *AFTER* the nice clean Qt includes.
+	/*
+	  If you get a strange compiler error on the line with None,
+	  it's probably because you're also including X11 headers,
+	  which #define the symbol None. Put the X11 includes after
+	  the Qt includes to solve this problem.
+	*/
 
 	None = 0,				// invalid event
 
@@ -62,7 +64,7 @@ public:
 	Timer = 1,				// timer event
 	MouseButtonPress = 2,			// mouse button pressed
 	MouseButtonRelease = 3,			// mouse button released
-	MouseButtonDblClick= 4,			// mouse button double click
+	MouseButtonDblClick = 4,		// mouse button double click
 	MouseMove = 5,				// mouse move
 	KeyPress = 6,				// key pressed
 	KeyRelease = 7,				// key released
@@ -112,16 +114,28 @@ public:
 	ChildRemoved = 71,			// deleted child widget
 	LayoutHint = 72,			// child min/max size changed
 	ShowWindowRequest = 73,			// widget's window should be mapped
+	WindowBlocked = 74,			// window is about to be blocked modally
+	WindowUnblocked = 75,			// windows modal blocking has ended
 	ActivateControl = 80,			// ActiveX activation
 	DeactivateControl = 81,			// ActiveX deactivation
-	ContextMenu = 82,                       // context popup menu
+	ContextMenu = 82,			// context popup menu
 	IMStart = 83,				// input method composition start
 	IMCompose = 84,				// input method composition
 	IMEnd = 85,				// input method composition end
 	Accessibility = 86,			// accessibility information is requested
-	Tablet = 87,				// Wacom Tablet Event
+	TabletMove = 87,			// Wacom tablet event
+	LocaleChange = 88,			// the system locale changed
+	LanguageChange = 89,			// the application language changed
+	LayoutDirectionChange = 90,		// the layout direction changed
+	Style = 91,				// internal style event
+	TabletPress = 92,			// tablet press
+	TabletRelease = 93,			// tablet release
+	OkRequest = 94,				// CE (Ok) button pressed
+	HelpRequest = 95,			// CE (?)  button pressed
+	WindowStateChange = 96,			// window state has changed
+	IconDrag = 97,                          // proxy icon dragged
 	User = 1000,				// first user event id
-	MaxUser  = 65535                        // last user event id
+	MaxUser = 65535				// last user event id
     };
 
 
@@ -137,6 +151,7 @@ private:
 
 
     friend class QApplication;
+    friend class QAccelManager;
     friend class QBaseApplication;
     friend class QETWidget;
 };
@@ -217,16 +232,13 @@ class Q_EXPORT QTabletEvent : public QEvent
 {
 public:
     enum TabletDevice { NoDevice = -1, Puck, Stylus, Eraser };
-    QTabletEvent( const QPoint &pos, int device, int pressure, int xTilt, 
-                  int yTilt, const QPair<int,int> &uId )
-	: QEvent( Tablet ), mPos( pos ), mDev( device ), mPress( pressure ),
-	  mXT( xTilt ), mYT( yTilt ), mType( uId.first ), mPhy( uId.second )
-    {}
+    QTabletEvent( Type t, const QPoint &pos, const QPoint &globalPos, int device,
+		  int pressure, int xTilt, int yTilt, const QPair<int,int> &uId );
     QTabletEvent( const QPoint &pos, const QPoint &globalPos, int device,
 		  int pressure, int xTilt, int yTilt, const QPair<int,int> &uId )
-	: QEvent( Tablet ), mPos( pos ), mGPos( globalPos ), mDev( device ),
+	: QEvent( TabletMove ), mPos( pos ), mGPos( globalPos ), mDev( device ),
 	  mPress( pressure ), mXT( xTilt ), mYT( yTilt ), mType( uId.first ),
-	  mPhy( uId.second )
+	  mPhy( uId.second ), mbAcc(TRUE)
     {}
     int pressure()	const { return mPress; }
     int xTilt()		const { return mXT; }
@@ -246,13 +258,13 @@ protected:
     QPoint mPos;
     QPoint mGPos;
     int mDev,
-        mPress,
-        mXT,
-        mYT,
-        mType,
+	mPress,
+	mXT,
+	mYT,
+	mType,
 	mPhy;
     bool mbAcc;
-    
+
 };
 
 class Q_EXPORT QKeyEvent : public QEvent
@@ -261,7 +273,11 @@ public:
     QKeyEvent( Type type, int key, int ascii, int state,
 		const QString& text=QString::null, bool autorep=FALSE, ushort count=1 )
 	: QEvent(type), txt(text), k((ushort)key), s((ushort)state),
-	    a((uchar)ascii), accpt(TRUE), autor(autorep), c(count) {}
+	    a((uchar)ascii), accpt(TRUE), autor(autorep), c(count)
+    {
+	if ( key >= Key_Back && key <= Key_MediaLast )
+	    accpt = FALSE;
+    }
     int	   key()	const	{ return k; }
     int	   ascii()	const	{ return a; }
     ButtonState state()	const	{ return ButtonState(s); }
@@ -317,6 +333,12 @@ public:
 	  rec(paintRect),
 	  reg(paintRect),
 	  erase(erased){}
+    QPaintEvent( const QRegion &paintRegion, const QRect &paintRect, bool erased = TRUE )
+	: QEvent(Paint),
+	  rec(paintRect),
+	  reg(paintRegion),
+	  erase(erased){}
+
     const QRect &rect() const	  { return rec; }
     const QRegion &region() const { return reg; }
     bool erased() const { return erase; }
@@ -370,6 +392,19 @@ protected:
 };
 
 
+class Q_EXPORT QIconDragEvent : public QEvent
+{
+public:
+    QIconDragEvent() 
+	: QEvent(IconDrag), accpt(FALSE) {}
+
+    bool   isAccepted() const	{ return accpt; }
+    void   accept()		{ accpt = TRUE; }
+    void   ignore()		{ accpt = FALSE; }
+protected:
+    bool   accpt;
+};
+
 class Q_EXPORT QShowEvent : public QEvent
 {
 public:
@@ -390,7 +425,7 @@ class Q_EXPORT QContextMenuEvent : public QEvent
 public:
     enum Reason { Mouse, Keyboard, Other };
     QContextMenuEvent( Reason reason, const QPoint &pos, const QPoint &globalPos, int state )
-	: QEvent( ContextMenu ), p( pos ), gp( globalPos ), accpt( FALSE ), consum(FALSE),
+	: QEvent( ContextMenu ), p( pos ), gp( globalPos ), accpt( TRUE ), consum( TRUE ),
 	reas( reason ), s((ushort)state) {}
     QContextMenuEvent( Reason reason, const QPoint &pos, int state );
 
@@ -404,8 +439,8 @@ public:
 
     ButtonState state()	const	{ return (ButtonState) s; }
     bool    isAccepted() const	{ return accpt; }
-    bool    isConsumed() const  { return consum; }
-    void    consume()           { consum = TRUE; }
+    bool    isConsumed() const	{ return consum; }
+    void    consume()		{ accpt = FALSE; consum = TRUE; }
     void    accept()		{ accpt = TRUE; consum = TRUE; }
     void    ignore()		{ accpt = FALSE; consum = FALSE; }
 
@@ -425,18 +460,40 @@ class Q_EXPORT QIMEvent : public QEvent
 {
 public:
     QIMEvent( Type type, const QString &text, int cursorPosition )
-	: QEvent(type), txt(text), cpos(cursorPosition), a(FALSE) {}
+	: QEvent(type), txt(text), cpos(cursorPosition), a(TRUE) {}
     const QString &text() const { return txt; }
     int cursorPos() const { return cpos; }
     bool isAccepted() const { return a; }
     void accept() { a = TRUE; }
     void ignore() { a = FALSE; }
+    int selectionLength() const;
+
 private:
     QString txt;
     int cpos;
     bool a;
 };
 
+class Q_EXPORT QIMComposeEvent : public QIMEvent
+{
+public:
+    QIMComposeEvent( Type type, const QString &text, int cursorPosition,
+		     int selLength )
+	: QIMEvent( type, text, cursorPosition ), selLen( selLength ) { }
+
+private:
+    int selLen;
+
+    friend class QIMEvent;
+};
+
+inline int QIMEvent::selectionLength() const
+{
+    if ( type() != IMCompose ) return 0;
+    QIMComposeEvent *that = (QIMComposeEvent *) this;
+    return that->selLen;
+}
+
 
 #ifndef QT_NO_DRAGANDDROP
 
