@@ -1,11 +1,11 @@
 /**********************************************************************
-** $Id:  qt/qlistbox.h   3.0.5   edited Mar 1 17:39 $
+** $Id: qt/qlistbox.h   3.3.7   edited Aug 31 2005 $
 **
 ** Definition of QListBox widget class
 **
 ** Created : 941121
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -130,6 +130,7 @@ public:
     void setSelected( int, bool );
     bool isSelected( int ) const;
     bool isSelected( const QListBoxItem * ) const;
+    QListBoxItem* selectedItem() const;
 
     QSize sizeHint() const;
     QSize	minimumSizeHint() const;
@@ -288,6 +289,7 @@ private:
     int currentRow() const;
     int currentColumn() const;
     void updateSelection();
+    void repaintSelection();
     void drawRubber();
     void doRubberSelection( const QRect &old, const QRect &rubber );
     void handleItemChange( QListBoxItem *old, bool shift, bool control );
