@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qnetworkprotocol.h   3.3.4   edited May 27 2003 $
+** $Id: qt/qnetworkprotocol.h   3.3.7   edited Mar 1 2006 $
 **
 ** Definition of QNetworkProtocol class
 **
 ** Created : 950429
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -47,6 +47,10 @@
 
 #ifndef QT_NO_NETWORKPROTOCOL
 
+#if __GNUC__ - 0 > 3
+#pragma GCC system_header
+#endif
+
 class QNetworkProtocol;
 class QNetworkOperation;
 class QTimer;
@@ -61,12 +65,12 @@ public:
 
 };
 
-template< class Protocol >
+template< class T >
 class QNetworkProtocolFactory : public QNetworkProtocolFactoryBase
 {
 public:
     QNetworkProtocol *createObject() {
-	return new Protocol;
+	return new T;
     }
 
 };
