@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qobjectdefs.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qobjectdefs.h   3.3.7   edited Aug 31 2005 $
 **
 ** Macros and definitions related to QObject
 **
 ** Created : 930419
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -61,7 +61,8 @@
 
 #ifndef QT_NO_PROPERTIES
 # define QT_PROP_FUNCTIONS \
-    virtual bool qt_property( int, int, QVariant* );
+    virtual bool qt_property( int id, int f, QVariant* v); \
+    static bool qt_static_property( QObject* , int, int, QVariant* );
 #else
 # define QT_PROP_FUNCTIONS
 #endif
@@ -161,5 +162,13 @@ class QObjectList;
 class QObjectListIt;
 class QMemberDict;
 
+Q_EXPORT void *qt_find_obj_child( QObject *, const char *, const char * );
+#define Q_CHILD(parent,type,name) \
+	((type*)qt_find_obj_child(parent,#type,name))
 
+Q_EXPORT void *qt_inheritedBy( QMetaObject *super, const QObject *cls );
+
+template <typename T>
+Q_INLINE_TEMPLATES T qt_cast(const QObject *object)
+{ return (T)qt_inheritedBy( ((T)0)->staticMetaObject(), object ); }
 #endif // QOBJECTDEFS_H
