@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qwmatrix.h   3.0.5   edited Oct 12 2001 $
+** $Id: qt/qwmatrix.h   3.3.7   edited Aug 31 2005 $
 **
 ** Definition of QWMatrix class
 **
 ** Created : 941020
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -42,6 +42,7 @@
 #include "qwindowdefs.h"
 #include "qpointarray.h"
 #include "qrect.h"
+#include "qregion.h"
 #endif // QT_H
 
 #ifndef QT_NO_WMATRIX
@@ -71,6 +72,9 @@ public:
     QPoint	map( const QPoint &p )	const { return operator *( p ); }
     QRect	map( const QRect &r )	const { return mapRect ( r ); }
     QPointArray map( const QPointArray &a ) const { return operator * ( a ); }
+    QRegion     map( const QRegion &r ) const { return operator *( r ); }
+    QRegion     mapToRegion( const QRect &r ) const { return operator *( r ); }
+    QPointArray	mapToPolygon( const QRect &r )	const;
 
     void	reset();
     bool	isIdentity() const;
@@ -81,6 +85,8 @@ public:
     QWMatrix   &rotate( double a );
 
     bool isInvertible() const { return (_m11*_m22 - _m12*_m21) != 0; }
+    double det() const { return _m11*_m22 - _m12*_m21; }
+
     QWMatrix	invert( bool * = 0 ) const;
 
     bool	operator==( const QWMatrix & ) const;
@@ -92,15 +98,18 @@ public:
     QRegion operator * (const QRect & ) const;
     QRegion operator * (const QRegion & ) const;
     QPointArray operator *  ( const QPointArray &a ) const;
-    
+
+    enum TransformationMode {
+	Points, Areas
+    };
+    static void setTransformationMode( QWMatrix::TransformationMode m );
+    static TransformationMode transformationMode();
 private:
-    QWMatrix   &bmul( const QWMatrix & );
     double	_m11, _m12;
     double	_m21, _m22;
     double	_dx,  _dy;
 };
 
-
 Q_EXPORT QWMatrix operator*( const QWMatrix &, const QWMatrix & );
 
 
