@@ -1,9 +1,9 @@
 /****************************************************************************
-** $Id: qt/msvc_objectmodel.h   3.2.3   edited May 30 12:57 $
+** $Id: qt/msvc_objectmodel.h   3.3.7   edited Feb 7 2006 $
 **
 ** Definition of VCProject class.
 **
-** Copyright (C) 2002-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 2002-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of qmake.
 **
@@ -57,7 +57,8 @@ enum customBuildCheck {
     none,
     moc,
     uic,
-    lexyacc
+    lexyacc,
+    resource
 };
 enum triState {
     unset = -1,
@@ -115,6 +116,7 @@ enum ConfigurationTypes {
     typeGeneric        = 10
 };
 enum debugOption {
+    debugUnknown = -1,
     debugDisabled,
     debugOldStyleInfo,
     debugLineInfoOnly,
@@ -318,6 +320,7 @@ enum pchOption {
     pchUseUsingSpecific
 };
 enum preprocessOption {
+    preprocessUnknown = -1,
     preprocessNo,
     preprocessYes,
     preprocessNoLineNumbers
@@ -333,6 +336,7 @@ enum RemoteDebuggerType {
     DbgRemoteTCPIP
 };
 enum runtimeLibraryOption {
+    rtUnknown = -1,
     rtMultiThreaded,
     rtMultiThreadedDebug,
     rtMultiThreadedDLL,
@@ -382,6 +386,7 @@ enum useOfMfc {
     useMfcDynamic
 };
 enum warningLevelOption {
+    warningLevelUnknown = -1,
     warningLevel_0,
     warningLevel_1,
     warningLevel_2,
@@ -450,6 +455,7 @@ public:
     triState		    MinimalRebuild;
     QString		    ObjectFile;
     triState		    OmitFramePointers;
+    triState                OpenMP;
     optimizeOption	    Optimization;
     ProcessorOptimizeOption OptimizeForProcessor;
     triState		    OptimizeForWindowsApplication;
@@ -614,9 +620,9 @@ public:
 
     // Variables
     QStringList		    AdditionalDependencies;
-    QString		    CommandLine;
+    QStringList		    CommandLine;
     QString		    Description;
-    QString		    Outputs;
+    QStringList		    Outputs;
     QString		    ToolName;
     QString		    ToolPath;
 };
@@ -722,8 +728,11 @@ public:
     // Functions
     VCFilter();
     ~VCFilter(){}
-    void generateMOC( QTextStream &strm, QString str ) const;
-    void generateUIC( QTextStream &strm, const QString& str ) const;
+    void addMOCstage( QTextStream &strm, QString str );
+    void addUICstage( QTextStream &strm, QString str );
+    bool addIMGstage( QTextStream &strm, QString str );
+    void modifyPCHstage( QTextStream &strm, QString str );
+    bool addLexYaccStage( QTextStream &strm, QString str );
 
     // Variables
     QString		Name;
@@ -731,8 +740,13 @@ public:
     triState		ParseFiles;
     QStringList		Files;
     VcprojGenerator*	Project;
-    VCConfiguration*	Config;
+    QValueList<VCConfiguration> *Config;
     customBuildCheck	CustomBuild;
+    bool		useCustomBuildTool;
+    VCCustomBuildTool   CustomBuildTool;
+    bool		useCompilerTool;
+    VCCLCompilerTool    CompilerTool;
+    bool		flat_files;
 };
 
 class VCProject
@@ -751,7 +765,7 @@ public:
     QString		PlatformName;
 
     // XML sub-parts
-    VCConfiguration	Configuration;
+    QValueList<VCConfiguration> Configuration;
     VCFilter		SourceFiles;
     VCFilter		HeaderFiles;
     VCFilter		MOCFiles;
@@ -770,7 +784,7 @@ QTextStream &operator<<( QTextStream &, const VCLibrarianTool & );
 QTextStream &operator<<( QTextStream &, const VCResourceCompilerTool & );
 QTextStream &operator<<( QTextStream &, const VCEventTool & );
 QTextStream &operator<<( QTextStream &, const VCConfiguration & );
-QTextStream &operator<<( QTextStream &, const VCFilter & );
+QTextStream &operator<<( QTextStream &, VCFilter & );
 QTextStream &operator<<( QTextStream &, const VCProject & );
 
 #endif //__MSVC_OBJECTMODEL_H__
