@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/project.cpp   3.3.5   edited Aug 31 12:13 $
+** $Id: qt/project.cpp   3.3.7   edited Nov 18 2005 $
 **
 ** Implementation of QMakeProject class.
 **
@@ -954,7 +954,7 @@ QMakeProject::doProjectTest(const QString& func, QStringList args, QMap<QString,
 	    QStringList include_roots;
 	    include_roots << Option::output_dir;
 	    QString pfilewd = QFileInfo(parser.file).dirPath();
-	    if(!pfilewd.isEmpty())
+	    if(!pfilewd.isEmpty() && pfilewd != QDir::currentDirPath() && pfilewd != ".")
 		include_roots << pfilewd;
 	    if(Option::output_dir != QDir::currentDirPath())
 		include_roots << QDir::currentDirPath();
