@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qcolordialog.cpp   3.2.3   edited May 24 15:31 $
+** $Id: qt/qcolordialog.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of QColorDialog class
 **
 ** Created : 990222
 **
-** Copyright (C) 1999-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1999-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the dialogs module of the Qt GUI Toolkit.
 **
@@ -559,7 +559,7 @@ void QColorWell::mouseMoveEvent( QMouseEvent *e )
 	return;
     if ( ( pressPos - e->pos() ).manhattanLength() > QApplication::startDragDistance() ) {
 	setCurrent( oldCurrent.x(), oldCurrent.y() );
-	int i = rowAt( e->y() ) + columnAt( e->x() ) * numRows();
+	int i = rowAt(pressPos.y()) + columnAt(pressPos.x()) * numRows();
 	QColor col( values[ i ] );
 	QColorDrag *drg = new QColorDrag( col, this );
 	QPixmap pix( cellWidth(), cellHeight() );
@@ -1078,10 +1078,10 @@ QColorShower::QColorShower( QWidget *parent, const char *name )
     lab = new QColorShowLabel( this );
     lab->setMinimumWidth( 60 ); //###
     gl->addMultiCellWidget(lab, 0,-1,0,0);
-    connect( lab, SIGNAL( colorDropped( QRgb ) ),
-	     this, SIGNAL( newCol( QRgb ) ) );
-    connect( lab, SIGNAL( colorDropped( QRgb ) ),
-	     this, SLOT( setRgb( QRgb ) ) );
+    connect( lab, SIGNAL( colorDropped(QRgb) ),
+	     this, SIGNAL( newCol(QRgb) ) );
+    connect( lab, SIGNAL( colorDropped(QRgb) ),
+	     this, SLOT( setRgb(QRgb) ) );
 
     hEd = new QColNumLineEdit( this, "qt_hue_edit" );
     hEd->setValidator( val360 );
@@ -1279,13 +1279,15 @@ void QColorDialogPrivate::newCustom( int r, int c )
     int i = r+2*c;
     setCurrentColor( cusrgb[i] );
     nextCust = i;
-    standard->setSelected(-1,-1);
+    if (standard)
+        standard->setSelected(-1,-1);
 }
 
 void QColorDialogPrivate::newStandard( int r, int c )
 {
     setCurrentColor( stdrgb[r+c*6] );
-    custom->setSelected(-1,-1);
+    if (custom)
+        custom->setSelected(-1,-1);
 }
 
 QColorDialogPrivate::QColorDialogPrivate( QColorDialog *dialog ) :
@@ -1342,6 +1344,9 @@ QColorDialogPrivate::QColorDialogPrivate( QColorDialog *dialog ) :
 	// better color picker size for small displays
 	pWidth = 150;
 	pHeight = 100;
+
+        custom = 0;
+        standard = 0;
     }
 
     QVBoxLayout *rightLay = new QVBoxLayout( topLay );
@@ -1397,7 +1402,8 @@ QColorDialogPrivate::QColorDialogPrivate( QColorDialog *dialog ) :
 void QColorDialogPrivate::addCustom()
 {
     cusrgb[nextCust] = cs->currentColor();
-    custom->repaintContents( FALSE );
+    if (custom)
+        custom->repaintContents( FALSE );
     nextCust = (nextCust+1) % 16;
 }
 
@@ -1615,28 +1621,32 @@ bool QColorDialog::selectColor( const QColor& col )
     QRgb color = col.rgb();
     int i = 0, j = 0;
     // Check standard colors
-    for ( i = 0; i < 6; i++ ) {
-	for ( j = 0; j < 8; j++ ) {
-	    if ( color == stdrgb[i + j*6] ) {
-		d->newStandard( i, j );
-		d->standard->setCurrent( i, j );
-		d->standard->setSelected( i, j );
-		d->standard->setFocus();
-		return TRUE;
-	    }
-	}
+    if (d->standard) {
+        for ( i = 0; i < 6; i++ ) {
+            for ( j = 0; j < 8; j++ ) {
+                if ( color == stdrgb[i + j*6] ) {
+                    d->newStandard( i, j );
+                    d->standard->setCurrent( i, j );
+                    d->standard->setSelected( i, j );
+                    d->standard->setFocus();
+                    return TRUE;
+                }
+            }
+        }
     }
     // Check custom colors
-    for ( i = 0; i < 2; i++ ) {
-	for ( j = 0; j < 8; j++ ) {
-	    if ( color == cusrgb[i + j*2] ) {
-		d->newCustom( i, j );
-		d->custom->setCurrent( i, j );
-		d->custom->setSelected( i, j );
-		d->custom->setFocus();
-		return TRUE;
-	    }
-	}
+    if (d->custom) {
+        for ( i = 0; i < 2; i++ ) {
+            for ( j = 0; j < 8; j++ ) {
+                if ( color == cusrgb[i + j*2] ) {
+                    d->newCustom( i, j );
+                    d->custom->setCurrent( i, j );
+                    d->custom->setSelected( i, j );
+                    d->custom->setFocus();
+                    return TRUE;
+                }
+            }
+        }
     }
     return FALSE;
 }
