@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qwizard.cpp   3.2.3   edited May 24 15:31 $
+** $Id: qt/qwizard.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of QWizard class.
 **
 ** Created : 990124
 **
-** Copyright (C) 1999-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1999-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the dialogs module of the Qt GUI Toolkit.
 **
@@ -725,8 +725,8 @@ QPushButton * QWizard::helpButton() const
 
 
 /*!
-    This virtual function is responsible for adding the bottom divider
-    and the buttons below it.
+    This virtual function is responsible for adding the buttons below
+    the bottom divider.
 
     \a layout is the horizontal layout of the entire wizard.
 */
@@ -763,6 +763,9 @@ void QWizard::layOutButtonRow( QHBoxLayout * layout )
 
     h->addSpacing( 6 );
 
+    if (d->current == d->pages.at( d->pages.count()-1 ))
+        hasEarlyFinish = FALSE;
+
     if ( hasEarlyFinish ) {
 	d->nextButton->show();
 	d->finishButton->show();
@@ -796,9 +799,9 @@ void QWizard::layOutButtonRow( QHBoxLayout * layout )
 
 
 /*!
-    This virtual function is responsible for laying out the title row
-    and adding the horizontal divider between the title and the wizard
-    page. \a layout is the horizontal layout for the wizard, and \a
+    This virtual function is responsible for laying out the title row.
+
+    \a layout is the horizontal layout for the wizard, and \a
     title is the title for this page. This function is called every
     time \a title changes.
 */
