@@ -1,12 +1,12 @@
 /****************************************************************************
-** $Id: qt/qcolor_p.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qcolor_p.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Named color support for non-X platforms.
 ** The color names have been borrowed from X.
 **
 ** Created : 000228
 **
-** Copyright (C) 2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -759,7 +759,7 @@ bool qt_get_named_rgb( const char *name, QRgb* rgb )
 
 uint qt_get_rgb_val( const char *name )
 {
-    QRgb r;
+    QRgb r = 0;
     qt_get_named_rgb(name,&r);
     return r;
 }
