@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qdnd_x11.cpp   3.2.3   edited Sep 22 17:17 $
+** $Id: qt/qdnd_x11.cpp   3.3.7   edited Jan 30 2006 $
 **
 ** XDND implementation for Qt.  See http://www.cco.caltech.edu/~jafl/xdnd/
 **
 ** Created : 980320
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -79,6 +79,7 @@ extern void qt_leave_modal( QWidget *widget );
 extern Window qt_x11_findClientWindow( Window, Atom, bool );
 extern Atom qt_wm_state;
 extern Time qt_x_time;
+extern Time qt_x_user_time;
 
 // this stuff is copied from qclb_x11.cpp
 
@@ -433,6 +434,7 @@ void qt_xdnd_cleanup()
     qt_xdnd_atom_numbers = 0;
     delete qt_xdnd_target_data;
     qt_xdnd_target_data = 0;
+    delete noDropCursor;
     noDropCursor = 0;
     delete copyCursor;
     copyCursor = 0;
@@ -616,18 +618,23 @@ void qt_handle_xdnd_position( QWidget *w, const XEvent * xe, bool passive )
 		//NOTUSED qt_xdnd_target_current_time = l[3]; // will be 0 for xdnd1
 
 		QDragEnterEvent de( p );
+		de.setAction(accepted_action);
 		QApplication::sendEvent( c, &de );
 		if ( de.isAccepted() ) {
 		    me.accept( de.answerRect() );
-		    if ( !me.isActionAccepted() ) // only as a copy (move if we del)
+		    if ( !de.isActionAccepted() ) // only as a copy (move if we del)
 			accepted_action = QDropEvent::Copy;
+		    else
+			me.acceptAction(TRUE);
 		} else {
 		    me.ignore( de.answerRect() );
 		}
 	    }
 	} else {
-	    if ( qt_xdnd_target_answerwas )
+	    if ( qt_xdnd_target_answerwas ) {
 		me.accept();
+		me.acceptAction(global_requested_action == global_accepted_action);
+	    }
 	}
 
 	if ( !c->acceptDrops() ) {
@@ -676,17 +683,14 @@ void qt_handle_xdnd_position( QWidget *w, const XEvent * xe, bool passive )
 
     QWidget * source = QWidget::find( qt_xdnd_dragsource_xid );
 
-    int emask = NoEventMask;
-    if ( source && source->isDesktop() && !source->acceptDrops() ) {
-	emask = EnterWindowMask;
+    if ( source && source->isDesktop() && !source->acceptDrops() )
 	source = 0;
-    }
 
     if ( source )
 	qt_handle_xdnd_status( source, (const XEvent *)&response, passive );
     else
 	XSendEvent( QPaintDevice::x11AppDisplay(), qt_xdnd_dragsource_xid, False,
-		    emask, (XEvent*)&response );
+		    NoEventMask, (XEvent*)&response );
 }
 
 
@@ -762,17 +766,14 @@ void qt_xdnd_send_leave()
 
     QWidget * w = QWidget::find( qt_xdnd_current_proxy_target );
 
-    int emask = NoEventMask;
-    if ( w && w->isDesktop() && !w->acceptDrops() ) {
-	emask = EnterWindowMask;
+    if ( w && w->isDesktop() && !w->acceptDrops() )
 	w = 0;
-    }
 
     if ( w )
 	qt_handle_xdnd_leave( w, (const XEvent *)&leave, FALSE );
     else
 	XSendEvent( QPaintDevice::x11AppDisplay(), qt_xdnd_current_proxy_target, False,
-		    emask, (XEvent*)&leave );
+		    NoEventMask, (XEvent*)&leave );
     qt_xdnd_current_target = 0;
     qt_xdnd_current_proxy_target = 0;
 }
@@ -801,6 +802,12 @@ void qt_handle_xdnd_drop( QWidget *, const XEvent * xe, bool passive )
 	//       l[0], qt_xdnd_dragsource_xid );
 	return;
     }
+
+    if (l[2] != 0) {
+        // update the "user time" from the timestamp in the event.
+        qt_x_user_time = l[2];
+    }
+
     if ( qt_xdnd_source_object )
 	qt_xdnd_source_object->setTarget( qt_xdnd_current_widget );
 
@@ -809,8 +816,9 @@ void qt_handle_xdnd_drop( QWidget *, const XEvent * xe, bool passive )
 	de.setAction( global_accepted_action );
 	QApplication::sendEvent( qt_xdnd_current_widget, &de );
 	if ( !de.isAccepted() ) {
-	    // Ignore a failed move
+	    // Ignore a failed drag
 	    global_accepted_action = QDropEvent::Copy;
+	    dndCancelled = TRUE;
 	}
 	XClientMessageEvent finished;
 	finished.type = ClientMessage;
@@ -936,6 +944,7 @@ bool QDragManager::eventFilter( QObject * o, QEvent * e)
       case QEvent::Wheel:
       case QEvent::Accel:
       case QEvent::AccelAvailable:
+      case QEvent::AccelOverride:
 	return TRUE;
       default:
 	return FALSE;
@@ -974,7 +983,7 @@ void QDragManager::updateMode( ButtonState newstate )
 }
 
 
-void QDragManager::updateCursor()
+void QDragManager::createCursors()
 {
     if ( !noDropCursor ) {
 	noDropCursor = new QCursor( ForbiddenCursor );
@@ -985,7 +994,10 @@ void QDragManager::updateCursor()
 	if ( !pm_cursor[2].isNull() )
 	    linkCursor = new QCursor(pm_cursor[2], 0,0);
     }
+}
 
+void QDragManager::updateCursor()
+{
     QCursor *c;
     if ( willDrop ) {
 	if ( global_accepted_action == QDropEvent::Copy ) {
@@ -1050,20 +1062,23 @@ Window findRealWindow( const QPoint & pos, Window w, int md )
 	return 0;
 
     if ( md ) {
+        qt_ignore_badwindow();
 	XWindowAttributes attr;
 	XGetWindowAttributes( QPaintDevice::x11AppDisplay(), w, &attr );
+        if (qt_badwindow())
+            return 0;
 
-	if ( attr.map_state != IsUnmapped
+	if ( attr.map_state == IsViewable
 	    && QRect(attr.x,attr.y,attr.width,attr.height)
-		.contains(pos) )
-	{
+             .contains(pos) )
+        {
 	    {
 		Atom   type = None;
 		int f;
 		unsigned long n, a;
 		unsigned char *data;
 
-		XGetWindowProperty( QPaintDevice::x11AppDisplay(), w, qt_wm_state, 0,
+		XGetWindowProperty( QPaintDevice::x11AppDisplay(), w, qt_xdnd_aware, 0,
 		    0, False, AnyPropertyType, &type, &f,&n,&a,&data );
 		if ( data ) XFree(data);
 		if ( type ) return w;
@@ -1095,13 +1110,21 @@ Window findRealWindow( const QPoint & pos, Window w, int md )
 
 void QDragManager::move( const QPoint & globalPos )
 {
-    Q_ASSERT( object != 0 ); // ### remove in Qt 4.0 (object should never be 0 here)
+    if (!object) {
+        // perhaps the target crashed?
+        return;
+    }
+
     int screen = QCursor::x11Screen();
     if ( ( qt_xdnd_current_screen == -1 && screen != QPaintDevice::x11AppScreen() ) ||
 	 ( screen != qt_xdnd_current_screen ) ) {
 	// recreate the pixmap on the new screen...
 	delete qt_xdnd_deco;
 	qt_xdnd_deco = new QShapedPixmapWidget( screen );
+	if (!QWidget::mouseGrabber()) {
+	    updatePixmap();
+	    qt_xdnd_deco->grabMouse();
+	}
     }
     updatePixmap();
 
@@ -1124,22 +1147,49 @@ void QDragManager::move( const QPoint & globalPos )
 	// Ok.
     } else if ( target ) {
 	//me
-	Window targetW = qt_x11_findClientWindow( target, qt_wm_state, TRUE );
-	if (targetW)
-	    target = targetW;
+        Window src = rootwin;
+        while (target != 0) {
+            int lx2, ly2;
+            Window t;
+            // translate coordinates
+            if (!XTranslateCoordinates(QPaintDevice::x11AppDisplay(), src, target,
+                                       lx, ly, &lx2, &ly2, &t)) {
+                target = 0;
+                break;
+            }
+            lx = lx2;
+            ly = ly2;
+            src = target;
+
+	    // check if it has XdndAware
+	    Atom type = None;
+	    int f;
+	    unsigned long n, a;
+	    unsigned char *data = 0;
+	    XGetWindowProperty(QPaintDevice::x11AppDisplay(), target, qt_xdnd_aware, 0, 0, False,
+                               AnyPropertyType, &type, &f,&n,&a,&data);
+	    if (data)
+                XFree(data);
+	    if (type)
+                break;
+
+            // find child at the coordinates
+            if (!XTranslateCoordinates( QPaintDevice::x11AppDisplay(), src, src,
+                                        lx, ly, &lx2, &ly2, &target)) {
+                target = 0;
+                break;
+            }
+        }
 	if ( qt_xdnd_deco && (!target || target == qt_xdnd_deco->winId()) ) {
 	    target = findRealWindow(globalPos,rootwin,6);
 	}
     }
 
-    int emask = NoEventMask;
     QWidget* w;
     if ( target ) {
 	w = QWidget::find( (WId)target );
-	if ( w && w->isDesktop() && !w->acceptDrops() ) {
-	    emask = EnterWindowMask;
+	if ( w && w->isDesktop() && !w->acceptDrops() )
 	    w = 0;
-	}
     } else {
 	w = 0;
 	target = rootwin;
@@ -1183,9 +1233,7 @@ void QDragManager::move( const QPoint & globalPos )
 		target_version = QMIN(qt_xdnd_version,tv ? *tv : 1);
 		if ( tv )
 		    XFree( tv );
-		if ( !qt_badwindow() && type )
-		    emask = EnterWindowMask;
-		else
+		if (!(!qt_badwindow() && type))
 		    target = 0;
 	    }
 	}
@@ -1231,7 +1279,7 @@ void QDragManager::move( const QPoint & globalPos )
 	    if ( w ) {
 		qt_handle_xdnd_enter( w, (const XEvent *)&enter, FALSE );
 	    } else if ( target ) {
-		XSendEvent( QPaintDevice::x11AppDisplay(), proxy_target, False, emask,
+		XSendEvent( QPaintDevice::x11AppDisplay(), proxy_target, False, NoEventMask,
 			    (XEvent*)&enter );
 	    }
 	}
@@ -1253,7 +1301,7 @@ void QDragManager::move( const QPoint & globalPos )
 	if ( w )
 	    qt_handle_xdnd_position( w, (const XEvent *)&move, FALSE );
 	else
-	    XSendEvent( QPaintDevice::x11AppDisplay(), proxy_target, False, emask,
+	    XSendEvent( QPaintDevice::x11AppDisplay(), proxy_target, False, NoEventMask,
 			(XEvent*)&move );
     } else {
 	if ( willDrop ) {
@@ -1280,24 +1328,21 @@ void QDragManager::drop()
     drop.format = 32;
     drop.message_type = qt_xdnd_drop;
     drop.data.l[0] = object->source()->winId();
-    drop.data.l[1] = 1 << 24; // flags
-    drop.data.l[2] = 0; // ###
-    drop.data.l[3] = qt_x_time;
+    drop.data.l[1] = 0; // flags
+    drop.data.l[2] = qt_x_time;
+    drop.data.l[3] = 0;
     drop.data.l[4] = 0;
 
     QWidget * w = QWidget::find( qt_xdnd_current_proxy_target );
 
-    int emask = NoEventMask;
-    if ( w && w->isDesktop() && !w->acceptDrops() ) {
-	emask = EnterWindowMask;
+    if ( w && w->isDesktop() && !w->acceptDrops() )
 	w = 0;
-    }
 
     if ( w )
 	qt_handle_xdnd_drop( w, (const XEvent *)&drop, FALSE );
     else
-	XSendEvent( QPaintDevice::x11AppDisplay(), qt_xdnd_current_proxy_target, False, emask,
-		    (XEvent*)&drop );
+	XSendEvent( QPaintDevice::x11AppDisplay(), qt_xdnd_current_proxy_target, False,
+		    NoEventMask, (XEvent*)&drop );
 
 #ifndef QT_NO_CURSOR
     if ( restoreCursor ) {
@@ -1634,7 +1679,7 @@ bool QDragManager::drag( QDragObject * o, QDragObject::DragMode mode )
 			qt_xdnd_source_current_time );
     oldstate = ButtonState(-1); // #### Should use state that caused the drag
     drag_mode = mode;
-    global_accepted_action = QDropEvent::Copy; // #####
+    global_accepted_action = QDropEvent::Copy;
     updateMode(ButtonState(0));
     qt_xdnd_source_sameanswer = QRect();
     move(QCursor::pos());
@@ -1649,6 +1694,9 @@ bool QDragManager::drag( QDragObject * o, QDragObject::DragMode mode )
     dndCancelled = FALSE;
     qt_xdnd_dragging = TRUE;
 
+    if (!QWidget::mouseGrabber())
+	qt_xdnd_deco->grabMouse();
+
     qApp->enter_loop(); // Do the DND.
 
 #ifndef QT_NO_CURSOR
