@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qfontdatabase.cpp   3.2.3   edited Oct 16 14:02 $
+** $Id: qt/qfontdatabase.cpp   3.3.7   edited Feb 9 2006 $
 **
 ** Implementation of font database class.
 **
 ** Created : 990603
 **
-** Copyright (C) 1999-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1999-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -47,16 +47,19 @@
 
 #include <qcleanuphandler.h>
 
+#ifdef Q_WS_X11
+#include <locale.h>
+#endif
 #include <stdlib.h>
 
-// #define QFONTDATABASE_DEBUG
+//#define QFONTDATABASE_DEBUG
 #ifdef QFONTDATABASE_DEBUG
 #  define FD_DEBUG qDebug
 #else
 #  define FD_DEBUG if (FALSE) qDebug
 #endif
 
-// #define FONT_MATCH_DEBUG
+//#define FONT_MATCH_DEBUG
 #ifdef FONT_MATCH_DEBUG
 #  define FM_DEBUG qDebug
 #else
@@ -120,7 +123,7 @@ static int getFontWeight( const QString &weightString )
 #ifdef Q_WS_X11
 struct QtFontEncoding
 {
-    int encoding : 16;
+    signed int encoding : 16;
 
     uint xpoint   : 16;
     uint xres     : 8;
@@ -179,15 +182,14 @@ struct QtFontStyle
 			      weight( o.weight ), stretch( o.stretch ) { }
 	uint italic : 1;
 	uint oblique : 1;
-	int  weight : 8;
-	int stretch : 12;
+	signed int  weight : 8;
+	signed int stretch : 12;
 
 	bool operator==( const Key & other ) {
 	    return ( italic == other.italic &&
 		     oblique == other.oblique &&
 		     weight == other.weight &&
-		     ( stretch == 0 || other.stretch == 0 ||
-		       stretch == other.stretch ) );
+		     (stretch == 0 || other.stretch == 0 || stretch == other.stretch) );
 	}
 	bool operator!=( const Key &other ) {
 	    return !operator==(other);
@@ -320,14 +322,16 @@ struct QtFontFamily
 			UnSupported_Xft= 2, UnSupported_Xlfd = 4, UnSupported = 6 };
 
     QtFontFamily(const QString &n )
-	: fixedPitch( FALSE ),
+	:
 #ifdef Q_WS_X11
-	hasXft( FALSE ), xftScriptCheck( FALSE ), xlfdLoaded( FALSE ),
+        fixedPitch( TRUE ), hasXft( FALSE ), xftScriptCheck( FALSE ), xlfdLoaded( FALSE ), synthetic(FALSE),
+#else
+        fixedPitch( FALSE ),
 #endif
 #ifdef Q_WS_WIN
 	scriptCheck( FALSE ),
 #endif
-#ifdef Q_OS_MAC
+#if defined(Q_OS_MAC) && !defined(QWS)
 	fixedPitchComputed(FALSE),
 #endif
 	fullyLoaded( FALSE ),
@@ -345,11 +349,12 @@ struct QtFontFamily
     bool hasXft : 1;
     bool xftScriptCheck : 1;
     bool xlfdLoaded : 1;
+    bool synthetic : 1;
 #endif
 #ifdef Q_WS_WIN
     bool scriptCheck : 1;
 #endif
-#ifdef Q_WS_MAC
+#if defined(Q_OS_MAC) && !defined(QWS)
     bool fixedPitchComputed : 1;
 #endif
     bool fullyLoaded : 1;
@@ -358,6 +363,12 @@ struct QtFontFamily
 #ifdef Q_WS_X11
     QCString fontFilename;
     int fontFileIndex;
+#endif
+#ifdef Q_WS_MAC
+    FMFontFamily macFamily;
+#endif
+#ifdef Q_WS_WIN
+    QString english_name;
 #endif
     int count;
     QtFontFoundry **foundries;
@@ -441,178 +452,194 @@ QtFontFamily *QFontDatabasePrivate::family( const QString &f, bool create )
 
 
 #if defined(Q_WS_X11) || defined(Q_WS_WIN)
-static const unsigned short sample_chars[QFont::LastPrivateScript] =
+static const unsigned short sample_chars[QFont::LastPrivateScript][14] =
 {
     // European Alphabetic Scripts
     // Latin,
-    0x0041,
+    { 0x0041, 0x0 },
     // Greek,
-    0x0390,
+    { 0x0391, 0x0 },
     // Cyrillic,
-    0x0410,
+    { 0x0410, 0x0 },
     // Armenian,
-    0x0540,
+    { 0x0540, 0x0 },
     // Georgian,
-    0x10d0,
+    { 0x10d0, 0x0 },
     // Runic,
-    0x16a0,
+    { 0x16a0, 0x0 },
     // Ogham,
-    0x1680,
+    { 0x1680, 0x0 },
     // SpacingModifiers,
-    0x02b0,
+    { 0x02c6, 0x0 },
     // CombiningMarks,
-    0x0300,
+    { 0x0300, 0x0 },
 
     // Middle Eastern Scripts
     // Hebrew,
-    0x05d0,
+    { 0x05d0, 0x0 },
     // Arabic,
-    0x0630,
+    { 0x0630, 0x0 },
     // Syriac,
-    0x0710,
+    { 0x0710, 0x0 },
     // Thaana,
-    0x0780,
+    { 0x0780, 0x0 },
 
     // South and Southeast Asian Scripts
     // Devanagari,
-    0x0910,
+    { 0x0910, 0x0 },
     // Bengali,
-    0x0990,
+    { 0x0990, 0x0 },
     // Gurmukhi,
-    0x0a10,
+    { 0x0a10, 0x0 },
     // Gujarati,
-    0x0a90,
+    { 0x0a90, 0x0 },
     // Oriya,
-    0x0b10,
+    { 0x0b10, 0x0 },
     // Tamil,
-    0x0b90,
+    { 0x0b90, 0x0 },
     // Telugu,
-    0x0c10,
+    { 0x0c10, 0x0 },
     // Kannada,
-    0x0c90,
+    { 0x0c90, 0x0 },
     // Malayalam,
-    0x0d10,
+    { 0x0d10, 0x0 },
     // Sinhala,
-    0x0d90,
+    { 0x0d90, 0x0 },
     // Thai,
-    0x0e10,
+    { 0x0e10, 0x0 },
     // Lao,
-    0x0e81,
+    { 0x0e81, 0x0 },
     // Tibetan,
-    0x0f00,
+    { 0x0f00, 0x0 },
     // Myanmar,
-    0x1000,
+    { 0x1000, 0x0 },
     // Khmer,
-    0x1780,
+    { 0x1780, 0x0 },
 
     // East Asian Scripts
     // Han,
-    0x4e00,
+    { 0x4e00, 0x0 },
     // Hiragana,
-    0x3050,
+    {  0x3050, 0x4e00, 0x25EF, 0x3012, 0x3013, 0x30FB, 0x30FC, 0x5CE0, 0xFF66, 0 },
     // Katakana,
-    0x30b0,
+    { 0x30b0, 0x4e00, 0x25EF, 0x3012, 0x3013, 0x30FB, 0x30FC, 0x5CE0, 0xFF66, 0 },
     // Hangul,
-    0xac00,
+    { 0xac00, 0x0 },
     // Bopomofo,
-    0x3110,
+    { 0x3110, 0x0 },
     // Yi,
-    0xa000,
+    { 0xa000, 0x0 },
 
     // Additional Scripts
     // Ethiopic,
-    0x1200,
+    { 0x1200, 0x0 },
     // Cherokee,
-    0x13a0,
+    { 0x13a0, 0x0 },
     // CanadianAboriginal,
-    0x1410,
+    { 0x1410, 0x0 },
     // Mongolian,
-    0x1800,
+    { 0x1800, 0x0 },
 
     // Symbols
     // CurrencySymbols,
-    0x20aa,
+    { 0x20aa, 0x0 },
     // LetterlikeSymbols,
-    0x2122,
+    { 0x2103, 0x0 },
     // NumberForms,
-    0x215b,
+    { 0x2160, 0x0 },
     // MathematicalOperators,
-    0x222b,
+    { 0x222b, 0x0 },
     // TechnicalSymbols,
-    0x2440,
+    { 0x2312, 0x0 },
     // GeometricSymbols,
-    0x2500,
+    { 0x2500, 0x0 },
     // MiscellaneousSymbols,
-    0x2600,
+    { 0x2640, 0x0 },
     // EnclosedAndSquare,
-    0x2460,
+    { 0x2460, 0x0 },
     // Braille,
-    0x2800,
+    { 0x2800, 0x0 },
 
     // Unicode,
-    0xfffd,
+    { 0xfffd, 0x0 },
 
     // some scripts added in Unicode 3.2
     // Tagalog,
-    0x1700,
+    { 0x1700, 0x0 },
     // Hanunoo,
-    0x1720,
+    { 0x1720, 0x0 },
     // Buhid,
-    0x1740,
+    { 0x1740, 0x0 },
     // Tagbanwa,
-    0x1770,
+    { 0x1770, 0x0 },
 
     // KatakanaHalfWidth
-    0xff65,
+    { 0xff65, 0x0 },
+
+    // Limbu
+    { 0x1901, 0x0 },
+    // TaiLe
+    { 0x1950, 0x0 },
 
     // NScripts
-    0x0000,
+    { 0x0000, 0x0 },
     // NoScript
-    0x0000,
+    { 0x0000, 0x0 },
 
     // Han_Japanese
-    0x4e00,
-    // Han_SimplifiedChinese,
-    0x4e00,
-    // Han_TraditionalChinese,
-    0x4e00,
+    { 0x4e00, 0x25EF, 0x3012, 0x3013, 0x30FB, 0x5CE0, 0xFF66, 0 },
+    // Han_SimplifiedChinese, 0x3400 is optional
+    { 0x4e00, 0x201C, 0x3002, 0x6237, 0x9555, 0xFFE5, 0 },
+    // Han_TraditionalChinese, 0xF6B1 is optional
+    // OR Han_HongkongChinese, 0x3435, 0xE000, 0xF6B1 are optional
+    { 0x4e00, 0x201C, 0x3002, 0x6236, 0x9F98, 0xFFE5, 0 },
     // Han_Korean
-    0x4e00
+    { 0x4e00, 0 }
+    // Taiwan would be 0x201C, 0x3002, 0x4E00, 0x9F98, 0xFFE5
 };
 
-// returns a sample unicode character for the specified script
-static QChar sampleCharacter(QFont::Script script)
+#if defined(Q_WS_X11) && !defined(QT_NO_XFTFREETYPE)
+static inline bool requiresOpenType(QFont::Script s)
 {
-    return QChar(sample_chars[script]);
+    return (s >= QFont::Syriac && s <= QFont::Sinhala)
+		 || (s >= QFont::Myanmar && s <= QFont::Khmer);
 }
+#endif
 
-static inline bool canRender( QFontEngine *fe, const QChar &sample )
+static inline bool canRender( QFontEngine *fe, QFont::Script script )
 {
     if ( !fe ) return FALSE;
 
-    bool hasChar = fe->canRender( &sample, 1 );
+    bool hasChar = true;
 
+    if (!sample_chars[script][0])
+        hasChar = false;
+
+    int i = 0;
+    while (hasChar && sample_chars[script][i]){
+        QChar sample(sample_chars[script][i]);
+        if ( !fe->canRender( &sample, 1  ) ) {
+             hasChar = false;
 #ifdef FONT_MATCH_DEBUG
-    if (hasChar)
+	FM_DEBUG("    font has NOT char 0x%04x", sample.unicode() );
+        } else {
 	FM_DEBUG("    font has char 0x%04x", sample.unicode() );
 #endif
+        }
+        ++i;
+    }
+#if defined(Q_WS_X11) && !defined(QT_NO_XFTFREETYPE)
+    if (hasChar && requiresOpenType(script)) {
+	QOpenType *ot = fe->openType();
+	if (!ot || !ot->supportsScript(script))
+	    return FALSE;
+    }
+#endif
 
     return hasChar;
 }
 #endif // Q_WS_X11 || Q_WS_WIN
 
-#if !defined(Q_WS_MAC) && !defined(Q_WS_QWS)
-static
-QFontEngine *loadEngine( QFont::Script script, const QFontPrivate *fp, const QFontDef &request,
-			 QtFontFamily *family, QtFontFoundry *foundry,
-			 QtFontStyle *style, QtFontSize *size
-#ifdef Q_WS_X11
-			 , QtFontEncoding *encoding, bool forced_encoding
-#endif
-			 );
-#endif
-
-
 
 static QSingleCleanupHandler<QFontDatabasePrivate> qfontdatabase_cleanup;
 static QFontDatabasePrivate *db=0;
@@ -628,7 +655,78 @@ static QFontDatabasePrivate *db=0;
 #  include "qfontdatabase_qws.cpp"
 #endif
 
+static QtFontStyle *bestStyle(QtFontFoundry *foundry, const QtFontStyle::Key &styleKey)
+{
+    int best = 0;
+    int dist = 0xffff;
 
+    for ( int i = 0; i < foundry->count; i++ ) {
+	QtFontStyle *style = foundry->styles[i];
+
+	int d = QABS( styleKey.weight - style->key.weight );
+
+	if ( styleKey.stretch != 0 && style->key.stretch != 0 ) {
+	    d += QABS( styleKey.stretch - style->key.stretch );
+	}
+
+	if ( styleKey.italic ) {
+	    if ( !style->key.italic )
+		d += style->key.oblique ? 0x0001 : 0x1000;
+	} else if ( styleKey.oblique ) {
+	    if (!style->key.oblique )
+		d += style->key.italic ? 0x0001 : 0x1000;
+	} else if ( style->key.italic || style->key.oblique ) {
+	    d += 0x1000;
+	}
+
+	if ( d < dist ) {
+	    best = i;
+	    dist = d;
+	}
+    }
+
+    FM_DEBUG( "          best style has distance 0x%x", dist );
+    return foundry->styles[best];
+}
+
+#if defined(Q_WS_X11)
+static QtFontEncoding *findEncoding(QFont::Script script, int styleStrategy,
+                                    QtFontSize *size, int force_encoding_id)
+{
+    QtFontEncoding *encoding = 0;
+
+    if (force_encoding_id >= 0) {
+        encoding = size->encodingID(force_encoding_id);
+        if (!encoding)
+            FM_DEBUG("            required encoding_id not available");
+        return encoding;
+    }
+
+    if (styleStrategy & (QFont::OpenGLCompatible | QFont::PreferBitmap)) {
+        FM_DEBUG("            PreferBitmap and/or OpenGL set, skipping Xft");
+    } else {
+        encoding = size->encodingID(-1); // -1 == prefer Xft
+        if (encoding) return encoding;
+    }
+
+    // Xft not available, find an XLFD font, trying the default encoding first
+    encoding = size->encodingID(QFontPrivate::defaultEncodingID);
+
+    if (!encoding || !scripts_for_xlfd_encoding[encoding->encoding][script]) {
+        // find the first encoding that supports the requested script
+        encoding = 0;
+        for (int x = 0; !encoding && x < size->count; ++x) {
+            const int enc = size->encodings[x].encoding;
+            if (scripts_for_xlfd_encoding[enc][script]) {
+                encoding = size->encodings + x;
+                break;
+            }
+        }
+    }
+
+    return encoding;
+}
+#endif // Q_WS_X11
 
 
 #if defined(Q_WS_X11) || defined(Q_WS_WIN)
@@ -657,49 +755,7 @@ unsigned int bestFoundry( QFont::Script script, unsigned int score, int styleStr
 	FM_DEBUG( "          looking for matching style in foundry '%s'",
 		  foundry->name.isEmpty() ? "-- none --" : foundry->name.latin1() );
 
-	QtFontStyle *style = foundry->style( styleKey );
-	if ( !style && styleKey.italic ) {
-	    FM_DEBUG( "          italic not available, looking for oblique" );
-	    styleKey.italic = FALSE;
-	    styleKey.oblique = TRUE;
-	    style = foundry->style( styleKey );
-	}
-
-	if ( !style ) {
-	    FM_DEBUG( "          style not available, looking for closest match" );
-
-	    int best = 0;
-	    int dist = 0xffff;
-
-	    for ( int i = 0; i < foundry->count; i++ ) {
-		style = foundry->styles[i];
-
-		int d = QABS( styleKey.weight - style->key.weight );
-
-		if ( styleKey.stretch > 0 && styleKey.stretch != 100 &&
-		     style->key.stretch != 0 ) {
-		    d += QABS( styleKey.stretch - style->key.stretch );
-		}
-
-		if ( styleKey.italic ) {
-		    if ( !style->key.italic )
-			d += style->key.oblique ? 0x0800 : 0x1000;
-		} else if ( styleKey.oblique ) {
-		    if (!style->key.oblique )
-			d += style->key.italic ? 0x0800 : 0x1000;
-		} else if ( style->key.italic || style->key.oblique ) {
-		    d += 0x1000;
-		}
-
-		if ( d < dist ) {
-		    best = i;
-		    dist = d;
-		}
-	    }
-
-	    FM_DEBUG( "          best style has distance 0x%x", dist );
-	    style = foundry->styles[best];
-	}
+	QtFontStyle *style = bestStyle(foundry, styleKey);
 
 	if ( ! style->smoothScalable && ( styleStrategy & QFont::ForceOutline ) ) {
 	    FM_DEBUG( "            ForceOutline set, but not smoothly scalable" );
@@ -712,32 +768,61 @@ unsigned int bestFoundry( QFont::Script script, unsigned int score, int styleStr
 	// 1. see if we have an exact matching size
 	if (! (styleStrategy & QFont::ForceOutline)) {
 	    size = style->pixelSize(pixelSize);
-	    if (size) px = size->pixelSize;
+	    if (size) {
+                FM_DEBUG("          found exact size match (%d pixels)", size->pixelSize);
+                px = size->pixelSize;
+            }
 	}
 
 	// 2. see if we have a smoothly scalable font
 	if (! size && style->smoothScalable && ! (styleStrategy & QFont::PreferBitmap)) {
 	    size = style->pixelSize(SMOOTH_SCALABLE);
-	    if (size) px = pixelSize;
+	    if (size) {
+                FM_DEBUG("          found smoothly scalable font (%d pixels)", pixelSize);
+                px = pixelSize;
+            }
 	}
 
 	// 3. see if we have a bitmap scalable font
 	if (! size && style->bitmapScalable && (styleStrategy & QFont::PreferMatch)) {
 	    size = style->pixelSize(0);
-	    if (size) px = pixelSize;
+	    if (size) {
+                FM_DEBUG("          found bitmap scalable font (%d pixels)", pixelSize);
+                px = pixelSize;
+            }
 	}
 
+#ifdef Q_WS_X11
+        QtFontEncoding *encoding = 0;
+#endif
+
 	// 4. find closest size match
 	if (! size) {
 	    unsigned int distance = ~0u;
 	    for (int x = 0; x < style->count; ++x) {
+#ifdef Q_WS_X11
+                encoding =
+                    findEncoding(script, styleStrategy, style->pixelSizes + x, force_encoding_id);
+                if (!encoding) {
+                    FM_DEBUG("          size %3d does not support the script we want",
+                             style->pixelSizes[x].pixelSize);
+                    continue;
+                }
+#endif
+
 		unsigned int d = QABS(style->pixelSizes[x].pixelSize - pixelSize);
 		if (d < distance) {
 		    distance = d;
 		    size = style->pixelSizes + x;
+		    FM_DEBUG("          best size so far: %3d (%d)", size->pixelSize, pixelSize);
 		}
 	    }
 
+            if (!size) {
+                FM_DEBUG("          no size supports the script we want");
+                continue;
+            }
+
 	    if (style->bitmapScalable && ! (styleStrategy & QFont::PreferQuality) &&
 		(distance * 10 / pixelSize) >= 2) {
 		// the closest size is not close enough, go ahead and
@@ -750,40 +835,10 @@ unsigned int bestFoundry( QFont::Script script, unsigned int score, int styleStr
 	}
 
 #ifdef Q_WS_X11
-	QtFontEncoding *encoding = 0;
-	if ( force_encoding_id >= 0 ) {
-	    encoding = size->encodingID( force_encoding_id );
-	    if ( ! encoding ) {
-		FM_DEBUG( "            required encoding_id not available" );
-		continue;
-	    }
-	} else {
-	    encoding = size->encodingID( -1 ); // -1 == prefer Xft
-
-	    if ( encoding && ( styleStrategy & ( QFont::OpenGLCompatible |
-						 QFont::PreferBitmap ) ) ) {
-		FM_DEBUG( "            PreferBitmap and/or OpenGL set, skipping Xft" );
-		continue;
-	    }
-
-	    if ( ! encoding ) { // Xft not available, find an XLFD font
-		// try the default encoding first
-		encoding = size->encodingID( QFontPrivate::defaultEncodingID );
-
-		if ( ! encoding || ! scripts_for_xlfd_encoding[encoding->encoding][script] ) {
-		    // find the first encoding that supports the requested script
-		    encoding = 0;
-		    for (int x = 0; !encoding && x < size->count; ++x ) {
-			const int enc = size->encodings[x].encoding;
-			if ( scripts_for_xlfd_encoding[enc][script] ) {
-			    encoding = &size->encodings[x];
-			    break;
-			}
-		    }
-		}
-	    }
-	}
-
+        if (size) {
+            encoding = findEncoding(script, styleStrategy, size, force_encoding_id);
+            if (!encoding) size = 0;
+        }
 	if ( ! encoding ) {
 	    FM_DEBUG( "          foundry doesn't support the script we want" );
 	    continue;
@@ -791,24 +846,43 @@ unsigned int bestFoundry( QFont::Script script, unsigned int score, int styleStr
 #endif // Q_WS_X11
 
 	unsigned int this_score = 0x0000;
+        enum {
+            PitchMismatch       = 0x4000,
+            StyleMismatch       = 0x2000,
+            BitmapScaledPenalty = 0x1000,
+            EncodingMismatch    = 0x0002,
+            XLFDPenalty         = 0x0001
+        };
+
 #ifdef Q_WS_X11
 	if ( encoding->encoding != -1 ) {
-	    this_score += 1;
+	    this_score += XLFDPenalty;
 	    if ( encoding->encoding != QFontPrivate::defaultEncodingID )
-		this_score += 10;
+		this_score += EncodingMismatch;
 	}
-	if (pitch != '*') {
-	    if ( !( pitch == 'm' && encoding->pitch == 'c' ) && pitch != encoding->pitch )
-		this_score += 200;
+ 	if (pitch != '*') {
+ 	    if ( !( pitch == 'm' && encoding->pitch == 'c' ) && pitch != encoding->pitch )
+ 		this_score += PitchMismatch;
+ 	}
+#else
+        // ignore pitch for asian fonts, some of them misreport it, and they are all
+        // fixed pitch anyway.
+        if (pitch != '*' && (script <= QFont::NScripts && script != QFont::KatakanaHalfWidth
+                             && (script < QFont::Han || script > QFont::Yi))) {
+	    if ((pitch == 'm' && !family->fixedPitch)
+		|| (pitch == 'p' && family->fixedPitch))
+		this_score += PitchMismatch;
 	}
 #endif
 	if ( styleKey != style->key )
-	    this_score += 100;
+	    this_score += StyleMismatch;
 	if ( !style->smoothScalable && px != size->pixelSize ) // bitmap scaled
-	    this_score += 50;
+	    this_score += BitmapScaledPenalty;
+        if (px != pixelSize) // close, but not exact, size match
+            this_score += QABS(px - pixelSize);
 
 	if ( this_score < score ) {
-	    FM_DEBUG( "          found a match: score %u best score so far %u",
+	    FM_DEBUG( "          found a match: score %x best score so far %x",
 		      this_score, score );
 
 	    score = this_score;
@@ -819,7 +893,7 @@ unsigned int bestFoundry( QFont::Script script, unsigned int score, int styleStr
 	    *best_encoding = encoding;
 #endif // Q_WS_X11
 	} else {
-	    FM_DEBUG( "          score %u no better than best %u", this_score, score);
+	    FM_DEBUG( "          score %x no better than best %x", this_score, score);
 	}
     }
 
@@ -833,6 +907,10 @@ QFontEngine *
 QFontDatabase::findFont( QFont::Script script, const QFontPrivate *fp,
 			 const QFontDef &request, int force_encoding_id )
 {
+#ifndef Q_WS_X11
+    Q_UNUSED( force_encoding_id );
+#endif
+
     if ( !db )
 	initializeDb();
 
@@ -843,24 +921,32 @@ QFontDatabase::findFont( QFont::Script script, const QFontPrivate *fp,
 #ifdef Q_WS_X11
 			     , 0, 0, FALSE
 #endif
-			     );
+		);
 
 	    // if we fail to load the rawmode font, use a 12pixel box engine instead
 	    if (! fe) fe = new QFontEngineBox( 12 );
 	    return fe;
 	}
 
-	QFontCache::Key key( request, script, 
+	QFontCache::Key key( request, script,
 #ifdef Q_WS_WIN
-			    (int)fp->paintdevice 
+			     (int)fp->paintdevice
 #else
-			    fp->screen 
+			     fp->screen
 #endif
 	    );
 	fe = QFontCache::instance->findEngine( key );
 	if ( fe ) return fe;
     }
 
+#ifdef Q_WS_WIN
+    if (request.styleStrategy & QFont::PreferDevice) {
+        QFontEngine *fe = loadEngine(script, fp, request, 0, 0, 0);
+        if(fe)
+            return fe;
+    }
+#endif
+
     QString family_name, foundry_name;
     QtFontStyle::Key styleKey;
     styleKey.italic = request.italic;
@@ -870,6 +956,25 @@ QFontDatabase::findFont( QFont::Script script, const QFontPrivate *fp,
 
     parseFontName( request.family, foundry_name, family_name );
 
+#ifdef Q_WS_X11
+    if (family_name.isEmpty() && script == QFont::Han) {
+	// modify script according to locale
+	static QFont::Script defaultHan = QFont::UnknownScript;
+	if (defaultHan == QFont::UnknownScript) {
+	    QCString locale = setlocale(LC_ALL, NULL);
+	    if (locale.contains("ko"))
+		defaultHan = QFont::Han_Korean;
+	    else if (locale.contains("zh_TW") || locale.contains("zh_HK"))
+		defaultHan = QFont::Han_TraditionalChinese;
+	    else if (locale.contains("zh"))
+		defaultHan = QFont::Han_SimplifiedChinese;
+	    else
+		defaultHan = QFont::Han_Japanese;
+	}
+	script = defaultHan;
+    }
+#endif
+
     FM_DEBUG( "QFontDatabase::findFont\n"
 	      "  request:\n"
 	      "    family: %s [%s], script: %d (%s)\n"
@@ -882,235 +987,255 @@ QFontDatabase::findFont( QFont::Script script, const QFontPrivate *fp,
 	      script, scriptName( script ).latin1(),
 	      request.weight, request.italic, request.stretch, request.pixelSize, pitch );
 
-    QtFontFamily *best_family = 0;
-    QtFontFoundry *best_foundry = 0;
-    QtFontStyle *best_style = 0;
-    QtFontSize *best_size = 0;
+    bool usesFontConfig = FALSE;
+#ifdef QT_XFT2
+    if (family_name.isEmpty()
+	|| family_name == "Sans Serif"
+	|| family_name == "Serif"
+	|| family_name == "Monospace") {
+	fe = loadFontConfigFont(fp, request, script);
+        usesFontConfig = (fe != 0);
+    }
+    if (!fe)
+#endif
+    {
+	QtFontFamily *best_family = 0;
+	QtFontFoundry *best_foundry = 0;
+	QtFontStyle *best_style = 0;
+	QtFontSize *best_size = 0;
 #ifdef Q_WS_X11
-    QtFontEncoding *best_encoding = 0;
+	QtFontEncoding *best_encoding = 0;
 #endif // Q_WS_X11
 
-    unsigned int score = ~0;
+	unsigned int score = ~0;
 
-    int loop = 0;
-    while ( loop++ < 2 ) {
-	if ( loop == 2 || !family_name.isEmpty() )
-	    load( family_name, script );
+	load( family_name, script );
 
 	for ( int x = 0; x < db->count; ++x ) {
-	    QtFontFamily *family = db->families[x];
-	    if ( !family_name.isEmpty() &&
-		 ucstricmp( family->name, family_name ) != 0 )
+	    QtFontFamily *try_family = db->families[x];
+#ifdef Q_WS_X11
+            if (try_family->synthetic) // skip generated fontconfig fonts
+                continue;
+#endif
+
+            if ( !family_name.isEmpty() &&
+		 ucstricmp( try_family->name, family_name ) != 0
+#ifdef Q_WS_WIN
+		 && ucstricmp( try_family->english_name, family_name ) != 0
+#endif
+                )
 		continue;
 
-	    if ( loop == 2 && family_name.isNull() )
-		load( family->name, script );
+	    if ( family_name.isEmpty() )
+		load( try_family->name, script );
 
+	    uint score_adjust = 0;
 	    QFont::Script override_script = script;
-	    if ( ! ( family->scripts[script] & QtFontFamily::Supported ) && script != QFont::Unicode) {
+	    if ( ! ( try_family->scripts[script] & QtFontFamily::Supported )
+		 && script != QFont::Unicode) {
 		// family not supported in the script we want
-		if (family_name.isEmpty()) continue;
-
-		if (family->scripts[QFont::UnknownScript] & QtFontFamily::Supported) {
-		    // try with the unknown script (for a symbol font)
-		    override_script = QFont::UnknownScript;
-		} else if (family->scripts[QFont::Unicode] & QtFontFamily::Supported) {
-		    // try with the unicode script instead
-		    override_script = QFont::Unicode;
-		} else {
-		    // family not supported by unicode/unknown scripts
-		    continue;
-		}
+#ifdef Q_WS_X11
+		if (script >= QFont::Han_Japanese && script <= QFont::Han_Korean
+		    && try_family->scripts[QFont::Han] == QtFontFamily::Supported) {
+		    // try with the han script instead, give it a penalty
+		    if (override_script == QFont::Han_TraditionalChinese
+			&& (try_family->scripts[QFont::Han_SimplifiedChinese] & QtFontFamily::Supported)) {
+			override_script = QFont::Han_SimplifiedChinese;
+			score_adjust = 200;
+		    } else if (override_script == QFont::Han_SimplifiedChinese
+                               && (try_family->scripts[QFont::Han_TraditionalChinese] & QtFontFamily::Supported)) {
+			override_script = QFont::Han_TraditionalChinese;
+			score_adjust = 200;
+		    } else {
+			override_script = QFont::Han;
+			score_adjust = 400;
+		    }
+		} else
+#endif
+                    if (family_name.isEmpty()) {
+                        continue;
+                    } else if (try_family->scripts[QFont::UnknownScript] & QtFontFamily::Supported) {
+                        // try with the unknown script (for a symbol font)
+                        override_script = QFont::UnknownScript;
+#ifndef QT_XFT2
+                    } else if (try_family->scripts[QFont::Unicode] & QtFontFamily::Supported) {
+                        // try with the unicode script instead
+                        override_script = QFont::Unicode;
+#endif
+                    } else {
+                        // family not supported by unicode/unknown scripts
+                        continue;
+                    }
 	    }
 
+	    QtFontFoundry *try_foundry = 0;
+	    QtFontStyle *try_style = 0;
+	    QtFontSize *try_size = 0;
+#ifdef Q_WS_X11
+	    QtFontEncoding *try_encoding = 0;
+#endif // Q_WS_X11
+
 	    // as we know the script is supported, we can be sure
 	    // to find a matching font here.
 	    unsigned int newscore =
 		bestFoundry( override_script, score, request.styleStrategy,
-			     family, foundry_name, styleKey, request.pixelSize, pitch,
-			     &best_foundry, &best_style, &best_size
+			     try_family, foundry_name, styleKey, request.pixelSize, pitch,
+			     &try_foundry, &try_style, &try_size
 #ifdef Q_WS_X11
-			     , &best_encoding, force_encoding_id
+			     , &try_encoding, force_encoding_id
 #endif
-			     );
-	    if ( best_foundry == 0 ) {
+		    );
+	    if ( try_foundry == 0 ) {
 		// the specific foundry was not found, so look for
 		// any foundry matching our requirements
-		newscore = bestFoundry( override_script, score, request.styleStrategy, family,
+		newscore = bestFoundry( override_script, score, request.styleStrategy, try_family,
 					QString::null, styleKey, request.pixelSize,
-					pitch, &best_foundry, &best_style, &best_size
+					pitch, &try_foundry, &try_style, &try_size
 #ifdef Q_WS_X11
-					, &best_encoding, force_encoding_id
+					, &try_encoding, force_encoding_id
 #endif
-					);
+		    );
 	    }
+	    newscore += score_adjust;
 
 	    if ( newscore < score ) {
 		score = newscore;
-		best_family = family;
+		best_family = try_family;
+		best_foundry = try_foundry;
+		best_style = try_style;
+		best_size = try_size;
+#ifdef Q_WS_X11
+		best_encoding = try_encoding;
+#endif // Q_WS_X11
 	    }
 	    if ( newscore < 10 ) // xlfd instead of xft... just accept it
 		break;
 	}
-	if ( score < 10 )  // xlfd instead of xft... just accept it
-	    break;
-    }
 
-    if ( best_family == 0 || best_foundry == 0 || best_style == 0
+	if ( best_family != 0 && best_foundry != 0 && best_style != 0
 #ifdef Q_WS_X11
-	 || best_size == 0 || best_encoding == 0
+	     && best_size != 0 && best_encoding != 0
 #endif
-	 ) {
-	FM_DEBUG( "no best match found" );
-
-	if ( ! request.family.isEmpty() )
-	    return 0;
-
-	FM_DEBUG( "returning box engine" );
-
-	fe = new QFontEngineBox( request.pixelSize );
-
-	if ( fp ) {
-	    QFontCache::Key key( request, script, 
-#ifdef Q_WS_WIN
-				(int)fp->paintdevice
+	    ) {
+	    FM_DEBUG( "  BEST:\n"
+		      "    family: %s [%s]\n"
+		      "    weight: %d, italic: %d, oblique: %d\n"
+		      "    stretch: %d\n"
+		      "    pixelSize: %d\n"
+		      "    pitch: %c\n"
+		      "    encoding: %d\n",
+		      best_family->name.latin1(),
+		      best_foundry->name.isEmpty() ? "-- none --" : best_foundry->name.latin1(),
+		      best_style->key.weight, best_style->key.italic, best_style->key.oblique,
+		      best_style->key.stretch, best_size ? best_size->pixelSize : 0xffff,
+#ifdef Q_WS_X11
+		      best_encoding->pitch, best_encoding->encoding
 #else
-				fp->screen 
+		      'p', 0
+#endif
+		);
+
+	    fe = loadEngine( script, fp, request, best_family, best_foundry, best_style
+#ifdef Q_WS_X11
+			     , best_size, best_encoding, ( force_encoding_id >= 0 )
 #endif
 		);
-	    QFontCache::instance->insertEngine( key, fe );
 	}
+	if (fe) {
+	    fe->fontDef.family = best_family->name;
+	    if ( ! best_foundry->name.isEmpty() ) {
+		fe->fontDef.family += QString::fromLatin1( " [" );
+		fe->fontDef.family += best_foundry->name;
+		fe->fontDef.family += QString::fromLatin1( "]" );
+	    }
 
-	return fe;
+	    if ( best_style->smoothScalable )
+		fe->fontDef.pixelSize = request.pixelSize;
+	    else if ( best_style->bitmapScalable &&
+		      ( request.styleStrategy & QFont::PreferMatch ) )
+		fe->fontDef.pixelSize = request.pixelSize;
+	    else
+		fe->fontDef.pixelSize = best_size->pixelSize;
+
+	    fe->fontDef.styleHint     = request.styleHint;
+	    fe->fontDef.styleStrategy = request.styleStrategy;
+
+	    fe->fontDef.weight        = best_style->key.weight;
+	    fe->fontDef.italic        = best_style->key.italic || best_style->key.oblique;
+	    fe->fontDef.fixedPitch    = best_family->fixedPitch;
+	    fe->fontDef.stretch       = best_style->key.stretch;
+	    fe->fontDef.ignorePitch   = FALSE;
+	}
     }
 
-    FM_DEBUG( "  BEST:\n"
-	      "    family: %s [%s]\n"
-	      "    weight: %d, italic: %d, oblique: %d\n"
-	      "    stretch: %d\n"
-	      "    pixelSize: %d\n"
-	      "    pitch: %c\n"
-	      "    encoding: %d\n",
-	      best_family->name.latin1(),
-	      best_foundry->name.isEmpty() ? "-- none --" : best_foundry->name.latin1(),
-	      best_style->key.weight, best_style->key.italic, best_style->key.oblique,
-	      best_style->key.stretch, best_size ? best_size->pixelSize : 0xffff,
-#ifdef Q_WS_X11
-	      best_encoding->pitch, best_encoding->encoding
-#else
-	      'p', 0
-#endif
-	      );
-
-    fe = loadEngine( script, fp, request, best_family, best_foundry, best_style
-#ifdef Q_WS_X11
-		     , best_size, best_encoding, ( force_encoding_id >= 0 )
-#endif
-		     );
-
     if ( fe ) {
-	QChar sample = sampleCharacter( script );
-	if ( script != QFont::Unicode && !canRender( fe, sample ) ) {
-	    FM_DEBUG( "  WARN: font loaded cannot render sample 0x%04x",
-		      sample.unicode() );
+	if ( script != QFont::Unicode && !canRender( fe, script ) ) {
+	    FM_DEBUG( "  WARN: font loaded cannot render a sample char" );
+
 	    delete fe;
-
-	    if ( ! request.family.isEmpty() )
-		return 0;
-
-	    FM_DEBUG( "returning box engine" );
-
-	    fe = new QFontEngineBox( request.pixelSize );
-
-	    if ( fp ) {
-		QFontCache::Key key( request, script, 
+            fe = 0;
+	} else if ( fp ) {
+            QFontDef def = request;
+            if (def.family.isEmpty()) {
+                def.family = fp->request.family;
+                def.family = def.family.left(def.family.find(','));
+            }
+	    QFontCache::Key key( def, script,
 #ifdef Q_WS_WIN
-				    (int)fp->paintdevice
+				 (int)fp->paintdevice
 #else
-				    fp->screen 
-#endif
-		    );
-		QFontCache::instance->insertEngine( key, fe );
-	    }
-
-	    return fe;
-	}
-
-	fe->fontDef.family = best_family->name;
-	if ( ! best_foundry->name.isEmpty() ) {
-	    fe->fontDef.family += QString::fromLatin1( " [" );
-	    fe->fontDef.family += best_foundry->name;
-	    fe->fontDef.family += QString::fromLatin1( "]" );
-	}
-
-	if ( best_style->smoothScalable )
-	    fe->fontDef.pixelSize = request.pixelSize;
-	else if ( best_style->bitmapScalable &&
-		  ( request.styleStrategy & QFont::PreferMatch ) )
-	    fe->fontDef.pixelSize = request.pixelSize;
-	else
-	    fe->fontDef.pixelSize = best_size->pixelSize;
-
-	if ( fp ) {
-#if defined(Q_WS_X11)
-	    fe->fontDef.pointSize =
-		qRound(10. * qt_pointSize(fe->fontDef.pixelSize, fp->paintdevice, fp->screen));
-#elif defined(Q_WS_WIN)
-	    fe->fontDef.pointSize     = int( double( fe->fontDef.pixelSize ) * 720.0 /
-					     GetDeviceCaps(shared_dc,LOGPIXELSY) );
-#else
-	    fe->fontDef.pointSize     = int( double( fe->fontDef.pixelSize ) * 720.0 /
-					     96.0 );
-#endif
-	} else {
-	    fe->fontDef.pointSize = request.pointSize;
-	}
-	fe->fontDef.styleHint     = request.styleHint;
-	fe->fontDef.styleStrategy = request.styleStrategy;
-
-	fe->fontDef.weight        = best_style->key.weight;
-	fe->fontDef.italic        = best_style->key.italic || best_style->key.oblique;
-	fe->fontDef.fixedPitch    = best_family->fixedPitch;
-	fe->fontDef.stretch       = best_style->key.stretch;
-	fe->fontDef.ignorePitch   = FALSE;
-
-	if ( fp ) {
-	    QFontCache::Key key( request, script, 
-#ifdef Q_WS_WIN
-				(int)fp->paintdevice
-#else
-				fp->screen 
+				 fp->screen
 #endif
 		);
 	    QFontCache::instance->insertEngine( key, fe );
+            if (!usesFontConfig) {
+                for ( int i = 0; i < QFont::NScripts; ++i ) {
+                    if ( i == script ) continue;
 
-	    for ( int i = 0; i < QFont::NScripts; ++i ) {
-		if ( i == script ) continue;
+                    if (!canRender(fe, (QFont::Script) i))
+                        continue;
 
-		if ( ! canRender( fe, sampleCharacter( (QFont::Script) i ) ) )
-		    continue;
-
-		key.script = i;
-		QFontCache::instance->insertEngine( key, fe );
-	    }
+                    key.script = i;
+                    QFontCache::instance->insertEngine( key, fe );
+                }
+            }
 	}
-    } else {
-	if ( request.family.isEmpty() ) {
-	    FM_DEBUG( "returning box engine" );
+    }
 
-	    fe = new QFontEngineBox( request.pixelSize );
+    if (!fe) {
+	if ( !request.family.isEmpty() )
+            return 0;
 
-	    if ( fp ) {
-		QFontCache::Key key( request, script, 
+        FM_DEBUG( "returning box engine" );
+
+        fe = new QFontEngineBox( request.pixelSize );
+        fe->fontDef = request;
+
+        if ( fp ) {
+            QFontCache::Key key( request, script,
 #ifdef Q_WS_WIN
-				    (int)fp->paintdevice
+                                 (int)fp->paintdevice
 #else
-				    fp->screen 
+                                 fp->screen
 #endif
-		    );
-		QFontCache::instance->insertEngine( key, fe );
-	    }
-	}
+                );
+            QFontCache::instance->insertEngine( key, fe );
+        }
+    }
+
+    if ( fp ) {
+#if defined(Q_WS_X11)
+        fe->fontDef.pointSize =
+            qRound(10. * qt_pointSize(fe->fontDef.pixelSize, fp->paintdevice, fp->screen));
+#elif defined(Q_WS_WIN)
+        fe->fontDef.pointSize     = int( double( fe->fontDef.pixelSize ) * 720.0 /
+                                         GetDeviceCaps(shared_dc,LOGPIXELSY) );
+#else
+        fe->fontDef.pointSize     = int( double( fe->fontDef.pixelSize ) * 720.0 /
+                                         96.0 );
+#endif
+    } else {
+        fe->fontDef.pointSize = request.pointSize;
     }
 
     return fe;
@@ -1197,6 +1322,7 @@ QString QFontDatabase::styleString( const QFont &f )
 \code
 #include <qapplication.h>
 #include <qfontdatabase.h>
+#include <else.h>
 
 int main( int argc, char **argv )
 {
@@ -1440,7 +1566,7 @@ bool QFontDatabase::isFixedPitch(const QString &family,
     load( familyName );
 
     QtFontFamily *f = d->family( familyName );
-#ifdef Q_OS_MAC
+#if defined(Q_OS_MAC) && !defined(QWS)
     if (f) {
 	if (!f->fixedPitchComputed) {
 	    QFontMetrics fm(familyName);
@@ -1481,9 +1607,8 @@ bool QFontDatabase::isBitmapScalable( const QString &family,
 	QtFontFoundry *foundry = f->foundries[j];
 	if ( foundryName.isEmpty() || ucstricmp( foundry->name, foundryName ) == 0 ) {
 	    for ( int k = 0; k < foundry->count; k++ )
-		if ( foundry->styles[k]->key == styleKey &&
-		     foundry->styles[k]->bitmapScalable &&
-		     !foundry->styles[k]->smoothScalable ) {
+		if ((style.isEmpty() || foundry->styles[k]->key == styleKey) &&
+                    foundry->styles[k]->bitmapScalable && !foundry->styles[k]->smoothScalable) {
 		    bitmapScalable = TRUE;
 		    goto end;
 		}
@@ -1518,11 +1643,9 @@ bool  QFontDatabase::isSmoothlyScalable( const QString &family,
 
     for ( int j = 0; j < f->count; j++ ) {
 	QtFontFoundry *foundry = f->foundries[j];
-	if ( foundryName.isEmpty() ||
-	     ucstricmp( foundry->name, foundryName ) == 0 ) {
+	if ( foundryName.isEmpty() || ucstricmp( foundry->name, foundryName ) == 0 ) {
 	    for ( int k = 0; k < foundry->count; k++ )
-		if ( foundry->styles[k]->key == styleKey &&
-		     foundry->styles[k]->smoothScalable ) {
+		if ((style.isEmpty() || foundry->styles[k]->key == styleKey) && foundry->styles[k]->smoothScalable) {
 		    smoothScalable = TRUE;
 		    goto end;
 		}
@@ -1557,7 +1680,7 @@ bool  QFontDatabase::isScalable( const QString &family,
 QValueList<int> QFontDatabase::pointSizes( const QString &family,
 					   const QString &style)
 {
-#if defined(Q_WS_WIN) || defined(Q_WS_MAC)
+#if defined(Q_WS_MAC)
     // windows and macosx are always smoothly scalable
     Q_UNUSED( family );
     Q_UNUSED( style );
@@ -1637,11 +1760,7 @@ QFont QFontDatabase::font( const QString &family, const QString &style,
     }
 
     QtFontStyle::Key styleKey( style );
-    QtFontStyle *s = allStyles.style( styleKey );
-
-    // ### perhaps do a bit of matching to find the most compatible font?
-    if ( !s && allStyles.count )
-	s = allStyles.styles[0];
+    QtFontStyle *s = bestStyle(&allStyles, styleKey);
 
     if ( !s ) // no styles found?
 	return QApplication::font();
