@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qmetaobject.h   3.0.5   edited Dec 4 2001 $
+** $Id: qt/qmetaobject.h   3.3.7   edited Aug 31 2005 $
 **
 ** Definition of QMetaObject class
 **
 ** Created : 930419
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -44,7 +44,7 @@
 #endif // QT_H
 
 #ifndef Q_MOC_OUTPUT_REVISION
-#define Q_MOC_OUTPUT_REVISION 19
+#define Q_MOC_OUTPUT_REVISION 26
 #endif
 
 class QObject;
@@ -94,9 +94,9 @@ public:
     int keysToValue( const QStrList& keys ) const;
     QStrList valueToKeys( int value ) const;
 
-    bool designable( QObject* ) const;
-    bool scriptable( QObject* ) const;
-    bool stored( QObject* ) const;
+    bool designable( QObject* = 0 ) const;
+    bool scriptable( QObject* = 0 ) const;
+    bool stored( QObject* = 0 ) const;
 
     bool reset( QObject* ) const;
 
@@ -108,6 +108,7 @@ public:
 	Readable	= 0x00000001,
 	Writable	= 0x00000002,
 	EnumOrSet	= 0x00000004,
+	UnresolvedEnum	= 0x00000008,
 	StdSet		= 0x00000100,
 	Override	= 0x00000200
     };
@@ -118,6 +119,7 @@ public:
     int id() const; 			// internal
 
     QMetaObject** meta; 		// internal
+
     const QMetaEnum* enumData;		// internal
     int _id; 				// internal
     void clear(); 			// internal
@@ -146,6 +148,16 @@ public:
 #endif
 		 const QClassInfo *const class_info, int n_info );
 
+#ifndef QT_NO_PROPERTIES
+    QMetaObject( const char * const class_name, QMetaObject *superclass,
+		 const QMetaData * const slot_data, int n_slots,
+		 const QMetaData * const signal_data, int n_signals,
+		 const QMetaProperty *const prop_data, int n_props,
+		 const QMetaEnum *const enum_data, int n_enums,
+		 bool (*qt_static_property)(QObject*, int, int, QVariant*),
+		 const QClassInfo *const class_info, int n_info );
+#endif
+
 
     virtual ~QMetaObject();
 
@@ -198,10 +210,20 @@ public:
 #endif
 					const QClassInfo *const  class_info, int n_info );
 #ifndef QT_NO_PROPERTIES
+    static QMetaObject	*new_metaobject( const char *, QMetaObject *,
+					const QMetaData *const, int,
+					const QMetaData *const, int,
+					const QMetaProperty *const prop_data, int n_props,
+					const QMetaEnum *const enum_data, int n_enums,
+					 bool (*qt_static_property)(QObject*, int, int, QVariant*),
+					const QClassInfo *const  class_info, int n_info );
+    QStrList		enumeratorNames( bool super = FALSE ) const;
+    int numEnumerators( bool super = FALSE ) const;
     const QMetaEnum		*enumerator( const char* name, bool super = FALSE ) const;
 #endif
 
     static QMetaObject *metaObject( const char *class_name );
+    static bool hasMetaObject( const char *class_name );
 
 private:
     QMemberDict		*init( const QMetaData *, int );
@@ -219,6 +241,10 @@ private:
     int slotoffset;
 #ifndef QT_NO_PROPERTIES
     int propertyoffset;
+public:
+    bool qt_static_property( QObject* o, int id, int f, QVariant* v);
+private:
+    friend class QMetaProperty;
 #endif
 
 private:	// Disabled copy constructor and operator=
@@ -239,9 +265,12 @@ inline int QMetaObject::propertyOffset() const
 { return propertyoffset; }
 #endif
 
+typedef QMetaObject *(*QtStaticMetaObjectFunction)();
+
 class Q_EXPORT QMetaObjectCleanUp
 {
 public:
+    QMetaObjectCleanUp( const char *mo_name, QtStaticMetaObjectFunction );
     QMetaObjectCleanUp();
     ~QMetaObjectCleanUp();
 
