@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qpaintdevice_x11.cpp   3.2.3   edited Jul 9 14:26 $
+** $Id: qt/qpaintdevice_x11.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of QPaintDevice class for X11
 **
 ** Created : 940721
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -277,14 +277,6 @@ Qt::HANDLE QPaintDevice::handle() const
     return hd;
 }
 
-
-#ifndef QT_NO_XFTFREETYPE
-#ifndef QT_XFT2
-// Xft1 doesn't have XftDrawPicture, so we fake it in qtaddons_x11.cpp
-extern "C" Qt::HANDLE XftDrawPicture( XftDraw * );
-#endif // QT_XFT2
-#endif // QT_NO_XFTFREETYPE
-
 /*!
     Returns the window system handle of the paint device for XRender
     support. Use of this function is not portable. This function will
@@ -804,7 +796,8 @@ static GC cache_mask_gc( Display *dpy, Drawable hd, int mask_no, Pixmap mask )
     \relates QPaintDevice
 
     Copies a block of pixels from \a src to \a dst, perhaps merging
-    each pixel according to the raster operation \a rop. \a sx, \a sy
+    each pixel according to the \link Qt::RasterOp raster operation \endlink
+    \a rop. \a sx, \a sy
     is the top-left pixel in \a src (0, 0) by default, \a dx, \a dy is
     the top-left position in \a dst and \a sw, \a sh is the size of
     the copied block (all of \a src by default).
@@ -894,8 +887,8 @@ void bitBlt( QPaintDevice *dst, int dx, int dy,
 	    return;
 	}
 	QPDevCmdParam param[3];
-	QPoint p(dx,dy);
-	param[0].point	= &p;
+    QRect r(dx, dy, pm->width(), pm->height());
+	param[0].rect	= &r;
 	param[1].pixmap = pm;
 	dst->cmd( QPaintDevice::PdcDrawPixmap, 0, param );
 	if ( tmp_pm )
