@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qpngio.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qpngio.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of PNG QImage IOHandler
 **
 ** Created : 970521
 **
-** Copyright (C) 1992-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -128,9 +128,23 @@ void setup_qt( QImage& image, png_structp png_ptr, png_infop info_ptr, float scr
 	if ( bit_depth == 1 && info_ptr->channels == 1 ) {
 	    png_set_invert_mono( png_ptr );
 	    png_read_update_info( png_ptr, info_ptr );
-	    image.create( width, height, 1, 2, QImage::BigEndian );
+	    if (!image.create( width, height, 1, 2, QImage::BigEndian ))
+		return;
 	    image.setColor( 1, qRgb(0,0,0) );
 	    image.setColor( 0, qRgb(255,255,255) );
+	} else if (bit_depth == 16 && png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS)) {
+	    png_set_expand(png_ptr);
+	    png_set_strip_16(png_ptr);
+	    png_set_gray_to_rgb(png_ptr);
+
+	    if (!image.create(width, height, 32))
+		return;
+	    image.setAlphaBuffer(TRUE);
+
+	    if (QImage::systemByteOrder() == QImage::BigEndian)
+		png_set_swap_alpha(png_ptr);
+
+	    png_read_update_info(png_ptr, info_ptr);
 	} else {
 	    if ( bit_depth == 16 )
 		png_set_strip_16(png_ptr);
@@ -138,18 +152,17 @@ void setup_qt( QImage& image, png_structp png_ptr, png_infop info_ptr, float scr
 		png_set_packing(png_ptr);
 	    int ncols = bit_depth < 8 ? 1 << bit_depth : 256;
 	    png_read_update_info(png_ptr, info_ptr);
-	    image.create(width, height, 8, ncols);
+	    if (!image.create(width, height, 8, ncols))
+		return;
 	    for (int i=0; i<ncols; i++) {
 		int c = i*255/(ncols-1);
 		image.setColor( i, qRgba(c,c,c,0xff) );
 	    }
 	    if ( png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS) ) {
-		int g = info_ptr->trans_values.gray;
-		if ( bit_depth > 8 ) {
-		    // transparency support disabled for now
-		} else {
-		    image.setAlphaBuffer( TRUE );
-		    image.setColor(g, RGB_MASK & image.color(g));
+		const int g = info_ptr->trans_values.gray;
+		if (g < ncols) {
+		    image.setAlphaBuffer(TRUE);
+		    image.setColor(g, image.color(g) & RGB_MASK);
 		}
 	    }
 	}
@@ -163,8 +176,9 @@ void setup_qt( QImage& image, png_structp png_ptr, png_infop info_ptr, float scr
 	png_read_update_info( png_ptr, info_ptr );
 	png_get_IHDR(png_ptr, info_ptr,
 	    &width, &height, &bit_depth, &color_type, 0, 0, 0);
-	image.create(width, height, bit_depth, info_ptr->num_palette,
-	    QImage::BigEndian);
+	if (!image.create(width, height, bit_depth, info_ptr->num_palette,
+	    QImage::BigEndian))
+	    return;
 	int i = 0;
 	if ( png_get_valid(png_ptr, info_ptr, PNG_INFO_tRNS) ) {
 	    image.setAlphaBuffer( TRUE );
@@ -199,7 +213,8 @@ void setup_qt( QImage& image, png_structp png_ptr, png_infop info_ptr, float scr
 	if ( color_type == PNG_COLOR_TYPE_GRAY_ALPHA )
 	    png_set_gray_to_rgb(png_ptr);
 
-	image.create(width, height, 32);
+	if (!image.create(width, height, 32))
+	    return;
 
 	// Only add filler if no alpha, or we can get 5 channel data.
 	if (!(color_type & PNG_COLOR_MASK_ALPHA)
@@ -280,7 +295,11 @@ void read_png_image(QImageIO* iio)
 
     QImage image;
     setup_qt(image, png_ptr, info_ptr, iio->gamma());
-
+    if (image.isNull()) {
+	png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
+	iio->setStatus(-5);
+	return;
+    }
 
     png_uint_32 width;
     png_uint_32 height;
