@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qurl.cpp   3.2.3   edited Sep 19 14:50 $
+** $Id: qt/qurl.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of QUrl class
 **
 ** Created : 950429
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -1032,12 +1032,14 @@ QString QUrl::path( bool correct ) const
 	    if ( ( d->path.left( 1 ) == "/" || d->path.left( 1 ) == "\\" ) &&
 		 d->path.length() > 1 ) {
 		d->cleanPath = d->path;
-		bool share = d->cleanPath[ 1 ] == '\\' || d->cleanPath[ 1 ] == '/';
+		bool share = (d->cleanPath[0] == '\\' && d->cleanPath[1] == '\\') ||
+		             (d->cleanPath[0] == '/' && d->cleanPath[1] == '/');
 		slashify( d->cleanPath, FALSE );
 		d->cleanPath = QDir::cleanDirPath( d->cleanPath );
 		if ( share ) {
 		    check = FALSE;
-		    d->cleanPath.prepend( "/" );
+		    while (d->cleanPath.at(0) != '/' || d->cleanPath.at(1) != '/')
+			d->cleanPath.prepend("/");
 		}
 	    }
 #endif
@@ -1046,10 +1048,13 @@ QString QUrl::path( bool correct ) const
 		if ( !fi.exists() )
 		    d->cleanPath = d->path;
 		else if ( fi.isDir() ) {
-		    QString dir =
-			QDir::cleanDirPath( (qt_resolve_symlinks ?
-					    QDir( d->path ).canonicalPath() :
-					    QDir( d->path ).absPath()) ) + "/";
+                    QString canPath = QDir( d->path ).canonicalPath();
+                    QString dir;
+                    if ( qt_resolve_symlinks && !canPath.isNull() )
+			dir = QDir::cleanDirPath( canPath );
+                    else
+			dir = QDir::cleanDirPath( QDir( d->path ).absPath() );
+		    dir += "/";
 		    if ( dir == "//" )
 			d->cleanPath = "/";
 		    else
@@ -1180,7 +1185,7 @@ void QUrl::encode( QString& url )
     QCString curl = url.utf8();
     int oldlen = curl.length();
 
-    const QCString special( "<>#@\"&%$:,;?={}|^~[]\'`\\ \n\t\r" );
+    const QCString special( "+<>#@\"&%$:,;?={}|^~[]\'`\\ \n\t\r" );
     QString newUrl;
     int newlen = 0;
 
@@ -1242,7 +1247,7 @@ void QUrl::decode( QString& url )
     int i = 0;
     while ( i < oldlen ) {
 	uchar c = (uchar)curl[ i++ ];
-	if ( c == '%' ) {
+	if ( c == '%' && i <= oldlen - 2 ) {
 	    c = hex_to_int( (uchar)curl[ i ] ) * 16 + hex_to_int( (uchar)curl[ i + 1 ] );
 	    i += 2;
 	}
