@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qurloperator.cpp   3.2.3   edited Aug 12 16:22 $
+** $Id: qt/qurloperator.cpp   3.3.7   edited Jan 25 2006 $
 **
 ** Implementation of QUrlOperator class
 **
 ** Created : 950429
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the kernel module of the Qt GUI Toolkit.
 **
@@ -374,6 +374,9 @@ QUrlOperator::~QUrlOperator()
 */
 const QNetworkOperation *QUrlOperator::startOperation( QNetworkOperation *op )
 {
+    if ( !d->networkProtocol )
+        getNetworkProtocol();
+    
     if ( d->networkProtocol && (d->networkProtocol->supportedOperations()&op->operation()) ) {
 	d->networkProtocol->addOperation( op );
 	if ( op->operation() == QNetworkProtocol::OpListChildren )
@@ -576,6 +579,10 @@ QPtrList<QNetworkOperation> QUrlOperator::copy( const QString &from, const QStri
     // prepare some string for later usage
     QString frm = *uFrom;
     QString file = uFrom->fileName();
+
+     if (frm == to + file)
+         return ops;
+    
     file.prepend( "/" );
 
     // uFrom and uTo are deleted when the QNetworkProtocol deletes itself via
@@ -590,22 +597,22 @@ QPtrList<QNetworkOperation> QUrlOperator::copy( const QString &from, const QStri
     if ( gProt && (gProt->supportedOperations()&QNetworkProtocol::OpGet) &&
 	 pProt && (pProt->supportedOperations()&QNetworkProtocol::OpPut) ) {
 
-	connect( gProt, SIGNAL( data( const QByteArray &, QNetworkOperation * ) ),
-		 this, SLOT( copyGotData( const QByteArray &, QNetworkOperation * ) ) );
-	connect( gProt, SIGNAL( dataTransferProgress( int, int, QNetworkOperation * ) ),
-		 this, SIGNAL( dataTransferProgress( int, int, QNetworkOperation * ) ) );
-	connect( gProt, SIGNAL( finished( QNetworkOperation * ) ),
-		 this, SLOT( continueCopy( QNetworkOperation * ) ) );
-	connect( gProt, SIGNAL( finished( QNetworkOperation * ) ),
-		 this, SIGNAL( finished( QNetworkOperation * ) ) );
-	connect( gProt, SIGNAL( connectionStateChanged( int, const QString & ) ),
-		 this, SIGNAL( connectionStateChanged( int, const QString & ) ) );
+	connect( gProt, SIGNAL( data(const QByteArray&,QNetworkOperation*) ),
+		 this, SLOT( copyGotData(const QByteArray&,QNetworkOperation*) ) );
+	connect( gProt, SIGNAL( dataTransferProgress(int,int,QNetworkOperation*) ),
+		 this, SIGNAL( dataTransferProgress(int,int,QNetworkOperation*) ) );
+	connect( gProt, SIGNAL( finished(QNetworkOperation*) ),
+		 this, SLOT( continueCopy(QNetworkOperation*) ) );
+	connect( gProt, SIGNAL( finished(QNetworkOperation*) ),
+		 this, SIGNAL( finished(QNetworkOperation*) ) );
+	connect( gProt, SIGNAL( connectionStateChanged(int,const QString&) ),
+		 this, SIGNAL( connectionStateChanged(int,const QString&) ) );
 
-	connect( pProt, SIGNAL( dataTransferProgress( int, int, QNetworkOperation * ) ),
-		 this, SIGNAL( dataTransferProgress( int, int, QNetworkOperation * ) ) );
-	connect( pProt, SIGNAL( finished( QNetworkOperation * ) ),
-		 this, SIGNAL( finished( QNetworkOperation * ) ) );
-	connect( pProt, SIGNAL( finished( QNetworkOperation * ) ),
+	connect( pProt, SIGNAL( dataTransferProgress(int,int,QNetworkOperation*) ),
+		 this, SIGNAL( dataTransferProgress(int,int,QNetworkOperation*) ) );
+	connect( pProt, SIGNAL( finished(QNetworkOperation*) ),
+		 this, SIGNAL( finished(QNetworkOperation*) ) );
+	connect( pProt, SIGNAL( finished(QNetworkOperation*) ),
 		 this, SLOT( finishedCopy() ) );
 
 	QNetworkOperation *opGet = new QNetworkOperation( QNetworkProtocol::OpGet, frm, QString::null, QString::null );
@@ -925,8 +932,8 @@ void QUrlOperator::getNetworkProtocol()
 
     d->networkProtocol = (QNetworkProtocol *)p;
     d->networkProtocol->setUrl( this );
-    connect( d->networkProtocol, SIGNAL( itemChanged( QNetworkOperation * ) ),
-	     this, SLOT( slotItemChanged( QNetworkOperation * ) ) );
+    connect( d->networkProtocol, SIGNAL( itemChanged(QNetworkOperation*) ),
+	     this, SLOT( slotItemChanged(QNetworkOperation*) ) );
 }
 
 /*!
@@ -935,8 +942,10 @@ void QUrlOperator::getNetworkProtocol()
 
 void QUrlOperator::deleteNetworkProtocol()
 {
-    delete d->networkProtocol;
-    d->networkProtocol = 0;
+    if (d->networkProtocol) {
+        d->networkProtocol->deleteLater();
+        d->networkProtocol = 0;
+    }
 }
 
 /*!
@@ -1109,10 +1118,10 @@ void QUrlOperator::continueCopy( QNetworkOperation *op )
 	    deleteOperation( rm );
 	}
     }
-    disconnect( gProt, SIGNAL( data( const QByteArray &, QNetworkOperation * ) ),
-		this, SLOT( copyGotData( const QByteArray &, QNetworkOperation * ) ) );
-    disconnect( gProt, SIGNAL( finished( QNetworkOperation * ) ),
-		this, SLOT( continueCopy( QNetworkOperation * ) ) );
+    disconnect( gProt, SIGNAL( data(const QByteArray&,QNetworkOperation*) ),
+		this, SLOT( copyGotData(const QByteArray&,QNetworkOperation*) ) );
+    disconnect( gProt, SIGNAL( finished(QNetworkOperation*) ),
+		this, SLOT( continueCopy(QNetworkOperation*) ) );
 }
 
 /*!
