@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qftp.cpp   3.2.3   edited Aug 12 16:23 $
+** $Id: qt/qftp.cpp   3.3.7   edited Sep 21 2005 $
 **
 ** Implementation of QFtp class.
 **
 ** Created : 970521
 **
-** Copyright (C) 1997-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1997-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the network module of the Qt GUI Toolkit.
 **
@@ -40,6 +40,7 @@
 #ifndef QT_NO_NETWORKPROTOCOL_FTP
 
 #include "qsocket.h"
+#include "qsocketdevice.h"
 #include "qurlinfo.h"
 #include "qurloperator.h"
 #include "qstringlist.h"
@@ -48,6 +49,10 @@
 #include "qfileinfo.h"
 #include "qptrdict.h" // binary compatibility
 
+#ifndef QT_NO_TEXTCODEC
+#include "qtextcodec.h"
+#endif
+
 //#define QFTPPI_DEBUG
 //#define QFTPDTP_DEBUG
 
@@ -293,12 +298,12 @@ QFtpDTP::QFtpDTP( QFtpPI *p, QObject *parent, const char *name ) :
 	     SLOT( socketConnected() ) );
     connect( &socket, SIGNAL( readyRead() ),
 	     SLOT( socketReadyRead() ) );
-    connect( &socket, SIGNAL( error( int ) ),
-	     SLOT( socketError( int ) ) );
+    connect( &socket, SIGNAL( error(int) ),
+	     SLOT( socketError(int) ) );
     connect( &socket, SIGNAL( connectionClosed() ),
 	     SLOT( socketConnectionClosed() ) );
-    connect( &socket, SIGNAL( bytesWritten( int ) ),
-	     SLOT( socketBytesWritten( int ) ) );
+    connect( &socket, SIGNAL( bytesWritten(int) ),
+	     SLOT( socketBytesWritten(int) ) );
 }
 
 void QFtpDTP::setData( QByteArray *ba )
@@ -465,6 +470,7 @@ bool QFtpDTP::parseDir( const QString &buffer, const QString &userName, QUrlInfo
     QTime time;
     QString dateStr;
     dateStr += "Sun ";
+    lst[ 5 ][ 0 ] = lst[ 5 ][ 0 ].upper();
     dateStr += lst[ 5 ];
     dateStr += ' ';
     dateStr += lst[ 6 ];
@@ -506,6 +512,15 @@ bool QFtpDTP::parseDir( const QString &buffer, const QString &userName, QUrlInfo
 
 void QFtpDTP::socketConnected()
 {
+#if !defined (Q_WS_QWS)
+    // Use a large send buffer to reduce the number
+    // of writeBlocks when download and uploading files.
+    // The actual size used here (128k) is default on most
+    // Unixes.
+    socket.socketDevice()->setSendBufferSize(128 * 1024);
+    socket.socketDevice()->setReceiveBufferSize(128 * 1024);
+#endif
+
     bytesDone = 0;
 #if defined(QFTPDTP_DEBUG)
     qDebug( "QFtpDTP::connectState( CsConnected )" );
@@ -667,6 +682,7 @@ bool QFtpPI::sendCommands( const QStringList &cmds )
 void QFtpPI::clearPendingCommands()
 {
     pendingCommands.clear();
+    dtp.abortConnection();
     currentCmd = QString::null;
     state = Idle;
 }
@@ -851,17 +867,20 @@ bool QFtpPI::processReply()
 	rawCommand = FALSE;
     } else if ( replyCodeInt == 227 ) {
 	// 227 Entering Passive Mode (h1,h2,h3,h4,p1,p2)
-	int l = replyText.find( "(" );
-	int r = replyText.find( ")" );
-	if ( l<0 || r<0 ) {
+        // rfc959 does not define this response precisely, and gives
+        // both examples where the parenthesis are used, and where
+        // they are missing. We need to scan for the address and host
+        // info.
+	QRegExp addrPortPattern("(\\d+),(\\d+),(\\d+),(\\d+),(\\d+),(\\d+)");
+	if (addrPortPattern.search(replyText) == -1) {
 #if defined(QFTPPI_DEBUG)
 	    qDebug( "QFtp: bad 227 response -- address and port information missing" );
 #endif
 	    // ### error handling
 	} else {
-	    QStringList lst = QStringList::split( ',', replyText.mid(l+1,r-l-1) );
-	    QString host = lst[0] + "." + lst[1] + "." + lst[2] + "." + lst[3];
-	    Q_UINT16 port = ( lst[4].toUInt() << 8 ) + lst[5].toUInt();
+	    QStringList lst = addrPortPattern.capturedTexts();
+	    QString host = lst[1] + "." + lst[2] + "." + lst[3] + "." + lst[4];
+	    Q_UINT16 port = ( lst[5].toUInt() << 8 ) + lst[6].toUInt();
 	    waitForDtpToConnect = TRUE;
 	    dtp.connectToHost( host, port );
 	}
@@ -912,6 +931,10 @@ bool QFtpPI::processReply()
     return TRUE;
 }
 
+#ifndef QT_NO_TEXTCODEC
+QM_EXPORT_FTP QTextCodec *qt_ftp_filename_codec = 0;
+#endif
+
 /*
   Starts next pending command. Returns FALSE if there are no pending commands,
   otherwise it returns TRUE.
@@ -937,7 +960,16 @@ bool QFtpPI::startNextCmd()
     qDebug( "QFtpPI send: %s", currentCmd.left( currentCmd.length()-2 ).latin1() );
 #endif
     state = Waiting;
-    commandSocket.writeBlock( currentCmd.latin1(), currentCmd.length() );
+#ifndef QT_NO_TEXTCODEC
+    if ( qt_ftp_filename_codec ) {
+	int len = 0;
+	QCString enc = qt_ftp_filename_codec->fromUnicode(currentCmd,len);
+	commandSocket.writeBlock( enc.data(), len );
+    } else
+#endif
+    {
+	commandSocket.writeBlock( currentCmd.latin1(), currentCmd.length() );
+    }
     return TRUE;
 }
 
@@ -1217,12 +1249,12 @@ void QFtp::init()
 
     connect( &d->pi, SIGNAL(connectState(int)),
 	    SLOT(piConnectState(int)) );
-    connect( &d->pi, SIGNAL(finished( const QString& )),
-	    SLOT(piFinished( const QString& )) );
-    connect( &d->pi, SIGNAL(error(int, const QString&)),
-	    SLOT(piError(int, const QString&)) );
-    connect( &d->pi, SIGNAL(rawFtpReply(int, const QString&)),
-	    SLOT(piFtpReply(int, const QString&)) );
+    connect( &d->pi, SIGNAL(finished(const QString&)),
+	    SLOT(piFinished(const QString&)) );
+    connect( &d->pi, SIGNAL(error(int,const QString&)),
+	    SLOT(piError(int,const QString&)) );
+    connect( &d->pi, SIGNAL(rawFtpReply(int,const QString&)),
+	    SLOT(piFtpReply(int,const QString&)) );
 
     connect( &d->pi.dtp, SIGNAL(readyRead()),
 	    SIGNAL(readyRead()) );
@@ -1529,7 +1561,7 @@ int QFtp::cd( const QString &dir )
     of the operation (it is safe to delete it when the
     commandFinished() signal is emitted). In this case the readyRead()
     signal is \e not emitted and you cannot read data with the
-    readBlcok or readAll() functions.
+    readBlock() or readAll() functions.
 
     If you don't read the data immediately it becomes available, i.e.
     when the readyRead() signal is emitted, it is still available
@@ -2050,10 +2082,14 @@ void QFtp::piError( int errorCode, const QString &text )
     }
 
     d->pi.clearPendingCommands();
+    clearPendingCommands();
     emit commandFinished( c->id, TRUE );
 
-    d->pending.clear();
-    emit done( TRUE );
+    d->pending.removeFirst();
+    if ( d->pending.isEmpty() )
+	emit done( TRUE );
+    else
+	startNextCommand();
 }
 
 void QFtp::piConnectState( int state )
@@ -2157,8 +2193,8 @@ bool QFtp::checkConnection( QNetworkOperation *op )
 {
     QFtpPrivate *d = ::d( this );
     if ( state() == Unconnected && !d->npWaitForLoginDone ) {
-	connect( this, SIGNAL(listInfo(const QUrlInfo &)),
-		this, SLOT(npListInfo(const QUrlInfo &)) );
+	connect( this, SIGNAL(listInfo(const QUrlInfo&)),
+		this, SLOT(npListInfo(const QUrlInfo&)) );
 	connect( this, SIGNAL(done(bool)),
 		this, SLOT(npDone(bool)) );
 	connect( this, SIGNAL(stateChanged(int)),
@@ -2281,8 +2317,8 @@ void QFtp::npDone( bool err )
     d->npWaitForLoginDone = FALSE;
 
     if ( state() == Unconnected ) {
-	disconnect( this, SIGNAL(listInfo(const QUrlInfo &)),
-		    this, SLOT(npListInfo(const QUrlInfo &)) );
+	disconnect( this, SIGNAL(listInfo(const QUrlInfo&)),
+		    this, SLOT(npListInfo(const QUrlInfo&)) );
 	disconnect( this, SIGNAL(done(bool)),
 		    this, SLOT(npDone(bool)) );
 	disconnect( this, SIGNAL(stateChanged(int)),
