@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qsocketdevice.cpp   3.2.3   edited Aug 12 16:25 $
+** $Id: qt/qsocketdevice.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of QSocketDevice class.
 **
 ** Created : 970521
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the network module of the Qt GUI Toolkit.
 **
@@ -48,8 +48,12 @@
 class QSocketDevicePrivate
 {
 public:
-};
+    QSocketDevicePrivate( QSocketDevice::Protocol p )
+	: protocol(p)
+    { }
 
+    QSocketDevice::Protocol protocol;
+};
 
 
 /*!
@@ -65,15 +69,38 @@ public:
     This class provides a low level API for working with sockets.  Users of
     this class are assumed to have networking experience. For most users the
     QSocket class provides a much easier and high level alternative, but
-    certain thing (like UDP) can't be done with QSocket and if you need a
+    certain things (like UDP) can't be done with QSocket and if you need a
     platform-independent API for those, QSocketDevice is the right choice.
 
     The essential purpose of the class is to provide a QIODevice that
     works on sockets, wrapped in a platform-independent API.
 
+    When calling connect() or bind(), QSocketDevice detects the
+    protocol family (IPv4, IPv6) automatically. Passing the protocol
+    family to QSocketDevice's constructor or to setSocket() forces
+    creation of a socket device of a specific protocol. If not set, the
+    protocol will be detected at the first call to connect() or bind().
+
     \sa QSocket, QSocketNotifier, QHostAddress
 */
 
+
+/*!
+    \enum QSocketDevice::Protocol
+
+    This enum type describes the protocol family of the socket. Possible values
+    are:
+
+    \value IPv4 The socket is an IPv4 socket.
+    \value IPv6 The socket is an IPv6 socket.
+    \value Unknown The protocol family of the socket is not known. This can
+	   happen if you use QSocketDevice with an already existing socket; it
+	   tries to determine the protocol family, but this can fail if the
+	   protocol family is not known to QSocketDevice.
+
+    \sa protocol() setSocket()
+*/
+
 /*!
     \enum QSocketDevice::Error
 
@@ -129,8 +156,8 @@ public:
     connectionless UDP socket.
 */
 QSocketDevice::QSocketDevice( int socket, Type type )
-    : fd( -1 ), t( type ), p( 0 ), pp( 0 ), e( NoError ),
-      d( 0 )
+    : fd( socket ), t( type ), p( 0 ), pp( 0 ), e( NoError ),
+      d(new QSocketDevicePrivate(Unknown))
 {
 #if defined(QSOCKETDEVICE_DEBUG)
     qDebug( "QSocketDevice: Created QSocketDevice %p (socket %x, type %d)",
@@ -147,10 +174,42 @@ QSocketDevice::QSocketDevice( int socket, Type type )
     reliable, connection-oriented TCP socket, or \c
     QSocketDevice::Datagram for an unreliable UDP socket.
 
-    \sa blocking()
+    The socket is created as an IPv4 socket.
+
+    \sa blocking() protocol()
 */
 QSocketDevice::QSocketDevice( Type type )
-    : fd( -1 ), t( type ), p( 0 ), pp( 0 ), e( NoError ), d( 0 )
+    : fd( -1 ), t( type ), p( 0 ), pp( 0 ), e( NoError ),
+      d(new QSocketDevicePrivate(IPv4))
+{
+#if defined(QSOCKETDEVICE_DEBUG)
+    qDebug( "QSocketDevice: Created QSocketDevice object %p, type %d",
+	    this, type );
+#endif
+    init();
+    setSocket( createNewSocket(), type );
+}
+
+/*!
+    Creates a QSocketDevice object for a stream or datagram socket.
+
+    The \a type argument must be either \c QSocketDevice::Stream for a
+    reliable, connection-oriented TCP socket, or \c
+    QSocketDevice::Datagram for an unreliable UDP socket.
+
+    The \a protocol indicates whether the socket should be of type IPv4
+    or IPv6. Passing \c Unknown is not meaningful in this context and you
+    should avoid using (it creates an IPv4 socket, but your code is not easily
+    readable).
+
+    The argument \a dummy is necessary for compatibility with some
+    compilers.
+
+    \sa blocking() protocol()
+*/
+QSocketDevice::QSocketDevice( Type type, Protocol protocol, int )
+    : fd( -1 ), t( type ), p( 0 ), pp( 0 ), e( NoError ),
+      d(new QSocketDevicePrivate(protocol))
 {
 #if defined(QSOCKETDEVICE_DEBUG)
     qDebug( "QSocketDevice: Created QSocketDevice object %p, type %d",
@@ -181,7 +240,7 @@ QSocketDevice::~QSocketDevice()
 */
 bool QSocketDevice::isValid() const
 {
-    return socket() != -1;
+    return fd != -1;
 }
 
 
@@ -198,6 +257,24 @@ QSocketDevice::Type QSocketDevice::type() const
     return t;
 }
 
+/*!
+    Returns the socket's protocol family, which is one of \c Unknown, \c IPv4,
+    or \c IPv6.
+
+    QSocketDevice either creates a socket with a well known protocol family or
+    it uses an already existing socket. In the first case, this function
+    returns the protocol family it was constructed with. In the second case, it
+    tries to determine the protocol family of the socket; if this fails, it
+    returns \c Unknown.
+
+    \sa Protocol setSocket()
+*/
+QSocketDevice::Protocol QSocketDevice::protocol() const
+{
+    if ( d->protocol == Unknown )
+	d->protocol = getProtocol();
+    return d->protocol;
+}
 
 /*!
     Returns the socket number, or -1 if it is an invalid socket.
@@ -232,6 +309,7 @@ void QSocketDevice::setSocket( int socket, Type type )
 #endif
     t = type;
     fd = socket;
+    d->protocol = Unknown;
     e = NoError;
     setFlags( IO_Sequential );
     resetStatus();
