@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qsocketdevice_unix.cpp   3.2.1   edited Jun 19 13:15 $
+** $Id: qt/qsocketdevice_unix.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of QSocketDevice class.
 **
 ** Created : 970521
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the network module of the Qt GUI Toolkit.
 **
@@ -84,6 +84,29 @@ static inline int qt_socket_socket(int domain, int type, int protocol)
 #include "qwindowdefs.h"
 
 #include <errno.h>
+#include <sys/types.h>
+
+
+static inline void qt_socket_getportaddr( struct sockaddr *sa,
+					  Q_UINT16 *port, QHostAddress *addr )
+{
+#if !defined(QT_NO_IPV6)
+    if ( sa->sa_family == AF_INET6 ) {
+	struct sockaddr_in6 *sa6 = ( struct sockaddr_in6 * )sa;
+	Q_IPV6ADDR tmp;
+	memcpy( &tmp, &sa6->sin6_addr.s6_addr, sizeof(tmp) );
+	QHostAddress a( tmp );
+	*addr = a;
+	*port = ntohs( sa6->sin6_port );
+	return;
+    }
+#endif
+    struct sockaddr_in *sa4 = (struct sockaddr_in *)sa;
+    QHostAddress a( ntohl( sa4->sin_addr.s_addr ) );
+    *port = ntohs( sa4->sin_port );
+    *addr = QHostAddress( ntohl( sa4->sin_addr.s_addr ) );
+    return;
+}
 
 
 //#define QSOCKETDEVICE_DEBUG
@@ -93,6 +116,43 @@ void QSocketDevice::init()
 {
 }
 
+
+QSocketDevice::Protocol QSocketDevice::getProtocol() const
+{
+    if ( isValid() ) {
+#if !defined (QT_NO_IPV6)
+	struct sockaddr_storage sa;
+#else
+	struct sockaddr sa;
+#endif
+	memset( &sa, 0, sizeof(sa) );
+	QT_SOCKLEN_T sz = sizeof( sa );
+#if !defined (QT_NO_IPV6)
+	struct sockaddr *sap = reinterpret_cast<struct sockaddr *>(&sa);
+	if ( !::getsockname(fd, sap, &sz) ) {
+	    switch ( sap->sa_family ) {
+		case AF_INET:
+		    return IPv4;
+		case AF_INET6:
+		    return IPv6;
+		default:
+		    return Unknown;
+	    }
+	}
+#else
+	if ( !::getsockname(fd, &sa, &sz) ) {
+	    switch ( sa.sa_family ) {
+		case AF_INET:
+		    return IPv4;
+		default:
+		    return Unknown;
+	    }
+	}
+#endif
+    }
+    return Unknown;
+}
+
 /*!
     Creates a new socket identifier. Returns -1 if there is a failure
     to create the new identifier; error() explains why.
@@ -100,9 +160,14 @@ void QSocketDevice::init()
     \sa setSocket()
 */
 
-int QSocketDevice::createNewSocket ()
+int QSocketDevice::createNewSocket()
 {
+#if !defined(QT_NO_IPV6)
+    int s = qt_socket_socket( protocol() == IPv6 ? AF_INET6 : AF_INET,
+			      t == Datagram ? SOCK_DGRAM : SOCK_STREAM, 0 );
+#else
     int s = qt_socket_socket( AF_INET, t==Datagram?SOCK_DGRAM:SOCK_STREAM, 0 );
+#endif
     if ( s < 0 ) {
 	switch( errno ) {
 	case EPROTONOSUPPORT:
@@ -126,6 +191,9 @@ int QSocketDevice::createNewSocket ()
 	    break;
 	}
     } else {
+	// ensure that the socket is closed on exec..() after being dup()'ed by
+	// fork() in QProcess.
+	::fcntl(s, F_SETFD, FD_CLOEXEC);
 	return s;
     }
     return -1;
@@ -333,23 +401,44 @@ void QSocketDevice::setOption( Option opt, int v )
 */
 bool QSocketDevice::connect( const QHostAddress &addr, Q_UINT16 port )
 {
-    if ( !isValid() ) {
+    if ( !isValid() )
 	return FALSE;
-    }
 
-    if ( !addr.isIp4Addr() ) {
-	qWarning( "QSocketDevice: IPv6 is not supported by this version" );
+    pa = addr;
+    pp = port;
+
+    struct sockaddr_in a4;
+    struct sockaddr *aa;
+    QT_SOCKLEN_T aalen;
+
+#if !defined(QT_NO_IPV6)
+    struct sockaddr_in6 a6;
+
+    if ( addr.isIPv6Address() ) {
+	memset( &a6, 0, sizeof(a6) );
+	a6.sin6_family = AF_INET6;
+	a6.sin6_port = htons( port );
+	Q_IPV6ADDR ip6 = addr.toIPv6Address();
+	memcpy( &a6.sin6_addr.s6_addr, &ip6, sizeof(ip6) );
+
+	aalen = sizeof( a6 );
+	aa = (struct sockaddr *)&a6;
+    } else
+#endif
+    if ( addr.isIPv4Address() ) {
+	memset( &a4, 0, sizeof(a4) );
+	a4.sin_family = AF_INET;
+	a4.sin_port = htons( port );
+	a4.sin_addr.s_addr = htonl( addr.toIPv4Address() );
+
+	aalen = sizeof(a4);
+	aa = (struct sockaddr *)&a4;
+    } else {
 	e = Impossible;
 	return FALSE;
     }
-    struct sockaddr_in aa;
-    memset( &aa, 0, sizeof(aa) );
-    aa.sin_family = AF_INET;
-    aa.sin_port = htons( port );
-    aa.sin_addr.s_addr = htonl( addr.ip4Addr() );
 
-    int r = qt_socket_connect( fd, (struct sockaddr*)&aa,
-		       sizeof(struct sockaddr_in) );
+    int r = qt_socket_connect( fd, aa, aalen );
     if ( r == 0 ) {
 	fetchConnectionParameters();
 	return TRUE;
@@ -405,19 +494,33 @@ bool QSocketDevice::bind( const QHostAddress &address, Q_UINT16 port )
 {
     if ( !isValid() )
 	return FALSE;
+    int r;
+    struct sockaddr_in a4;
+#if !defined(QT_NO_IPV6)
+    struct sockaddr_in6 a6;
 
-    if ( !address.isIp4Addr() ) {
-	qWarning( "QSocketDevice: IPv6 is not supported by this version" );
+    if ( address.isIPv6Address() ) {
+	memset( &a6, 0, sizeof(a6) );
+	a6.sin6_family = AF_INET6;
+	a6.sin6_port = htons( port );
+	Q_IPV6ADDR tmp = address.toIPv6Address();
+	memcpy( &a6.sin6_addr.s6_addr, &tmp, sizeof(tmp) );
+
+	r = qt_socket_bind( fd, (struct sockaddr *)&a6, sizeof(a6) );
+    } else
+#endif
+    if ( address.isIPv4Address() ) {
+	memset( &a4, 0, sizeof(a4) );
+	a4.sin_family = AF_INET;
+	a4.sin_port = htons( port );
+	a4.sin_addr.s_addr = htonl( address.toIPv4Address() );
+
+	r = qt_socket_bind( fd, (struct sockaddr*)&a4, sizeof(a4) );
+    } else {
 	e = Impossible;
 	return FALSE;
     }
-    struct sockaddr_in aa;
-    memset( &aa, 0, sizeof(aa) );
-    aa.sin_family = AF_INET;
-    aa.sin_port = htons( port );
-    aa.sin_addr.s_addr = htonl( address.ip4Addr() );
 
-    int r = qt_socket_bind( fd, (struct sockaddr*)&aa, sizeof(struct sockaddr_in) );
     if ( r < 0 ) {
 	switch( errno ) {
 	case EINVAL:
@@ -485,55 +588,71 @@ bool QSocketDevice::listen( int backlog )
 int QSocketDevice::accept()
 {
     if ( !isValid() )
-	return FALSE;
+	return -1;
+
+#if !defined (QT_NO_IPV6)
+    struct sockaddr_storage aa;
+#else
     struct sockaddr aa;
-    QT_SOCKLEN_T l = sizeof(struct sockaddr);
-    int s = qt_socket_accept( fd, (struct sockaddr*)&aa, &l );
-    // we'll blithely throw away the stuff accept() wrote to aa
-    if ( s < 0 && e == NoError ) {
-	switch( errno ) {
+#endif
+    QT_SOCKLEN_T l = sizeof( aa );
+    bool done;
+    int s;
+    do {
+        s = qt_socket_accept( fd, (struct sockaddr*)&aa, &l );
+        // we'll blithely throw away the stuff accept() wrote to aa
+        done = TRUE;
+        if ( s < 0 && e == NoError ) {
+            switch( errno ) {
+            case EINTR:
+                done = FALSE;
+                break;
 #if defined(EPROTO)
-	case EPROTO:
+	    case EPROTO:
 #endif
 #if defined(ENONET)
-	case ENONET:
+	    case ENONET:
 #endif
-	case ENOPROTOOPT:
-	case EHOSTDOWN:
-	case EOPNOTSUPP:
-	case EHOSTUNREACH:
-	case ENETDOWN:
-	case ENETUNREACH:
-	case ETIMEDOUT:
-	    // in all these cases, an error happened during connection
-	    // setup.  we're not interested in what happened, so we
-	    // just treat it like the client-closed-quickly case.
-	case EPERM:
-	    // firewalling wouldn't let us accept.  we treat it like
-	    // the client-closed-quickly case.
-	case EAGAIN:
+	    case ENOPROTOOPT:
+	    case EHOSTDOWN:
+	    case EOPNOTSUPP:
+	    case EHOSTUNREACH:
+	    case ENETDOWN:
+	    case ENETUNREACH:
+	    case ETIMEDOUT:
+		// in all these cases, an error happened during connection
+		// setup.  we're not interested in what happened, so we
+		// just treat it like the client-closed-quickly case.
+	    case EPERM:
+		// firewalling wouldn't let us accept.  we treat it like
+		// the client-closed-quickly case.
+	    case EAGAIN:
 #if EAGAIN != EWOULDBLOCK
-	case EWOULDBLOCK:
+	    case EWOULDBLOCK:
 #endif
-	    // the client closed the connection before we got around
-	    // to accept()ing it.
-	    break;
-	case EBADF:
-	case ENOTSOCK:
-	    e = Impossible;
-	    break;
-	case EFAULT:
-	    e = InternalError;
-	    break;
-	case ENOMEM:
-	case ENOBUFS:
-	    e = NoResources;
-	    break;
-	default:
-	    e = UnknownError;
-	    break;
+		// the client closed the connection before we got around
+		// to accept()ing it.
+		break;
+	    case EBADF:
+	    case ENOTSOCK:
+		e = Impossible;
+		break;
+	    case EFAULT:
+		e = InternalError;
+		break;
+	    case ENOMEM:
+	    case ENOBUFS:
+		e = NoResources;
+		break;
+	    default:
+		e = UnknownError;
+		break;
+	    }
 	}
-    }
+    } while (!done);
+    // ensure that the socket is closed on exec..() after being dup()'ed by
+    // fork() in QProcess.
+    ::fcntl(s, F_SETFD, FD_CLOEXEC);
     return s;
 }
 
@@ -599,6 +718,8 @@ Q_LONG QSocketDevice::waitForMore( int msecs, bool *timeout ) const
 {
     if ( !isValid() )
 	return -1;
+    if ( fd >= FD_SETSIZE )
+	return -1;
 
     fd_set fds;
     struct timeval tv;
@@ -627,7 +748,10 @@ Q_LONG QSocketDevice::waitForMore( int msecs, bool *timeout ) const
 
 /*!
     Reads \a maxlen bytes from the socket into \a data and returns the
-    number of bytes read. Returns -1 if an error occurred.
+    number of bytes read. Returns -1 if an error occurred. Returning 0
+    is not an error. For Stream sockets, 0 is returned when the remote
+    host closes the connection. For Datagram sockets, 0 is a valid
+    datagram size.
 */
 Q_LONG QSocketDevice::readBlock( char *data, Q_ULONG maxlen )
 {
@@ -654,19 +778,27 @@ Q_LONG QSocketDevice::readBlock( char *data, Q_ULONG maxlen )
     int r = 0;
     while ( done == FALSE ) {
 	if ( t == Datagram ) {
-	    struct sockaddr_in aa;
+#if !defined(QT_NO_IPV6)
+            struct sockaddr_storage aa;
+#else
+            struct sockaddr_in aa;
+#endif
 	    memset( &aa, 0, sizeof(aa) );
 	    QT_SOCKLEN_T sz;
 	    sz = sizeof( aa );
 	    r = ::recvfrom( fd, data, maxlen, 0,
 			    (struct sockaddr *)&aa, &sz );
-	    pp = ntohs( aa.sin_port );
-	    pa = QHostAddress( ntohl( aa.sin_addr.s_addr ) );
+
+	    qt_socket_getportaddr( (struct sockaddr *)&aa, &pp, &pa);
+
 	} else {
 	    r = ::read( fd, data, maxlen );
 	}
 	done = TRUE;
-	if ( r >= 0 || errno == EAGAIN || errno == EWOULDBLOCK ) {
+	if ( r == 0 && t == Stream && maxlen > 0 ) {
+	    // connection closed
+	    close();
+	} else if ( r >= 0 || errno == EAGAIN || errno == EWOULDBLOCK ) {
 	    // nothing
 	} else if ( errno == EINTR ) {
 	    done = FALSE;
@@ -751,6 +883,7 @@ Q_LONG QSocketDevice::writeBlock( const char *data, Q_ULONG len )
 		done = FALSE;
 		break;
 	    case EPIPE:
+            case ECONNRESET:
 		// connection closed
 		close();
 		r = 0;
@@ -832,24 +965,40 @@ Q_LONG QSocketDevice::writeBlock( const char * data, Q_ULONG len,
 #endif
 	return -1;
     }
-    if ( !host.isIp4Addr() ) {
-	qWarning( "QSocketDevice: IPv6 is not supported by this version" );
+    struct sockaddr_in a4;
+    struct sockaddr *aa;
+    QT_SOCKLEN_T slen;
+#if !defined(QT_NO_IPV6)
+    struct sockaddr_in6 a6;
+    if ( host.isIPv6Address() ) {
+	memset( &a6, 0, sizeof(a6) );
+	a6.sin6_family = AF_INET6;
+	a6.sin6_port = htons( port );
+
+	Q_IPV6ADDR tmp = host.toIPv6Address();
+	memcpy( &a6.sin6_addr.s6_addr, &tmp, sizeof(tmp) );
+	slen = sizeof( a6 );
+	aa = (struct sockaddr *)&a6;
+    } else
+#endif
+    if ( host.isIPv4Address() ) {
+	memset( &a4, 0, sizeof(a4) );
+	a4.sin_family = AF_INET;
+	a4.sin_port = htons( port );
+	a4.sin_addr.s_addr = htonl( host.toIPv4Address() );
+	slen = sizeof(a4);
+	aa = (struct sockaddr *)&a4;
+    } else {
 	e = Impossible;
 	return -1;
     }
-    struct sockaddr_in aa;
-    memset( &aa, 0, sizeof(aa) );
-    aa.sin_family = AF_INET;
-    aa.sin_port = htons( port );
-    aa.sin_addr.s_addr = htonl( host.ip4Addr() );
 
     // we'd use MSG_DONTWAIT + MSG_NOSIGNAL if Stevens were right.
     // but apparently Stevens and most implementors disagree
     bool done = FALSE;
     int r = 0;
     while ( !done ) {
-	r = ::sendto( fd, data, len, 0,
-		      (struct sockaddr *)(&aa), sizeof(sockaddr_in) );
+	r = ::sendto( fd, data, len, 0, aa, slen);
 	done = TRUE;
 	if ( r < 0 && e == NoError &&
 	     errno != EAGAIN && errno != EWOULDBLOCK ) {
@@ -900,19 +1049,20 @@ void QSocketDevice::fetchConnectionParameters()
 	pa = QHostAddress();
 	return;
     }
+#if !defined(QT_NO_IPV6)
+    struct sockaddr_storage sa;
+#else
     struct sockaddr_in sa;
+#endif
     memset( &sa, 0, sizeof(sa) );
     QT_SOCKLEN_T sz;
     sz = sizeof( sa );
-    if ( !::getsockname( fd, (struct sockaddr *)(&sa), &sz ) ) {
-	p = ntohs( sa.sin_port );
-	a = QHostAddress( ntohl( sa.sin_addr.s_addr ) );
-    }
+    if ( !::getsockname( fd, (struct sockaddr *)(&sa), &sz ) )
+	qt_socket_getportaddr( (struct sockaddr *)&sa, &p, &a );
+
     sz = sizeof( sa );
-    if ( !::getpeername( fd, (struct sockaddr *)(&sa), &sz ) ) {
-	pp = ntohs( sa.sin_port );
-	pa = QHostAddress( ntohl( sa.sin_addr.s_addr ) );
-    }
+    if ( !::getpeername( fd, (struct sockaddr *)(&sa), &sz ) )
+	qt_socket_getportaddr( (struct sockaddr *)&sa, &pp, &pa );
 }
 
 
