@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qgl_x11.cpp   3.2.3   edited Jun 16 09:02 $
+** $Id: qt/qgl_x11.cpp   3.3.7   edited Sep 5 2005 $
 **
 ** Implementation of OpenGL classes for Qt
 **
 ** Created : 970112
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the opengl module of the Qt GUI Toolkit.
 **
@@ -39,10 +39,13 @@
 
 #if defined(Q_WS_X11)
 
+#include "qmap.h"
 #include "qpixmap.h"
 #include "qapplication.h"
 
 #include "qintdict.h"
+#include "private/qfontengine_p.h"
+
 #define INT8  dummy_INT8
 #define INT32 dummy_INT32
 #include <GL/glx.h>
@@ -52,13 +55,134 @@
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
 #include <X11/Xatom.h>
-#include <X11/Xmu/StdCmap.h>
 
 // POSIX Large File Support redefines truncate -> truncate64
 #if defined(truncate)
 # undef truncate
 #endif
 
+#ifndef QT_DLOPEN_OPENGL
+extern "C" {
+    Status XmuLookupStandardColormap( Display *dpy, int screen, VisualID visualid,
+				      unsigned int depth, Atom property,
+				      Bool replace, Bool retain );
+}
+#endif
+
+#include "qgl_x11_p.h"
+#ifdef QT_DLOPEN_OPENGL
+#include "qlibrary.h"
+
+extern "C" {
+_glCallLists qt_glCallLists;
+_glClearColor qt_glClearColor;
+_glClearIndex qt_glClearIndex;
+_glColor3ub qt_glColor3ub;
+_glDeleteLists qt_glDeleteLists;
+_glDrawBuffer qt_glDrawBuffer;
+_glFlush qt_glFlush;
+_glIndexi qt_glIndexi;
+_glListBase qt_glListBase;
+_glLoadIdentity qt_glLoadIdentity;
+_glMatrixMode qt_glMatrixMode;
+_glOrtho qt_glOrtho;
+_glPopAttrib qt_glPopAttrib;
+_glPopMatrix qt_glPopMatrix;
+_glPushAttrib qt_glPushAttrib;
+_glPushMatrix qt_glPushMatrix;
+_glRasterPos2i qt_glRasterPos2i;
+_glRasterPos3d qt_glRasterPos3d;
+_glReadPixels qt_glReadPixels;
+_glViewport qt_glViewport;
+_glPixelStorei qt_glPixelStorei;
+_glBitmap qt_glBitmap;
+_glDrawPixels qt_glDrawPixels;
+_glNewList qt_glNewList;
+_glGetFloatv qt_glGetFloatv;
+_glGetIntegerv qt_glGetIntegerv;
+_glEndList qt_glEndList;
+
+_glXChooseVisual qt_glXChooseVisual;
+_glXCreateContext qt_glXCreateContext;
+_glXCreateGLXPixmap qt_glXCreateGLXPixmap;
+_glXDestroyContext qt_glXDestroyContext;
+_glXDestroyGLXPixmap qt_glXDestroyGLXPixmap;
+_glXGetClientString qt_glXGetClientString;
+_glXGetConfig qt_glXGetConfig;
+_glXIsDirect qt_glXIsDirect;
+_glXMakeCurrent qt_glXMakeCurrent;
+_glXQueryExtension qt_glXQueryExtension;
+_glXQueryExtensionsString qt_glXQueryExtensionsString;
+_glXQueryServerString qt_glXQueryServerString;
+_glXSwapBuffers qt_glXSwapBuffers;
+_glXUseXFont qt_glXUseXFont;
+_glXWaitX qt_glXWaitX;
+};
+
+bool qt_resolve_gl_symbols(bool fatal)
+{
+    static bool gl_syms_resolved = FALSE;
+    if (gl_syms_resolved)
+	return TRUE;
+
+    QLibrary gl("GL");
+    gl.setAutoUnload(FALSE);
+
+    qt_glCallLists = (_glCallLists) gl.resolve("glCallLists");
+
+    if (!qt_glCallLists) { // if this fails the rest will surely fail
+	if (fatal)
+	    qFatal("Unable to resolve GL/GLX symbols - please check your GL library installation.");
+	return FALSE;
+    }
+
+    qt_glClearColor = (_glClearColor) gl.resolve("glClearColor");
+    qt_glClearIndex = (_glClearIndex) gl.resolve("glClearIndex");
+    qt_glColor3ub = (_glColor3ub) gl.resolve("glColor3ub");
+    qt_glDeleteLists = (_glDeleteLists) gl.resolve("glDeleteLists");
+    qt_glDrawBuffer = (_glDrawBuffer) gl.resolve("glDrawBuffer");
+    qt_glFlush = (_glFlush) gl.resolve("glFlush");
+    qt_glIndexi = (_glIndexi) gl.resolve("glIndexi");
+    qt_glListBase = (_glListBase) gl.resolve("glListBase");
+    qt_glLoadIdentity = (_glLoadIdentity) gl.resolve("glLoadIdentity");
+    qt_glMatrixMode = (_glMatrixMode) gl.resolve("glMatrixMode");
+    qt_glOrtho = (_glOrtho) gl.resolve("glOrtho");
+    qt_glPopAttrib = (_glPopAttrib) gl.resolve("glPopAttrib");
+    qt_glPopMatrix = (_glPopMatrix) gl.resolve("glPopMatrix");
+    qt_glPushAttrib = (_glPushAttrib) gl.resolve("glPushAttrib");
+    qt_glPushMatrix = (_glPushMatrix) gl.resolve("glPushMatrix");
+    qt_glRasterPos2i = (_glRasterPos2i) gl.resolve("glRasterPos2i");
+    qt_glRasterPos3d = (_glRasterPos3d) gl.resolve("glRasterPos3d");
+    qt_glReadPixels = (_glReadPixels) gl.resolve("glReadPixels");
+    qt_glViewport = (_glViewport) gl.resolve("glViewport");
+    qt_glPixelStorei = (_glPixelStorei) gl.resolve("glPixelStorei");
+    qt_glBitmap = (_glBitmap) gl.resolve("glBitmap");
+    qt_glDrawPixels = (_glDrawPixels) gl.resolve("glDrawPixels");
+    qt_glNewList = (_glNewList) gl.resolve("glNewList");
+    qt_glGetFloatv = (_glGetFloatv) gl.resolve("glGetFloatv");
+    qt_glGetIntegerv = (_glGetIntegerv) gl.resolve("glGetIntegerv");
+    qt_glEndList = (_glEndList) gl.resolve("glEndList");
+
+    qt_glXChooseVisual = (_glXChooseVisual) gl.resolve("glXChooseVisual");
+    qt_glXCreateContext = (_glXCreateContext) gl.resolve("glXCreateContext");
+    qt_glXCreateGLXPixmap = (_glXCreateGLXPixmap) gl.resolve("glXCreateGLXPixmap");
+    qt_glXDestroyContext = (_glXDestroyContext) gl.resolve("glXDestroyContext");
+    qt_glXDestroyGLXPixmap = (_glXDestroyGLXPixmap) gl.resolve("glXDestroyGLXPixmap");
+    qt_glXGetClientString = (_glXGetClientString) gl.resolve("glXGetClientString");
+    qt_glXGetConfig = (_glXGetConfig) gl.resolve("glXGetConfig");
+    qt_glXIsDirect = (_glXIsDirect) gl.resolve("glXIsDirect");
+    qt_glXMakeCurrent = (_glXMakeCurrent) gl.resolve("glXMakeCurrent");
+    qt_glXQueryExtension = (_glXQueryExtension) gl.resolve("glXQueryExtension");
+    qt_glXQueryExtensionsString = (_glXQueryExtensionsString) gl.resolve("glXQueryExtensionsString");
+    qt_glXQueryServerString = (_glXQueryServerString) gl.resolve("glXQueryServerString");
+    qt_glXSwapBuffers = (_glXSwapBuffers) gl.resolve("glXSwapBuffers");
+    qt_glXUseXFont = (_glXUseXFont) gl.resolve("glXUseXFont");
+    qt_glXWaitX = (_glXWaitX) gl.resolve("glXWaitX");
+    gl_syms_resolved = TRUE;
+    return TRUE;
+}
+#endif // QT_DLOPEN_OPENGL
+
 
 /*
   The choose_cmap function is internal and used by QGLWidget::setContext()
@@ -91,14 +215,20 @@ CMapEntry::~CMapEntry()
 
 static QIntDict<CMapEntry> *cmap_dict = 0;
 static bool		    mesa_gl   = FALSE;
+static QIntDict< QMap<int, QRgb> > *qglcmap_dict = 0;
 
 static void cleanup_cmaps()
 {
-    if ( !cmap_dict )
-	return;
-    cmap_dict->setAutoDelete( TRUE );
-    delete cmap_dict;
-    cmap_dict = 0;
+    if (cmap_dict) {
+	cmap_dict->setAutoDelete(TRUE);
+	delete cmap_dict;
+	cmap_dict = 0;
+    }
+    if (qglcmap_dict) {
+	qglcmap_dict->setAutoDelete(TRUE);
+	delete qglcmap_dict;
+	qglcmap_dict = 0;
+    }
 }
 
 static Colormap choose_cmap( Display *dpy, XVisualInfo *vi )
@@ -149,8 +279,19 @@ static Colormap choose_cmap( Display *dpy, XVisualInfo *vi )
     }
 #if !defined(Q_OS_SOLARIS)
     if ( !x->cmap ) {
+#ifdef QT_DLOPEN_OPENGL
+	typedef Status (*_XmuLookupStandardColormap)( Display *dpy, int screen, VisualID visualid, unsigned int depth,
+						      Atom property, Bool replace, Bool retain );
+	_XmuLookupStandardColormap qt_XmuLookupStandardColormap;
+	qt_XmuLookupStandardColormap = (_XmuLookupStandardColormap) QLibrary::resolve("Xmu", "XmuLookupStandardColormap");
+	if (!qt_XmuLookupStandardColormap)
+	    qFatal("Unable to resolve Xmu symbols - please check your Xmu library installation.");
+#define XmuLookupStandardColormap qt_XmuLookupStandardColormap
+
+#endif
+
 	if ( XmuLookupStandardColormap(dpy,vi->screen,vi->visualid,vi->depth,
-				       XA_RGB_DEFAULT_MAP,FALSE,TRUE) ) {
+					  XA_RGB_DEFAULT_MAP,FALSE,TRUE) ) {
 	    if ( XGetRGBColormaps(dpy,RootWindow(dpy,vi->screen),&c,&n,
 				  XA_RGB_DEFAULT_MAP) ) {
 		i = 0;
@@ -253,12 +394,15 @@ static void find_trans_colors()
 
 bool QGLFormat::hasOpenGL()
 {
+    if (!qt_resolve_gl_symbols(FALSE))
+	return FALSE;
     return glXQueryExtension(qt_xdisplay(),0,0) != 0;
 }
 
 
 bool QGLFormat::hasOpenGLOverlays()
 {
+    qt_resolve_gl_symbols();
     if ( !trans_colors_init )
 	find_trans_colors();
     return trans_colors.size() > 0;
@@ -609,8 +753,16 @@ QColor QGLContext::overlayTransparentColor() const
 		 trans_colors[i].screen == myScreen ) {
 		XColor col;
 		col.pixel = trans_colors[i].color;
-		XQueryColor( d->paintDevice->x11Display(), d->paintDevice->x11Colormap(), &col );
-		return QColor( qRgb( col.red, col.green, col.blue ), trans_colors[i].color );
+		col.red = col.green = col.blue = 0;
+		col.flags = 0;
+		Display *dpy = d->paintDevice->x11Display();
+		if (col.pixel > (uint) ((XVisualInfo *)vi)->colormap_size - 1)
+		    col.pixel = ((XVisualInfo *)vi)->colormap_size - 1;
+		XQueryColor(dpy, choose_cmap(dpy, (XVisualInfo *) vi), &col);
+		uchar r = (uchar)((col.red / 65535.0) * 255.0 + 0.5);
+		uchar g = (uchar)((col.green / 65535.0) * 255.0 + 0.5);
+		uchar b = (uchar)((col.blue / 65535.0) * 255.0 + 0.5);
+		return QColor(qRgb(r,g,b), trans_colors[i].color);
 	    }
 	}
     }
@@ -640,19 +792,177 @@ uint QGLContext::colorIndex( const QColor& c ) const
 		     + ( gf * x->scmap.green_mult )
 		     + ( bf * x->scmap.blue_mult );
 	    return p;
-	}
-	else {
-	    return c.pixel( screen ); // ### wrong; should really ask QColor to alloc
+	} else {
+	    if (!qglcmap_dict) {
+		qglcmap_dict = new QIntDict< QMap<int, QRgb> >;
+	    }
+	    QMap<int, QRgb> *cmap;
+	    if ((cmap = qglcmap_dict->find((long) info->visualid)) == 0) {
+		cmap = new QMap<int, QRgb>;
+		qglcmap_dict->insert((long) info->visualid, cmap);
+	    }
+
+	    // already in the map?
+	    QRgb target = c.rgb();
+	    QMap<int, QRgb>::Iterator it = cmap->begin();
+	    for (; it != cmap->end(); ++it) {
+		if ((*it) == target)
+		    return it.key();
+	    }
+
+	    // need to alloc color
+	    unsigned long plane_mask[2];
+	    unsigned long color_map_entry;
+	    if (!XAllocColorCells (QPaintDevice::x11AppDisplay(), x->cmap, TRUE, plane_mask, 0,
+				   &color_map_entry, 1))
+		return c.pixel(screen);
+
+	    XColor col;
+	    col.flags = DoRed | DoGreen | DoBlue;
+	    col.pixel = color_map_entry;
+	    col.red   = (ushort)((qRed(c.rgb()) / 255.0) * 65535.0 + 0.5);
+	    col.green = (ushort)((qGreen(c.rgb()) / 255.0) * 65535.0 + 0.5);
+	    col.blue  = (ushort)((qBlue(c.rgb()) / 255.0) * 65535.0 + 0.5);
+	    XStoreColor(QPaintDevice::x11AppDisplay(), x->cmap, &col);
+
+	    cmap->insert(color_map_entry, target);
+	    return color_map_entry;
 	}
     }
     return 0;
 }
 
+#ifndef QT_NO_XFTFREETYPE
+/*! \internal
+    This is basically a substitute for glxUseXFont() which can only
+    handle XLFD fonts. This version relies on XFT v2 to render the
+    glyphs, but it works with all fonts that XFT2 provides - both
+    antialiased and aliased bitmap and outline fonts.
+*/
+void qgl_use_font(QFontEngineXft *engine, int first, int count, int listBase)
+{
+    GLfloat color[4];
+    glGetFloatv(GL_CURRENT_COLOR, color);
+
+    // save the pixel unpack state
+    GLint gl_swapbytes, gl_lsbfirst, gl_rowlength, gl_skiprows, gl_skippixels, gl_alignment;
+    glGetIntegerv (GL_UNPACK_SWAP_BYTES, &gl_swapbytes);
+    glGetIntegerv (GL_UNPACK_LSB_FIRST, &gl_lsbfirst);
+    glGetIntegerv (GL_UNPACK_ROW_LENGTH, &gl_rowlength);
+    glGetIntegerv (GL_UNPACK_SKIP_ROWS, &gl_skiprows);
+    glGetIntegerv (GL_UNPACK_SKIP_PIXELS, &gl_skippixels);
+    glGetIntegerv (GL_UNPACK_ALIGNMENT, &gl_alignment);
+
+    glPixelStorei(GL_UNPACK_SWAP_BYTES, GL_FALSE);
+    glPixelStorei(GL_UNPACK_LSB_FIRST, GL_FALSE);
+    glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
+    glPixelStorei(GL_UNPACK_SKIP_ROWS, 0);
+    glPixelStorei(GL_UNPACK_SKIP_PIXELS, 0);
+    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
+
+    Bool antialiased = False;
+#if 0 // disable antialias support for now
+    XftPatternGetBool(engine->pattern(), XFT_ANTIALIAS, 0, &antialiased);
+#endif
+#ifdef QT_XFT2
+    FT_Face face = XftLockFace(engine->font());
+#else
+    FT_Face face = engine->face();
+#endif
+    // start generating font glyphs
+    for (int i = first; i < count; ++i) {
+	int list = listBase + i;
+	GLfloat x0, y0, dx, dy;
+
+	FT_Error err;
+
+	err = FT_Load_Glyph(face, FT_Get_Char_Index(face, i), FT_LOAD_DEFAULT);
+	if (err) {
+	    qDebug("failed loading glyph %d from font", i);
+	    Q_ASSERT(!err);
+	}
+	err = FT_Render_Glyph(face->glyph, (antialiased ? ft_render_mode_normal
+					    : ft_render_mode_mono));
+	if (err) {
+	    qDebug("failed rendering glyph %d from font", i);
+	    Q_ASSERT(!err);
+	}
+
+	FT_Bitmap bm = face->glyph->bitmap;
+	x0 = face->glyph->metrics.horiBearingX >> 6;
+	y0 = (face->glyph->metrics.height - face->glyph->metrics.horiBearingY) >> 6;
+	dx = face->glyph->metrics.horiAdvance >> 6;
+	dy = 0;
+ 	int sz = bm.pitch * bm.rows;
+	uint *aa_glyph = 0;
+	uchar *ua_glyph = 0;
+
+	if (antialiased)
+	    aa_glyph = new uint[sz];
+	else
+	    ua_glyph = new uchar[sz];
+
+	// convert to GL format
+	for (int y = 0; y < bm.rows; ++y) {
+	    for (int x = 0; x < bm.pitch; ++x) {
+		int c1 = y*bm.pitch + x;
+		int c2 = (bm.rows - y - 1) > 0 ? (bm.rows-y-1)*bm.pitch + x : x;
+		if (antialiased) {
+		    aa_glyph[c1] = (int(color[0]*255) << 24)
+					 | (int(color[1]*255) << 16)
+					 | (int(color[2]*255) << 8) | bm.buffer[c2];
+		} else {
+		    ua_glyph[c1] = bm.buffer[c2];
+		}
+	    }
+	}
+
+	glNewList(list, GL_COMPILE);
+	if (antialiased) {
+	    // calling glBitmap() is just a trick to move the current
+	    // raster pos, since glGet*() won't work in display lists
+	    glBitmap(0, 0, 0, 0, x0, -y0, 0);
+	    glDrawPixels(bm.pitch, bm.rows, GL_RGBA, GL_UNSIGNED_INT_8_8_8_8, aa_glyph);
+	    glBitmap(0, 0, 0, 0, dx-x0, y0, 0);
+	} else {
+	    glBitmap(bm.pitch*8, bm.rows, -x0, y0, dx, dy, ua_glyph);
+	}
+	glEndList();
+	antialiased ? delete[] aa_glyph : delete[] ua_glyph;
+    }
+
+#ifdef QT_XFT2
+    XftUnlockFace(engine->font());
+#endif
+
+    // restore pixel unpack settings
+    glPixelStorei(GL_UNPACK_SWAP_BYTES, gl_swapbytes);
+    glPixelStorei(GL_UNPACK_LSB_FIRST, gl_lsbfirst);
+    glPixelStorei(GL_UNPACK_ROW_LENGTH, gl_rowlength);
+    glPixelStorei(GL_UNPACK_SKIP_ROWS, gl_skiprows);
+    glPixelStorei(GL_UNPACK_SKIP_PIXELS, gl_skippixels);
+    glPixelStorei(GL_UNPACK_ALIGNMENT, gl_alignment);
+}
+#endif
+
 void QGLContext::generateFontDisplayLists( const QFont & fnt, int listBase )
 {
-    QFont f = fnt;
-    f.setStyleStrategy( QFont::OpenGLCompatible );
-    glXUseXFont( (Font) f.handle(), 0, 256, listBase );
+    QFont f(fnt);
+    QFontEngine *engine = f.d->engineForScript(QFont::Latin);
+
+#ifndef QT_NO_XFTFREETYPE
+    if(engine->type() == QFontEngine::Xft) {
+	qgl_use_font((QFontEngineXft *) engine, 0, 256, listBase);
+	return;
+    }
+#endif
+    // glXUseXFont() only works with XLFD font structures and a few GL
+    // drivers crash if 0 is passed as the font handle
+    f.setStyleStrategy(QFont::OpenGLCompatible);
+    if (f.handle() && (engine->type() == QFontEngine::XLFD
+		       || engine->type() == QFontEngine::LatinXLFD)) {
+ 	glXUseXFont((Font) f.handle(), 0, 256, listBase);
+    }
 }
 
 /*****************************************************************************
@@ -723,6 +1033,8 @@ void QGLOverlayWidget::paintGL()
  *****************************************************************************/
 void QGLWidget::init( QGLContext *context, const QGLWidget *shareWidget )
 {
+    qt_resolve_gl_symbols();
+
     glcx = 0;
     olw = 0;
     autoSwap = TRUE;
@@ -756,6 +1068,8 @@ void QGLWidget::init( QGLContext *context, const QGLWidget *shareWidget )
 void QGLWidget::reparent( QWidget* parent, WFlags f, const QPoint& p,
 			  bool showIt )
 {
+    if (glcx)
+        glcx->doneCurrent();
     QWidget::reparent( parent, f, p, FALSE );
     if ( showIt )
 	show();
@@ -929,7 +1243,7 @@ bool QGLWidget::renderCxPm( QPixmap* pm )
 	return FALSE;
     }
 
-    glDrawBuffer( GL_FRONT_LEFT );
+    glDrawBuffer( GL_FRONT );
     if ( !glcx->initialized() )
 	glInit();
     resizeGL( pm->width(), pm->height() );
