@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qcdestyle.h   3.0.3   edited Oct 12 12:18 $
+** $Id: qt/qcdestyle.h   3.3.7   edited May 27 2003 $
 **
 ** Definition of the CDE-like style class
 **
@@ -43,7 +43,7 @@
 #include "qmotifstyle.h"
 #endif // QT_H
 
-#ifndef QT_NO_STYLE_CDE
+#if !defined(QT_NO_STYLE_CDE) || defined(QT_PLUGIN)
 
 #if defined(QT_PLUGIN)
 #define Q_EXPORT_STYLE_CDE
