@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qsgistyle.cpp   3.2.3   edited May 27 08:45 $
+** $Id: qt/qsgistyle.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of Motif-like style class
 **
 ** Created : 981231
 **
-** Copyright (C) 1998-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1998-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -1387,6 +1387,9 @@ QSize QSGIStyle::sizeFromContents( ContentsType contents,
 #endif
 	    break;
 	}
+    case CT_ComboBox:
+        sz.rwidth() += 30;
+        break;
 
     default:
 	sz = QMotifStyle::sizeFromContents( contents, widget, contentsSize, opt );
