@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id: qt/qstylefactory.cpp   3.3.5   edited Aug 31 12:13 $
+** $Id: qt/qstylefactory.cpp   3.3.7   edited Feb 20 2006 $
 **
 ** Implementation of QStyleFactory class
 **
@@ -106,14 +106,14 @@ QStyleFactoryPrivate::~QStyleFactoryPrivate()
     plugin (see \l QStylePlugin).
 
     QStyleFactory::keys() returns a list of valid keys, typically
-    including "windows", "motif", "cde", "motifplus", "platinum",
-    "sgi" and "compact". Depending on the platform, "windowsxp",
-    "aqua" or "macintosh" may be available.
+    including "Windows", "Motif", "CDE", "MotifPlus", "Platinum",
+    "SGI" and "Compact". Depending on the platform, "WindowsXP",
+    "Aqua" or "Macintosh" may be available.
 */
 
 /*!
-    Creates a QStyle object that matches \a key. This is either a
-    built-in style, or a style from a style plugin.
+    Creates a QStyle object that matches \a key case-insensitively.
+    This is either a built-in style, or a style from a style plugin.
 
     \sa keys()
 */
