@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qfileinfo.cpp   3.3.4   edited Jan 6 14:16 $
+** $Id: qt/qfileinfo.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of QFileInfo class
 **
 ** Created : 950628
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -525,7 +525,11 @@ bool QFileInfo::isWritable() const
 bool QFileInfo::isExecutable() const
 {
 #ifdef Q_WS_WIN
+#if defined(_MSC_VER) && _MSC_VER >= 1400
+    return permission( ExeUser );
+#else
     return qt_file_access( fn, X_OK ) && permission( ExeUser );
+#endif //_MSC_VER >= 1400
 #else
     return qt_file_access( fn, X_OK );
 #endif
