@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qiodevice.cpp   3.2.3   edited Jun 1 18:37 $
+** $Id: qt/qiodevice.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of QIODevice class
 **
 ** Created : 940913
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the tools module of the Qt GUI Toolkit.
 **
@@ -414,12 +414,11 @@ QIODevice::~QIODevice()
 
 /*!
   \fn void QIODevice::setFlags( int f )
-  \internal
+
   Used by subclasses to set the device flags.
 */
 
 /*!
-  \internal
   Used by subclasses to set the device type.
 */
 
@@ -434,7 +433,6 @@ void QIODevice::setType( int t )
 }
 
 /*!
-  \internal
   Used by subclasses to set the device mode.
 */
 
@@ -449,7 +447,6 @@ void QIODevice::setMode( int m )
 }
 
 /*!
-  \internal
   Used by subclasses to set the device state.
 */
 
@@ -464,7 +461,6 @@ void QIODevice::setState( int s )
 }
 
 /*!
-  \internal
   Used by subclasses to set the device status (not state) to \a s.
 */
 
@@ -554,7 +550,7 @@ QIODevice::Offset QIODevice::at() const
 /*!
     Virtual function that sets the I/O device position to \a pos.
     Returns TRUE if the position was successfully set, i.e. \a pos is
-    within range; otherwise returns FALSE.
+    within range and the seek was successful; otherwise returns FALSE.
 
     \sa size()
 */
@@ -563,8 +559,8 @@ bool QIODevice::at( Offset pos )
 {
 #if defined(QT_CHECK_RANGE)
     if ( pos > size() ) {
-#if defined(QT_LARGEFILE_SUPPORT) && defined(QT_ABI_QT4)
-	qWarning( "QIODevice::at: Index %llu out of range", pos );
+#if defined(QT_ABI_QT4)
+	qWarning( "QIODevice::at: Index %lld out of range", pos );
 #else
 	qWarning( "QIODevice::at: Index %lu out of range", pos );
 #endif
@@ -584,9 +580,12 @@ bool QIODevice::atEnd() const
 {
     if ( isSequentialAccess() || isTranslated() ) {
 	QIODevice* that = (QIODevice*)this;
+        const int oldStatus = ioSt;
 	int c = that->getch();
 	bool result = c < 0;
 	that->ungetch(c);
+        if (ioSt != oldStatus)
+            that->ioSt = oldStatus;
 	return result;
     } else {
 	return at() == size();
