@@ -1,9 +1,9 @@
 /****************************************************************************
-** $Id: qt/qaction.cpp   3.2.3   edited Sep 19 16:06 $
+** $Id: qt/qaction.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of QAction class
 **
-** Copyright (C) 2000-2003 Trolltech AS.  All rights reserved.
+** Copyright (C) 2000-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -123,7 +123,7 @@
 class QActionPrivate
 {
 public:
-    QActionPrivate();
+    QActionPrivate(QAction *act);
     ~QActionPrivate();
     QIconSet *iconset;
     QString text;
@@ -141,10 +141,12 @@ public:
     uint toggleaction : 1;
     uint on : 1;
     uint forceDisabled : 1;
+    uint forceInvisible : 1;
 #ifndef QT_NO_TOOLTIP
     QToolTipGroup tipGroup;
 #endif
     QActionGroupPrivate* d_group;
+    QAction *action;
 
     struct MenuItem {
 	MenuItem():popup(0),id(0){}
@@ -171,17 +173,17 @@ public:
     QString statusTip() const;
 };
 
-QActionPrivate::QActionPrivate()
+QActionPrivate::QActionPrivate(QAction *act)
     : iconset( 0 ),
 #ifndef QT_NO_ACCEL
       key( 0 ), accel( 0 ), accelid( 0 ),
 #endif
       enabled( TRUE ), visible( TRUE ), toggleaction( FALSE ), on( FALSE ),
-      forceDisabled( FALSE ),
+      forceDisabled( FALSE ), forceInvisible( FALSE ),
 #ifndef QT_NO_TOOLTIP
       tipGroup( 0 ),
 #endif
-      d_group( 0 )
+      d_group( 0 ), action(act)
 {
     menuitems.setAutoDelete( TRUE );
     comboitems.setAutoDelete( TRUE );
@@ -209,6 +211,33 @@ QActionPrivate::~QActionPrivate()
 	    menu->removeItem( mi->id );
     }
 
+    QPtrListIterator<QActionPrivate::ComboItem> itci(comboitems);
+    QActionPrivate::ComboItem* ci;
+    while ( ( ci = itci.current() ) ) {
+	++itci;
+	QComboBox* combo = ci->combo;
+	combo->clear();
+	QActionGroup *group = ::qt_cast<QActionGroup*>(action->parent());
+	QObjectList *siblings = group ? group->queryList("QAction") : 0;
+	if (siblings) {
+	    QObjectListIt it(*siblings);
+	    while (it.current()) {
+		QAction *sib = ::qt_cast<QAction*>(it.current());
+		++it;
+		sib->removeFrom(combo);
+	    }
+	    it = QObjectListIt(*siblings);
+	    while (it.current()) {
+		QAction *sib = ::qt_cast<QAction*>(it.current());
+		++it;
+		if (sib == action)
+		    continue;
+		sib->addTo(combo);
+	    }
+	}
+	delete siblings;
+    }
+
 #ifndef QT_NO_ACCEL
     delete accel;
 #endif
@@ -238,7 +267,6 @@ public:
     void update( const QActionGroup * );
 };
 
-
 void QActionPrivate::update( uint upd )
 {
     for ( QPtrListIterator<MenuItem> it( menuitems); it.current(); ++it ) {
@@ -322,8 +350,11 @@ void QActionPrivate::update( uint upd )
 
 QString QActionPrivate::menuText() const
 {
-    if ( menutext.isNull() )
-	return text;
+    if ( menutext.isNull() ) {
+	QString t(text);
+	t.replace('&', "&&");
+	return t;
+    }
     return menutext;
 }
 
@@ -346,7 +377,6 @@ QString QActionPrivate::statusTip() const
     return statustip;
 }
 
-
 /*
   internal: guesses a descriptive text from a menu text
  */
@@ -355,7 +385,7 @@ static QString qt_stripMenuText( QString s )
     s.remove( QString::fromLatin1("...") );
     s.remove( QChar('&' ) );
     return s.stripWhiteSpace();
-};
+}
 
 /*!
     Constructs an action called \a name with parent \a parent.
@@ -372,7 +402,7 @@ static QString qt_stripMenuText( QString s )
 QAction::QAction( QObject* parent, const char* name )
     : QObject( parent, name )
 {
-    d = new QActionPrivate;
+    d = new QActionPrivate(this);
     init();
 }
 
@@ -391,7 +421,7 @@ QAction::QAction( QObject* parent, const char* name )
 QAction::QAction( QObject* parent, const char* name, bool toggle )
     : QObject( parent, name )
 {
-    d = new QActionPrivate;
+    d = new QActionPrivate(this);
     d->toggleaction = toggle;
     init();
 }
@@ -427,7 +457,7 @@ QAction::QAction( const QIconSet& icon, const QString& menuText, QKeySequence ac
 		  QObject* parent, const char* name )
     : QObject( parent, name )
 {
-    d = new QActionPrivate;
+    d = new QActionPrivate(this);
     if ( !icon.isNull() )
 	setIconSet( icon );
     d->text = qt_stripMenuText( menuText );
@@ -463,7 +493,7 @@ QAction::QAction( const QString& menuText, QKeySequence accel,
 		  QObject* parent, const char* name )
     : QObject( parent, name )
 {
-    d = new QActionPrivate;
+    d = new QActionPrivate(this);
     d->text = qt_stripMenuText( menuText );
     d->menutext = menuText;
     setAccel( accel );
@@ -490,7 +520,7 @@ QAction::QAction( const QString& menuText, QKeySequence accel,
 QAction::QAction( const QString& text, const QIconSet& icon, const QString& menuText, QKeySequence accel, QObject* parent, const char* name, bool toggle )
     : QObject( parent, name )
 {
-    d = new QActionPrivate;
+    d = new QActionPrivate(this);
     d->toggleaction = toggle;
     if ( !icon.isNull() )
 	setIconSet( icon );
@@ -521,7 +551,7 @@ QAction::QAction( const QString& text, const QIconSet& icon, const QString& menu
 QAction::QAction( const QString& text, const QString& menuText, QKeySequence accel, QObject* parent, const char* name, bool toggle )
     : QObject( parent, name )
 {
-    d = new QActionPrivate;
+    d = new QActionPrivate(this);
     d->toggleaction = toggle;
     d->text = text;
     d->menutext = menuText;
@@ -623,7 +653,10 @@ void QAction::setMenuText( const QString& text )
     d->update();
 }
 
-QString QAction::menuText() const { return d->menuText(); }
+QString QAction::menuText() const
+{
+    return d->menuText();
+}
 
 /*!
     \property QAction::toolTip
@@ -795,10 +828,28 @@ bool QAction::isToggleAction() const
     return d->toggleaction;
 }
 
+/*!
+    Activates the action and executes all connected slots.
+    This only works for actions that are not toggle action.
+
+    \sa toggle()
+*/
+void QAction::activate()
+{
+    if ( isToggleAction() ) {
+#if defined(QT_CHECK_STATE)
+	qWarning( "QAction::%s() (%s) Toggle actions "
+		  "can not be activated", "activate", name() );
+#endif
+	return;
+    }
+    emit activated();
+}
+
 /*!
     Toggles the state of a toggle action.
 
-    \sa on, toggled(), isToggleAction()
+    \sa on, activate(), toggled(), isToggleAction()
 */
 void QAction::toggle()
 {
@@ -873,6 +924,17 @@ bool QAction::isEnabled() const
     return d->enabled;
 }
 
+/*!
+    Disables the action if \a disable is TRUE; otherwise
+    enables the action.
+
+    See the \l enabled documentation for more information.
+*/
+void QAction::setDisabled( bool disable )
+{
+    setEnabled( !disable );
+}
+
 /*!
     \property QAction::visible
     \brief whether the action can be seen (e.g. in menus and toolbars)
@@ -886,6 +948,8 @@ bool QAction::isEnabled() const
 */
 void QAction::setVisible( bool visible )
 {
+    d->forceInvisible = !visible;
+
     if ( (bool)d->visible == visible )
 	return;
     d->visible = visible;
@@ -964,25 +1028,23 @@ bool QAction::addTo( QWidget* w )
     } else
 #endif
     if ( ::qt_cast<QPopupMenu*>(w) ) {
-	if ( !qstrcmp( name(), "qt_separator_action" ) ) {
-	    ((QPopupMenu*)w)->insertSeparator();
-	} else {
-	    QActionPrivate::MenuItem* mi = new QActionPrivate::MenuItem;
-	    mi->popup = (QPopupMenu*) w;
-	    QIconSet* diconset = d->iconset;
-	    if ( diconset )
-		mi->id = mi->popup->insertItem( *diconset, QString::fromLatin1("") );
-	    else
-		mi->id = mi->popup->insertItem( QString::fromLatin1("") );
-	    addedTo( mi->popup->indexOf( mi->id ), mi->popup );
-	    mi->popup->connectItem( mi->id, this, SLOT(internalActivation()) );
-	    d->menuitems.append( mi );
-	    d->update( QActionPrivate::State | QActionPrivate::Visibility | QActionPrivate::EverythingElse ) ;
-	    w->topLevelWidget()->className();
-	    connect( mi->popup, SIGNAL(highlighted( int )), this, SLOT(menuStatusText( int )) );
-	    connect( mi->popup, SIGNAL(aboutToHide()), this, SLOT(clearStatusText()) );
-	    connect( mi->popup, SIGNAL( destroyed() ), this, SLOT( objectDestroyed() ) );
-	}
+	QActionPrivate::MenuItem* mi = new QActionPrivate::MenuItem;
+	mi->popup = (QPopupMenu*) w;
+	QIconSet* diconset = d->iconset;
+	if ( !qstrcmp( name(), "qt_separator_action" ) )
+	    mi->id = ((QPopupMenu*)w)->insertSeparator();
+	else if ( diconset )
+	    mi->id = mi->popup->insertItem( *diconset, QString::fromLatin1("") );
+	else
+	    mi->id = mi->popup->insertItem( QString::fromLatin1("") );
+	addedTo( mi->popup->indexOf( mi->id ), mi->popup );
+	mi->popup->connectItem( mi->id, this, SLOT(internalActivation()) );
+	d->menuitems.append( mi );
+	d->update( QActionPrivate::State | QActionPrivate::Visibility | QActionPrivate::EverythingElse ) ;
+	w->topLevelWidget()->className();
+	connect( mi->popup, SIGNAL(highlighted(int)), this, SLOT(menuStatusText(int)) );
+	connect( mi->popup, SIGNAL(aboutToHide()), this, SLOT(clearStatusText()) );
+	connect( mi->popup, SIGNAL( destroyed() ), this, SLOT( objectDestroyed() ) );
     // Makes only sense when called by QActionGroup::addTo
     } else if ( ::qt_cast<QComboBox*>(w) ) {
 	QActionPrivate::ComboItem *ci = new QActionPrivate::ComboItem;
@@ -1108,7 +1170,8 @@ void QAction::menuStatusText( int id )
 */
 void QAction::clearStatusText()
 {
-    showStatusText( QString::null );
+    if (!statusTip().isEmpty())
+	showStatusText( QString::null );
 }
 
 /*!
@@ -1142,7 +1205,7 @@ bool QAction::removeFrom( QWidget* w )
 	while ( ( mi = it.current() ) ) {
 	    ++it;
 	    if ( mi->popup == w ) {
-		disconnect( mi->popup, SIGNAL(highlighted( int )), this, SLOT(menuStatusText(int)) );
+		disconnect( mi->popup, SIGNAL(highlighted(int)), this, SLOT(menuStatusText(int)) );
 		disconnect( mi->popup, SIGNAL(aboutToHide()), this, SLOT(clearStatusText()) );
 		disconnect( mi->popup, SIGNAL( destroyed() ), this, SLOT( objectDestroyed() ) );
 		mi->popup->removeItem( mi->id );
@@ -1225,18 +1288,21 @@ void QAction::objectDestroyed()
     \sa activated() setToggleAction() setOn()
 */
 
-
-
 void QActionGroupPrivate::update( const QActionGroup* that )
 {
     for ( QPtrListIterator<QAction> it( actions ); it.current(); ++it ) {
-	if ( that->isEnabled() && !it.current()->d->forceDisabled )
+	if ( that->isEnabled() && !it.current()->d->forceDisabled ) {
 	    it.current()->setEnabled( TRUE );
-	else if ( !that->isEnabled() && it.current()->isEnabled() ) {
+	} else if ( !that->isEnabled() && it.current()->isEnabled() ) {
 	    it.current()->setEnabled( FALSE );
 	    it.current()->d->forceDisabled = FALSE;
 	}
-	it.current()->setVisible( that->isVisible() );
+	if ( that->isVisible() && !it.current()->d->forceInvisible ) {
+	    it.current()->setVisible( TRUE );
+	} else if ( !that->isVisible() && it.current()->isVisible() ) {
+	    it.current()->setVisible( FALSE );
+	    it.current()->d->forceInvisible = FALSE;
+	}
     }
     for ( QPtrListIterator<QComboBox> cb( comboboxes ); cb.current(); ++cb ) {
 	QComboBox *combobox = cb.current();
@@ -1480,6 +1546,11 @@ bool QActionGroup::isExclusive() const
 
     Changing usesDropDown only affects \e subsequent calls to addTo().
 
+    Note that setting this property for actions in a combobox causes
+    calls to their \link QAction::setVisible()\endlink,
+    \link QAction::setEnabled()\endlink, and
+    \link QAction::setDisabled()\endlink functions to have no effect.
+
     This property's default is FALSE.
 
 */
@@ -1512,11 +1583,15 @@ void QActionGroup::add( QAction* action )
 	action->setWhatsThis( whatsThis() );
     if ( action->toolTip().isNull() )
 	action->setToolTip( toolTip() );
-    action->setEnabled( isEnabled() );
+
+    if (!action->d->forceDisabled)
+	action->d->enabled = isEnabled();
+    if (!action->d->forceInvisible)
+	action->d->visible = isVisible();
 
     connect( action, SIGNAL( destroyed() ), this, SLOT( childDestroyed() ) );
     connect( action, SIGNAL( activated() ), this, SIGNAL( activated() ) );
-    connect( action, SIGNAL( toggled( bool ) ), this, SLOT( childToggled( bool ) ) );
+    connect( action, SIGNAL( toggled(bool) ), this, SLOT( childToggled(bool) ) );
 
     for ( QPtrListIterator<QComboBox> cb( d->comboboxes ); cb.current(); ++cb ) {
 	action->addTo( cb.current() );
@@ -1652,6 +1727,7 @@ bool QActionGroup::addTo( QWidget* w )
 		if ( foundOn )
 		    box->setCurrentItem( onIndex );
 		connect( box, SIGNAL(activated(int)), this, SLOT( internalComboBoxActivated(int)) );
+		connect( box, SIGNAL(highlighted(int)), this, SLOT( internalComboBoxHighlighted(int)) );
 		d->update( this );
 		return TRUE;
 	    }
@@ -1946,9 +2022,27 @@ void QActionGroup::internalComboBoxActivated( int index )
 	    emit activated();
 	    emit ((QActionGroup*)a)->activated();
 	}
+	a->clearStatusText();
     }
 }
 
+/*! \internal
+*/
+void QActionGroup::internalComboBoxHighlighted( int index )
+{
+    QAction *a = 0;
+    for ( int i = 0; i <= index && i < (int)d->actions.count(); ++i ) {
+	a = d->actions.at( i );
+	if ( a && !qstrcmp( a->name(), "qt_separator_action" ) )
+	    index++;
+    }
+    a = d->actions.at( index );
+    if ( a )
+	a->showStatusText(a->statusTip());
+    else
+	clearStatusText();
+}
+
 /*! \internal
 */
 void QActionGroup::internalToggle( QAction *a )
@@ -1985,6 +2079,8 @@ void QActionGroup::objectDestroyed()
 }
 
 /*!
+    \internal
+    
     This function is called from the addTo() function when it has
     created a widget (\a actionWidget) for the child action \a a in
     the \a container.
@@ -1999,7 +2095,8 @@ void QActionGroup::addedTo( QWidget *actionWidget, QWidget *container, QAction *
 
 /*!
     \overload
-
+    \internal
+    
     This function is called from the addTo() function when it has
     created a menu item for the child action at the index position \a
     index in the popup menu \a menu.
