@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qbuttongroup.cpp   3.2.3   edited May 24 15:31 $
+** $Id: qt/qbuttongroup.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of QButtonGroup class
 **
 ** Created : 950130
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -472,11 +472,6 @@ void QButtonGroup::buttonToggled( bool on )
 
 
 
-/*!
-    Sets the button with id \a id to be on; if this is an exclusive
-    group, all the other buttons in the group will be set to off.
-*/
-
 void QButtonGroup::setButton( int id )
 {
     QButton * b = find( id );
@@ -590,8 +585,14 @@ void QButtonGroup::moveFocus( int key )
 	buttoncand->animateTimeout(); // ### crude l&f hack
     }
 
-    if ( candidate )
+    if ( candidate ) {
+	if (key == Key_Up || key == Key_Left)
+	    QFocusEvent::setReason(QFocusEvent::Backtab);
+	else
+	    QFocusEvent::setReason(QFocusEvent::Tab);
 	candidate->setFocus();
+	QFocusEvent::resetReason();
+    }
 }
 
 
@@ -622,8 +623,16 @@ QButton * QButtonGroup::selected() const
 }
 
 /*!
-    Returns the id of the selected toggle button if exactly one is selected;
-    otherwise returns -1.
+    \property QButtonGroup::selectedId
+    \brief the selected toggle button
+
+    The toggle button is specified as an ID.
+
+    If no toggle button is selected, this property holds -1.
+
+    If setButton() is called on an exclusive group, the button with
+    the given id will be set to on and all the others will be set to
+    off.
 
     \sa selected()
 */
