@@ -1,11 +1,11 @@
 /**********************************************************************
-** $Id: qt/qcombobox.cpp   3.3.4   edited Nov 25 14:57 $
+** $Id: qt/qcombobox.cpp   3.3.7   edited Jan 30 2006 $
 **
 ** Implementation of QComboBox widget class
 **
 ** Created : 940426
 **
-** Copyright (C) 1992-2004 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -147,6 +147,10 @@
     <img src="qcombo3-m.png">(Motif 2, read-only)<br clear=all>
     <img src="qcombo1-w.png">(Windows style)
 
+    Depending on the style, QComboBox will use a QListBox or a
+    QPopupMenu to display the list of items. See setListBox() for
+    more information.
+
     \sa QLineEdit QListBox QSpinBox QRadioButton QButtonGroup
     \link guibooks.html#fowler GUI Design Handbook: Combo Box,\endlink
     \link guibooks.html#fowler GUI Design Handbook: Drop-Down List Box.\endlink
@@ -343,9 +347,14 @@ public:
     {
 	return QPopupMenu::itemHeight( index );
     }
-
 };
 
+static inline QString escapedComboString(const QString &str)
+{
+    QString stringToReturn = str;
+    return stringToReturn.replace('&', "&&");
+}
+
 class QComboBoxPopupItem : public QCustomMenuItem
 {
     QListBoxItem *li;
@@ -630,7 +639,7 @@ void QComboBox::insertStrList( const QStrList *list, int index )
 	if ( d->usingListBox() )
 	    d->listBox()->insertItem( QString::fromLatin1(tmp), index );
 	else
-	    d->popup()->insertItem( QString::fromLatin1(tmp), index, index );
+	    d->popup()->insertItem( escapedComboString(QString::fromLatin1(tmp)), index, index );
 	if ( index++ == d->current && d->current < count() ) {
 	    if ( d->ed ) {
 		d->ed->setText( text( d->current ) );
@@ -658,7 +667,7 @@ void QComboBox::insertStringList( const QStringList &list, int index )
 	if ( d->usingListBox() )
 	    d->listBox()->insertItem( *it, index );
 	else
-	    d->popup()->insertItem( *it, index, index );
+	    d->popup()->insertItem( escapedComboString(*it), index, index );
 	if ( index++ == d->current && d->current < count() ) {
 	    if ( d->ed ) {
 		d->ed->setText( text( d->current ) );
@@ -705,7 +714,7 @@ void QComboBox::insertStrList( const char **strings, int numStrings, int index)
 	if ( d->usingListBox() )
 	    d->listBox()->insertItem( QString::fromLatin1(strings[i]), index );
 	else
-	    d->popup()->insertItem( QString::fromLatin1(strings[i]), index, index );
+	    d->popup()->insertItem( escapedComboString(QString::fromLatin1(strings[i])), index, index );
 	i++;
 	if ( index++ == d->current && d->current < count()  ) {
 	    if ( d->ed ) {
@@ -734,7 +743,7 @@ void QComboBox::insertItem( const QString &t, int index )
     if ( d->usingListBox() )
 	d->listBox()->insertItem( t, index );
     else
-	d->popup()->insertItem( t, index, index );
+	d->popup()->insertItem( escapedComboString(t), index, index );
     if ( index != cnt )
 	reIndex();
     if ( index == d->current && d->current < count()  ) {
@@ -792,7 +801,7 @@ void QComboBox::insertItem( const QPixmap &pixmap, const QString& text, int inde
     if ( d->usingListBox() )
 	d->listBox()->insertItem( pixmap, text, index );
     else
-	d->popup()->insertItem( pixmap, text, index, index );
+	d->popup()->insertItem( pixmap, escapedComboString(text), index, index );
     if ( index != cnt )
 	reIndex();
     if ( index == d->current && d->current < count()  ) {
@@ -918,10 +927,13 @@ QString QComboBox::text( int index ) const
 {
     if ( !checkIndex( "text", name(), count(), index ) )
 	return QString::null;
-    if ( d->usingListBox() )
+    if ( d->usingListBox() ) {
 	return d->listBox()->text( index );
-    else
-	return d->popup()->text( index );
+    } else {
+        QString retText = d->popup()->text(index);
+        retText.replace("&&", "&");
+	return retText;
+    }
 }
 
 /*!
@@ -1531,12 +1543,12 @@ void QComboBox::popup()
 	    for(unsigned int i = 0; i < d->listBox()->count(); i++) {
 		QListBoxItem *item = d->listBox()->item(i);
 		if(item->rtti() == QListBoxText::RTTI) {
-		    d->popup()->insertItem(item->text(), i, i);
+		    d->popup()->insertItem(escapedComboString(item->text()), i, i);
 		} else if(item->rtti() == QListBoxPixmap::RTTI) {
 		    if(item->pixmap())
-			d->popup()->insertItem(QIconSet(*item->pixmap()), item->text(), i, i);
+			d->popup()->insertItem(QIconSet(*item->pixmap()), escapedComboString(item->text()), i, i);
 		    else
-			d->popup()->insertItem(item->text(), i, i);
+			d->popup()->insertItem(escapedComboString(item->text()), i, i);
 		} else {
 		    d->popup()->insertItem(new QComboBoxPopupItem(item), i, i);
 		}
