@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qdockwindow.cpp   3.3.4   edited Jan 4 19:27 $
+** $Id: qt/qdockwindow.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of the QDockWindow class
 **
 ** Created : 001010
 **
-** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the workspace module of the Qt GUI Toolkit.
 **
@@ -1195,7 +1195,7 @@ void QDockWindow::handleMove( const QPoint &pos, const QPoint &gp, bool drawRect
 	w = 0;
     currRect.moveBy( pos.x(), pos.y() );
     if ( !::qt_cast<QDockArea*>(w) ) {
-	if ( startOrientation != Horizontal && ::qt_cast<QToolBar*>(this) )
+        if ( startOrientation != Horizontal && ::qt_cast<QToolBar*>(this) )
 	    swapRect( currRect, Horizontal, startOffset, (QDockArea*)w );
 	if ( drawRect ) {
 	    unclippedPainter->setPen( QPen( gray, 3 ) );
@@ -1218,12 +1218,12 @@ void QDockWindow::handleMove( const QPoint &pos, const QPoint &gp, bool drawRect
 	    move( mp );
 	}
 	state = OutsideDock;
-	return;
+        return;
     }
 
     QDockArea *area = (QDockArea*)w;
     if( area->isVisible() ) {
-	state = InDock;
+        state = InDock;
 	Orientation o = ( area ? area->orientation() :
 			  ( boxLayout()->direction() == QBoxLayout::LeftToRight ||
 			    boxLayout()->direction() == QBoxLayout::RightToLeft ?
@@ -1360,8 +1360,8 @@ void QDockWindow::updatePosition( const QPoint &globalPos )
     bool doUpdate = TRUE;
     bool doOrientationChange = TRUE;
     if ( state != curPlace && state == InDock ) {
-	doUpdate = FALSE;
-	curPlace = state;
+        doUpdate = FALSE;
+        curPlace = state;
 	updateGui();
 	QApplication::sendPostedEvents();
     }
@@ -1380,7 +1380,7 @@ void QDockWindow::updatePosition( const QPoint &globalPos )
 	    if ( differentDocks ) {
 		if ( doUpdate ) {
 		    doUpdate = FALSE;
-		    curPlace = state;
+                    curPlace = state;
 		    updateGui();
 		}
 		emit orientationChanged( tmpDockArea->orientation() );
@@ -1400,7 +1400,7 @@ void QDockWindow::updatePosition( const QPoint &globalPos )
 	    delete (QDockArea::DockWindowData*)dockWindowData;
 	    dockWindowData = dockArea->dockWindowData( this );
 	    dockArea->removeDockWindow( this, TRUE,
-		startOrientation != Horizontal && ::qt_cast<QToolBar*>(this) );
+                                        startOrientation != Horizontal && ::qt_cast<QToolBar*>(this) );
 	}
 	dockArea = 0;
 	QPoint topLeft = currRect.topLeft();
@@ -1420,7 +1420,7 @@ void QDockWindow::updatePosition( const QPoint &globalPos )
     }
 
     if ( doUpdate ) {
-	curPlace = state;
+        curPlace = state;
 	updateGui();
     }
     if ( doOrientationChange )
@@ -1430,7 +1430,8 @@ void QDockWindow::updatePosition( const QPoint &globalPos )
 	QApplication::sendPostedEvents( this, QEvent::LayoutHint );
 	if ( ::qt_cast<QToolBar*>(this) )
 	    adjustSize();
-	clearWState(WState_Resized); // Ensures size is recalculated (non-opaque).
+        if (lastSize == QSize(-1, -1))
+            clearWState(WState_Resized); // Ensures size is recalculated (non-opaque).
 	show();
 	if ( parentWidget() && isTopLevel() )
 	    parentWidget()->setActiveWindow();
@@ -1886,7 +1887,8 @@ void QDockWindow::undock( QWidget *w )
 	adjustSize();
     if ( !w ) {
 	if ( !parentWidget() || parentWidget()->isVisible() ) {
-	    clearWState(WState_Resized); // Ensures size is recalculated (opaque).
+            if (lastSize == QSize(-1, -1))
+                clearWState(WState_Resized); // Ensures size is recalculated (opaque).
 	    show();
 	}
     } else {
