@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qlcdnumber.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qlcdnumber.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of QLCDNumber class
 **
 ** Created : 940518
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -77,8 +77,8 @@
 
     Incidentally, QLCDNumber is the very oldest part of Qt, tracing
     back to a BASIC program on the \link
-    http://www.nvg.ntnu.no/sinclair/spectrum.htm Sinclair
-    Spectrum\endlink.
+    http://www.nvg.ntnu.no/sinclair/computers/zxspectrum/zxspectrum.htm
+    Sinclair Spectrum\endlink.
 
     <img src=qlcdnum-m.png> <img src=qlcdnum-w.png>
 
@@ -377,6 +377,13 @@ void QLCDNumber::setNumDigits( int numDigits )
 #endif
         numDigits = 99;
     }
+    if (numDigits < 0 ) {
+#if defined(QT_CHECK_RANGE)
+        qWarning( "QLCDNumber::setNumDigits: (%s) Min 0 digits allowed",
+                 name( "unnamed" ) );
+#endif
+        numDigits = 0;
+    }
     if ( digitStr.isNull() ) {                  // from constructor
         ndigits = numDigits;
         digitStr.fill( ' ', ndigits );
@@ -713,8 +720,7 @@ void QLCDNumber::internalSetString( const QString& s )
         if ( smallPoint )
             points = newPoints;
         repaint( contentsRect() );
-    }
-    else {
+    } else {
         QPainter p( this );
         if ( !smallPoint )
             drawString( buffer, p );
