@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qmainwindow.cpp   3.2.3   edited May 30 15:52 $
+** $Id: qt/qmainwindow.cpp   3.3.7   edited Jan 30 2006 $
 **
 ** Implementation of QMainWindow class
 **
 ** Created : 980312
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -287,8 +287,9 @@ class QHideToolTip : public QToolTip
 {
 public:
     QHideToolTip( QWidget *parent ) : QToolTip( parent ) {}
+    ~QHideToolTip() {}
 
-    void maybeTip( const QPoint &pos );
+    virtual void maybeTip( const QPoint &pos );
 };
 #endif
 
@@ -1144,7 +1145,7 @@ void QMainWindow::setDockEnabled( QDockWindow *dw, Dock dock, bool enable )
 {
     if ( d->dockWindows.find( dw ) == -1 ) {
 	d->dockWindows.append( dw );
-	connect( dw, SIGNAL( placeChanged( QDockWindow::Place ) ),
+	connect( dw, SIGNAL( placeChanged(QDockWindow::Place) ),
 		 this, SLOT( slotPlaceChanged() ) );
     }
     QString s;
@@ -1241,11 +1242,12 @@ void QMainWindow::addDockWindow( QDockWindow *dockWindow,
 #endif
     moveDockWindow( dockWindow, edge );
     dockWindow->setNewLine( newLine );
-    if ( d->dockWindows.find( dockWindow ) == -1 )
+    if ( d->dockWindows.find( dockWindow ) == -1 ) {
 	d->dockWindows.append( dockWindow );
-    connect( dockWindow, SIGNAL( placeChanged( QDockWindow::Place ) ),
-	     this, SLOT( slotPlaceChanged() ) );
-    dockWindow->installEventFilter( this );
+	connect( dockWindow, SIGNAL( placeChanged(QDockWindow::Place) ),
+		 this, SLOT( slotPlaceChanged() ) );
+	dockWindow->installEventFilter( this );
+    }
     dockWindow->setOpaqueMoving( d->opaque );
 }
 
@@ -1403,13 +1405,13 @@ void QMainWindow::moveDockWindow( QDockWindow * dockWindow, Dock edge, bool nl,
 void QMainWindow::removeDockWindow( QDockWindow * dockWindow )
 {
 #ifdef Q_WS_MAC
-    if(isTopLevel() && dockWindow->area() == topDock() && !dockWindows( DockTop ).count()) 
+    if(isTopLevel() && dockWindow->area() == topDock() && !dockWindows( DockTop ).count())
 	ChangeWindowAttributes((WindowPtr)handle(), 0, kWindowToolbarButtonAttribute);
 #endif
 
     dockWindow->hide();
     d->dockWindows.removeRef( dockWindow );
-    disconnect( dockWindow, SIGNAL( placeChanged( QDockWindow::Place ) ),
+    disconnect( dockWindow, SIGNAL( placeChanged(QDockWindow::Place) ),
 		this, SLOT( slotPlaceChanged() ) );
     dockWindow->removeEventFilter( this );
 }
@@ -1440,13 +1442,16 @@ void QMainWindow::setUpLayout()
 	delete l;
     }
 
-    QLayout::ResizeMode tll_rm = minimumSize().isNull() ? QLayout::Minimum : QLayout::FreeResize;
-
-    if ( d->tll )
-	tll_rm = d->tll->resizeMode();
-    delete d->tll;
-    d->tll = new QBoxLayout( this, QBoxLayout::Down );
-    d->tll->setResizeMode( tll_rm );
+    if (!d->tll) {
+        d->tll = new QBoxLayout( this, QBoxLayout::Down );
+        d->tll->setResizeMode( minimumSize().isNull() ? QLayout::Minimum : QLayout::FreeResize );
+    } else {
+        d->tll->setMenuBar( 0 );
+        QLayoutIterator it = d->tll->iterator();
+        QLayoutItem *item;
+        while ( (item = it.takeCurrent()) )
+	    delete item;
+    }
 
 #ifndef QT_NO_MENUBAR
     if ( d->mb && d->mb->isVisibleTo( this ) ) {
@@ -1486,6 +1491,19 @@ void QMainWindow::show()
 {
     if ( !d->tll )
 	setUpLayout();
+
+    // show all floating dock windows not explicitly hidden
+    if (!isVisible()) {
+	QPtrListIterator<QDockWindow> it(d->dockWindows);
+	while ( it.current() ) {
+	    QDockWindow *dw = it.current();
+	    ++it;
+	    if ( dw->isTopLevel() && !dw->isVisible() && !dw->testWState(WState_ForceHide) )
+		dw->show();
+	}
+    }
+
+    // show us last so we get focus
     QWidget::show();
 }
 
@@ -1495,15 +1513,15 @@ void QMainWindow::show()
 void QMainWindow::hide()
 {
     if ( isVisible() ) {
-	QObjectList *list = queryList( "QDockWindow" );
-	QObjectListIt it( *list );
+	QPtrListIterator<QDockWindow> it(d->dockWindows);
 	while ( it.current() ) {
-	    QDockWindow *dw = (QDockWindow*)it.current();
+	    QDockWindow *dw = it.current();
 	    ++it;
-	    if ( dw->isTopLevel() && dw->isVisible() )
-		dw->hide();
+	    if ( dw->isTopLevel() && dw->isVisible() ) {
+		dw->hide(); // implicit hide, so clear forcehide
+		((QMainWindow*)dw)->clearWState(WState_ForceHide);
+	    }
 	}
-	delete list;
     }
 
     QWidget::hide();
@@ -1809,10 +1827,8 @@ bool QMainWindow::rightJustification() const
 
 void QMainWindow::triggerLayout( bool deleteLayout )
 {
-    if ( !deleteLayout && d->tll ) {
-    } else {
+    if ( deleteLayout || !d->tll )
 	setUpLayout();
-    }
     QApplication::postEvent( this, new QEvent( QEvent::LayoutHint ) );
 }
 
