@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qstatusbar.cpp   3.2.3   edited May 13 09:08 $
+** $Id: qt/qstatusbar.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of QStatusBar class
 **
 ** Created : 980119
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -468,7 +468,7 @@ void QStatusBar::paintEvent( QPaintEvent * )
 	item = d->items.next();
     }
     if ( haveMessage ) {
-	p.setPen( colorGroup().text() );
+	p.setPen( colorGroup().foreground() );
 	p.drawText( 6, 0, psx, height(), AlignVCenter | SingleLine, d->tempItem );
     }
 }
