@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id:  qt/qtextedit.h   3.0.5   edited Jun 11 11:18 $
+** $Id: qt/qtextedit.h   3.3.7   edited Aug 31 2005 $
 **
 ** Definition of the QTextEdit class
 **
 ** Created : 990101
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the widgets module of the Qt GUI Toolkit.
 **
@@ -43,9 +43,14 @@
 #include "qstylesheet.h"
 #include "qptrvector.h"
 #include "qvaluelist.h"
+#include "qptrlist.h"
 #endif // QT_H
 
 #ifndef QT_NO_TEXTEDIT
+// uncomment below to enable optimization mode - also uncomment the
+// optimDoAutoScroll() private slot since moc ignores #ifdefs..
+#define QT_TEXTEDIT_OPTIMIZATION
+
 class QPainter;
 class QTextDocument;
 class QTextCursor;
@@ -65,13 +70,73 @@ class QTextString;
 struct QUndoRedoInfoPrivate;
 class QPopupMenu;
 class QTextEditPrivate;
+class QSyntaxHighlighter;
+
+#ifdef QT_TEXTEDIT_OPTIMIZATION
+class QTextEditOptimPrivate
+{
+public:
+    // Note: no left-tag has any value for leftTag or parent, and
+    // no right-tag has any formatting flags set.
+    enum TagType { Color = 0, Format = 1 };
+    struct Tag {
+	TagType type:2;
+	bool bold:1;
+	bool italic:1;
+	bool underline:1;
+	int line;
+	int index;
+	Tag * leftTag; // ptr to left-tag in a left-right tag pair
+	Tag * parent;  // ptr to parent left-tag in a nested tag
+	Tag * prev;
+	Tag * next;
+	QString tag;
+    };
+    QTextEditOptimPrivate()
+    {
+	len = numLines = maxLineWidth = 0;
+	selStart.line = selStart.index = -1;
+	selEnd.line = selEnd.index = -1;
+	search.line = search.index = 0;
+	tags = lastTag = 0;
+    }
+    void clearTags()
+    {
+	Tag * itr = tags;
+	while ( tags ) {
+	    itr  = tags;
+	    tags = tags->next;
+	    delete itr;
+	}
+	tags = lastTag = 0;
+	tagIndex.clear();
+    }
+    ~QTextEditOptimPrivate()
+    {
+	clearTags();
+    }
+    int len;
+    int numLines;
+    int maxLineWidth;
+    struct Selection {
+	int line;
+	int index;
+    };
+    Selection selStart, selEnd, search;
+    Tag * tags, * lastTag;
+    QMap<int, QString> lines;
+    QMap<int, Tag *> tagIndex;
+};
+#endif
 
 class Q_EXPORT QTextEdit : public QScrollView
 {
     friend class QTextBrowser;
+    friend class QSyntaxHighlighter;
 
     Q_OBJECT
     Q_ENUMS( WordWrap WrapPolicy )
+    Q_SETS( AutoFormatting )
     Q_PROPERTY( TextFormat textFormat READ textFormat WRITE setTextFormat )
     Q_PROPERTY( QString text READ text WRITE setText )
     Q_PROPERTY( QBrush paper READ paper WRITE setPaper )
@@ -89,6 +154,8 @@ class Q_EXPORT QTextEdit : public QScrollView
     Q_PROPERTY( bool readOnly READ isReadOnly WRITE setReadOnly )
     Q_PROPERTY( bool undoRedoEnabled READ isUndoRedoEnabled WRITE setUndoRedoEnabled )
     Q_PROPERTY( int tabStopWidth READ tabStopWidth WRITE setTabStopWidth )
+    Q_PROPERTY( bool tabChangesFocus READ tabChangesFocus WRITE setTabChangesFocus )
+    Q_PROPERTY( AutoFormatting autoFormatting READ autoFormatting WRITE setAutoFormatting )
 
 public:
     enum WordWrap {
@@ -100,15 +167,24 @@ public:
 
     enum WrapPolicy {
 	AtWordBoundary,
+	AtWhiteSpace = AtWordBoundary, // AtWhiteSpace is deprecated
 	Anywhere,
-	AtWhiteSpace = AtWordBoundary // deprecated, don't use
+	AtWordOrDocumentBoundary
+    };
+
+    enum AutoFormatting {
+	AutoNone = 0,
+	AutoBulletList = 0x00000001,
+	AutoAll = 0xffffffff
     };
 
     enum KeyboardAction {
 	ActionBackspace,
 	ActionDelete,
 	ActionReturn,
-	ActionKill
+	ActionKill,
+	ActionWordBackspace,
+	ActionWordDelete
     };
 
     enum CursorAction {
@@ -132,6 +208,12 @@ public:
 	AlignSubScript
     };
 
+    enum TextInsertionFlags {
+	RedoIndentation = 0x0001,
+	CheckNewLines = 0x0002,
+	RemoveSelected = 0x0004
+    };
+
     QTextEdit( const QString& text, const QString& context = QString::null,
 	       QWidget* parent=0, const char* name=0);
     QTextEdit( QWidget* parent=0, const char* name=0 );
@@ -160,8 +242,9 @@ public:
     int paragraphLength( int para ) const;
 
     QStyleSheet* styleSheet() const;
+#ifndef QT_NO_MIME
     QMimeSourceFactory* mimeSourceFactory() const;
-
+#endif
     QBrush paper() const;
     bool linkUnderline() const;
 
@@ -179,6 +262,7 @@ public:
     int tabStopWidth() const;
 
     QString anchorAt( const QPoint& pos );
+    QString anchorAt( const QPoint& pos, AnchorAttribute a );
 
     QSize sizeHint() const;
 
@@ -194,6 +278,7 @@ public:
     int pointSize() const;
     QColor color() const;
     QFont font() const;
+    QFont currentFont() const;
     int alignment() const;
     int undoDepth() const;
 
@@ -212,10 +297,17 @@ public:
 
     bool isUndoRedoEnabled() const;
     bool eventFilter( QObject *o, QEvent *e );
+    bool tabChangesFocus() const;
+
+    void setAutoFormatting( uint features );
+    uint autoFormatting() const;
+    QSyntaxHighlighter *syntaxHighlighter() const;
 
 public slots:
     void setEnabled( bool );
+#ifndef QT_NO_MIME
     virtual void setMimeSourceFactory( QMimeSourceFactory* factory );
+#endif
     virtual void setStyleSheet( QStyleSheet* styleSheet );
     virtual void scrollToAnchor( const QString& name );
     virtual void setPaper( const QBrush& pap );
@@ -284,7 +376,11 @@ public slots:
 
     virtual void scrollToBottom();
 
-    virtual void insert( const QString &text, bool indent = FALSE, bool checkNewLine = TRUE, bool removeSelected = TRUE );
+    void insert( const QString &text, uint insertionFlags = CheckNewLines | RemoveSelected ); // ## virtual in 4.0
+
+    // obsolete
+    virtual void insert( const QString &text, bool, bool = TRUE, bool = TRUE );
+
     virtual void insertAt( const QString &text, int para, int index );
     virtual void removeParagraph( int para );
     virtual void insertParagraph( const QString &text, int para );
@@ -293,6 +389,13 @@ public slots:
     virtual void clearParagraphBackground( int para );
 
     virtual void setUndoRedoEnabled( bool b );
+    void setTabChangesFocus( bool b ); // ### make virtual in 4.0
+
+#ifdef QT_TEXTEDIT_OPTIMIZATION
+    void polish();
+    void setMaxLogLines( int numLines );
+    int maxLogLines();
+#endif
 
 signals:
     void textChanged();
@@ -308,6 +411,8 @@ signals:
     void cursorPositionChanged( int para, int pos );
     void returnPressed();
     void modificationChanged( bool m );
+    void clicked( int parag, int index );
+    void doubleClicked( int parag, int index );
 
 protected:
     void repaintChanged();
@@ -346,6 +451,10 @@ protected:
 
 protected slots:
     virtual void doChangeInterval();
+    void sliderReleased(); // ### make virtual in 4.0
+#if (QT_VERSION >= 0x040000)
+#error "Some functions need to be changed to virtual for Qt 4.0"
+#endif
 
 private slots:
     void formatMore();
@@ -359,7 +468,7 @@ private slots:
 
 private:
     struct Q_EXPORT UndoRedoInfo {
-	enum Type { Invalid, Insert, Delete, Backspace, Return, RemoveSelected, Format, Style };
+	enum Type { Invalid, Insert, Delete, Backspace, Return, RemoveSelected, Format, Style, IME };
 
 	UndoRedoInfo( QTextDocument *dc );
 	~UndoRedoInfo();
@@ -388,8 +497,11 @@ private:
     void updateCurrentFormat();
     bool handleReadOnlyKeyEvent( QKeyEvent *e );
     void makeParagVisible( QTextParagraph *p );
+    void normalCopy();
+    void copyToClipboard();
 #ifndef QT_NO_MIME
     QCString pickSpecial(QMimeSource* ms, bool always_ask, const QPoint&);
+    QTextDrag *dragObject( QWidget *parent = 0 ) const;
 #endif
 #ifndef QT_NO_MIMECLIPBOARD
     void pasteSpecial(const QPoint&);
@@ -407,6 +519,42 @@ private:
     void placeCursor( const QPoint &pos, QTextCursor *c, bool link );
     void updateMicroFocusHint();
 
+#ifdef QT_TEXTEDIT_OPTIMIZATION
+    bool checkOptimMode();
+    QString optimText() const;
+    void optimSetText( const QString &str );
+    void optimAppend( const QString &str );
+    void optimInsert( const QString &str, int line, int index );
+    void optimDrawContents( QPainter * p, int cx, int cy, int cw, int ch );
+    void optimMousePressEvent( QMouseEvent * e );
+    void optimMouseReleaseEvent( QMouseEvent * e );
+    void optimMouseMoveEvent( QMouseEvent * e );
+    int  optimCharIndex( const QString &str, int mx ) const;
+    void optimSelectAll();
+    void optimRemoveSelection();
+    void optimSetSelection( int startLine, int startIdx, int endLine,
+			    int endIdx );
+    bool optimHasSelection() const;
+    QString optimSelectedText() const;
+    bool optimFind( const QString & str, bool, bool, bool, int *, int * );
+    void optimParseTags( QString * str, int lineNo = -1, int indexOffset = 0 );
+    QTextEditOptimPrivate::Tag * optimPreviousLeftTag( int line );
+    void optimSetTextFormat( QTextDocument *, QTextCursor *, QTextFormat * f,
+			     int, int, QTextEditOptimPrivate::Tag * t );
+    QTextEditOptimPrivate::Tag * optimAppendTag( int index, const QString & tag );
+    QTextEditOptimPrivate::Tag * optimInsertTag( int line, int index, const QString & tag );
+    void optimCheckLimit( const QString& str );
+    bool optimHasBoldMetrics( int line );
+
+private slots:
+    void optimDoAutoScroll();
+#endif // QT_TEXTEDIT_OPTIMIZATION
+
+private:
+#ifndef QT_NO_CLIPBOARD
+    void pasteSubType( const QCString &subtype, QMimeSource *m );
+#endif
+
 private:
     QTextDocument *doc;
     QTextCursor *cursor;
@@ -435,6 +583,12 @@ private:
     bool readonly : 1;
     bool undoEnabled : 1;
     bool overWrite : 1;
+
+private:	// Disabled copy constructor and operator=
+#if defined(Q_DISABLE_COPY)
+    QTextEdit( const QTextEdit & );
+    QTextEdit &operator=( const QTextEdit & );
+#endif
 };
 
 inline QTextDocument *QTextEdit::document() const
