@@ -1,11 +1,11 @@
 /****************************************************************************
-** $Id: qt/qworkspace.cpp   3.3.4   edited Jan 4 15:13 $
+** $Id: qt/qworkspace.cpp   3.3.7   edited Aug 31 2005 $
 **
 ** Implementation of the QWorkspace class
 **
 ** Created : 931107
 **
-** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 1992-2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the workspace module of the Qt GUI Toolkit.
 **
@@ -413,7 +413,7 @@ void QWorkspace::childEvent( QChildEvent * e)
 {
     if (e->inserted() && e->child()->isWidgetType()) {
 	QWidget* w = (QWidget*) e->child();
-	if ( !w || !w->testWFlags( WStyle_Title | WStyle_NormalBorder | WStyle_DialogBorder )
+	if ( !w || !w->testWFlags( WStyle_Title | WStyle_NormalBorder | WStyle_DialogBorder) || w->testWFlags(WType_Dialog)
 	     || d->icons.contains( w ) || w == d->vbar || w == d->hbar || w == d->corner )
 	    return;	    // nothing to do
 
@@ -458,7 +458,7 @@ void QWorkspace::childEvent( QChildEvent * e)
 	    w->showMaximized();
 	else if ( wasMinimized )
 	    w->showMinimized();
-	else
+        else if (!hasBeenHidden)
 	    activateWindow( w );
 
 	updateWorkspace();
@@ -553,10 +553,10 @@ void QWorkspace::activateWindow( QWidget* w, bool change_focus )
 		d->maxtools->setPixmap( pm );
 	    }
 	}
+    } else { // done implicitly in maximizeWindow otherwise        
+        d->active->internalRaise();
     }
 
-    d->active->internalRaise();
-
     if ( change_focus ) {
 	if ( d->focus.find( d->active ) >=0 ) {
 	    d->focus.removeRef( d->active );
