@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of Qt Designer.
 **
@@ -8,10 +8,16 @@
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.
 **
+** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
+** licenses may use this file in accordance with the Qt Commercial License
+** Agreement provided with the Software.
+**
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** See http://www.trolltech.com/gpl/ for GPL licensing information.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+**   information about Qt Commercial License Agreements.
 **
 ** Contact info@trolltech.com if any conditions of this licensing are
 ** not clear to you.
@@ -26,12 +32,13 @@
 #include "metadatabase.h"
 #include "resource.h"
 #include "layout.h"
-#include "connectioneditorimpl.h"
+#include "connectiondialog.h"
 #include <widgetdatabase.h>
 #include "pixmapchooser.h"
 #include "orderindicator.h"
 #include "hierarchyview.h"
 #include "designerappiface.h"
+#include "menubareditor.h"
 #define NO_STATIC_COLORS
 #include "globaldefs.h"
 
@@ -60,6 +67,15 @@
 #include <qaccel.h>
 #include <qpixmapcache.h>
 #include <qbitmap.h>
+#include <qsplitter.h>
+
+// FormWindow should be able to work to some limited degree
+// (existance, loading) without a MainWindow. Functions which require
+// a MainWindow in theory should never be called if no MainWindow
+// exists. These macros are used to let us know if that happens anyway
+// and to ensure that we don't crash
+#define CHECK_MAINWINDOW Q_ASSERT( mainWindow() ); if ( !mainWindow() ) return
+#define CHECK_MAINWINDOW_VALUE( v ) Q_ASSERT( mainWindow() ); if ( !mainWindow() ) return v
 
 static void setCursorToAll( const QCursor &c, QWidget *start )
 {
@@ -67,7 +83,7 @@ static void setCursorToAll( const QCursor &c, QWidget *start )
     QObjectList *l = (QObjectList*)start->children();
     if ( l ) {
 	for ( QObject *o = l->first(); o; o = l->next() ) {
-	    if ( o->isWidgetType() && !o->inherits( "SizeHandle" ) )
+	    if ( o->isWidgetType() && !::qt_cast<SizeHandle*>(o) )
 		setCursorToAll( c, ( (QWidget*)o ) );
 	}
     }
@@ -82,7 +98,7 @@ static void restoreCursors( QWidget *start, FormWindow *fw )
     QObjectList *l = (QObjectList*)start->children();
     if ( l ) {
 	for ( QObject *o = l->first(); o; o = l->next() ) {
-	    if ( o->isWidgetType() && !o->inherits( "SizeHandle" ) )
+	    if ( o->isWidgetType() && !::qt_cast<SizeHandle*>(o) )
 		restoreCursors( ( (QWidget*)o ), fw );
 	}
     }
@@ -133,6 +149,7 @@ FormWindow::FormWindow( FormFile *f, QWidget *parent, const char *name )
 
 void FormWindow::init()
 {
+    fake = qstrcmp( name(), "qt_fakewindow" ) == 0;
     MetaDataBase::addEntry( this );
     ff->setFormWindow( this );
     iface = 0;
@@ -141,7 +158,7 @@ void FormWindow::init()
     toolFixed = FALSE;
     checkedSelectionsForMove = FALSE;
     mContainer = 0;
-    connectSender = connectReceiver = 0;
+    startWidget = endWidget = 0;
     currTool = POINTER_TOOL;
     unclippedPainter = 0;
     widgetPressed = FALSE;
@@ -168,7 +185,7 @@ void FormWindow::init()
 	     this, SIGNAL( undoRedoChanged( bool, bool, const QString &, const QString & ) ) );
     propShowBlocked = FALSE;
 
-    setIcon( PixmapChooser::loadPixmap( "form.xpm", PixmapChooser::Mini ) );
+    setIcon( QPixmap::fromMimeSource( "designer_form.png" ) );
 
     connect( &commands, SIGNAL( modificationChanged( bool ) ),
 	     this, SLOT( modificationChanged( bool ) ) );
@@ -182,6 +199,7 @@ void FormWindow::init()
 
     defSpacing = BOXLAYOUT_DEFAULT_SPACING;
     defMargin = BOXLAYOUT_DEFAULT_MARGIN;
+    hasLayoutFunc = FALSE;
 }
 
 void FormWindow::setMainWindow( MainWindow *w )
@@ -193,35 +211,22 @@ void FormWindow::setMainWindow( MainWindow *w )
 
 void FormWindow::initSlots()
 {
-    QString lang = "C++";
-    if ( project() )
-	lang = project()->language();
-    else
-	lang = MainWindow::self->currProject()->language();
-    if ( lang != "C++" ) {
-	if ( !MetaDataBase::hasSlot( this, "init()" ) )
-	    MetaDataBase::addSlot( this, "init()", "", "protected", mainWindow()->currProject()->language(), "void" );
-	if ( !MetaDataBase::hasSlot( this, "destroy()" ) )
-	    MetaDataBase::addSlot( this, "destroy()", "", "protected", mainWindow()->currProject()->language(), "void" );
-    } else {
+    if ( isFake() )
+	return;
+    Q_ASSERT( project() || MainWindow::self );
+    if ( !project() && !MainWindow::self )
+	return;
+    Project *p = project() ? project() : MainWindow::self->currProject();
+    if ( p && p->isCpp() ) {
 	QString code = formFile()->code();
-	if ( code.isEmpty() ) {
-	    code =
-		"/****************************************************************************\n"
-		"** ui.h extension file, included from the uic-generated form implementation.\n"
-		"**\n"
-		"** If you wish to add, delete or rename slots use Qt Designer which will\n"
-		"** update this file, preserving your code. Create an init() slot in place of\n"
-		"** a constructor, and a destroy() slot in place of a destructor.\n"
-		"*****************************************************************************/\n";
-	    formFile()->setCode( code );
-	}
+	if ( code.isEmpty() )
+	    formFile()->setCode( formFile()->codeComment() );
     }
 }
 
 FormWindow::~FormWindow()
 {
-    if ( MainWindow::self->objectHierarchy()->formWindow() == this )
+    if ( MainWindow::self && MainWindow::self->objectHierarchy()->formWindow() == this )
 	MainWindow::self->objectHierarchy()->setFormWindow( 0, 0 );
 
     MetaDataBase::clear( this );
@@ -232,7 +237,8 @@ FormWindow::~FormWindow()
 
 void FormWindow::closeEvent( QCloseEvent *e )
 {
-    if ( ff->closeEvent() && mainwindow->unregisterClient( this ) )
+    QGuardedPtr<FormWindow> that = this;
+    if ( ff->closeEvent() && ( !that || ( mainwindow && mainwindow->unregisterClient( this ) ) ) )
 	e->accept();
     else
 	e->ignore();
@@ -240,7 +246,7 @@ void FormWindow::closeEvent( QCloseEvent *e )
 
 void FormWindow::paintGrid( QWidget *w, QPaintEvent *e )
 {
-    if ( !mainWindow()->showGrid() )
+    if ( !mainWindow() || !mainWindow()->showGrid() )
 	return;
     QPixmap grid;
     QString grid_name;
@@ -332,6 +338,7 @@ void FormWindow::drawSizePreview( const QPoint &pos, const QString& text )
 
 void FormWindow::insertWidget()
 {
+    CHECK_MAINWINDOW;
     if ( !insertParent )
 	return;
 
@@ -355,8 +362,12 @@ void FormWindow::insertWidget()
     if ( !w )
 	return;
 
-    if ( !savePixmapInline() && currTool == WidgetDatabase::idFromClassName( "PixmapLabel" ) ) // ### what to do for pixmaps in project
-	( (QLabel*)w )->setPixmap( PixmapChooser::loadPixmap( "image.xpm" ) );
+    if ( !savePixmapInline() && currTool == WidgetDatabase::idFromClassName( "PixmapLabel" ) ) { // ### what to do for pixmaps in project
+	QPixmap pix;
+	// we have to force the pixmap to get a new and unique serial number. Unfortunately detatch() doesn't do that
+	pix.convertFromImage( QPixmap::fromMimeSource( "designer_image.png" ).convertToImage() );
+	( (QLabel*)w )->setPixmap( pix );
+    }
     int id = WidgetDatabase::idFromClassName( WidgetFactory::classNameOf(w) );
     if ( WidgetDatabase::isCustomWidget( id ) ) {
 	QWhatsThis::add( w, tr("<b>A %1 (custom widget)</b> "
@@ -389,8 +400,18 @@ void FormWindow::insertWidget()
     r = QRect( p, r.size() );
 
     if ( useSizeHint ) {
-	r.setWidth( w->sizeHint().width() );
-	r.setHeight( w->sizeHint().height() );
+	if ( n == "Spacer" ) {
+	    if ( orient == Vertical ) {
+		r.setWidth( 20 );
+		r.setHeight( 40 );
+	    } else {
+		r.setWidth( 40 );
+		r.setHeight( 20 );
+	    }
+	} else {
+	    r.setWidth( w->sizeHint().width() );
+	    r.setHeight( w->sizeHint().height() );
+	}
     }
 
     if ( r.width() < 2 * grid().x() )
@@ -418,30 +439,33 @@ void FormWindow::insertWidget()
 
     if ( !lst.isEmpty() ) {
 	QWidget *pw = WidgetFactory::containerOfWidget( w );
-	QValueList<QPoint> op, np;
-	for ( QWidget *i = lst.first(); i; i = lst.next() ) {
-	    op.append( i->pos() );
-	    QPoint pos = pw->mapFromGlobal( i->mapToGlobal( QPoint( 0, 0 ) ) );
-	    pos -= r.topLeft();
-	    np.append( pos );
-	}
+        if (pw) {
+            QValueList<QPoint> op, np;
+            for ( QWidget *i = lst.first(); i; i = lst.next() ) {
+                op.append( i->pos() );
+                QPoint pos = pw->mapFromGlobal( i->mapToGlobal( QPoint( 0, 0 ) ) );
+                pos -= r.topLeft();
+                np.append( pos );
+            }
+            
+            MoveCommand *mv = new MoveCommand( tr( "Reparent Widgets" ), this,
+                                               lst, op, np, insertParent, pw );
 
-	MoveCommand *mv = new MoveCommand( tr( "Reparent Widgets" ), this, lst, op, np, insertParent, pw );
-
-	if ( !toolFixed )
-	    mainwindow->resetTool();
-	else
-	    setCursorToAll( CrossCursor, w );
-
-	InsertCommand *cmd = new InsertCommand( tr( "Insert %1" ).arg( w->name() ), this, w, r );
-
-	QPtrList<Command> commands;
-	commands.append( mv );
-	commands.append( cmd );
-
-	MacroCommand *mc = new MacroCommand( tr( "Insert %1" ).arg( w->name() ), this, commands );
-	commandHistory()->addCommand( mc );
-	mc->execute();
+            if ( !toolFixed )
+                mainwindow->resetTool();
+            else
+                setCursorToAll( CrossCursor, w );
+            
+            InsertCommand *cmd = new InsertCommand( tr( "Insert %1" ).arg( w->name() ), this, w, r );
+            
+            QPtrList<Command> commands;
+            commands.append( mv );
+            commands.append( cmd );
+            
+            MacroCommand *mc = new MacroCommand( tr( "Insert %1" ).arg( w->name() ), this, commands );
+            commandHistory()->addCommand( mc );
+            mc->execute();
+        }
     } else {
 	if ( !toolFixed )
 	    mainwindow->resetTool();
@@ -462,6 +486,8 @@ void FormWindow::insertWidget()
 
 void FormWindow::insertWidget( QWidget *w, bool checkName )
 {
+    if ( !w )
+	return;
     if ( checkName ) {
 	QString s = w->name();
 	unify( w, s, TRUE );
@@ -499,6 +525,7 @@ void FormWindow::removeWidget( QWidget *w )
 
 void FormWindow::handleContextMenu( QContextMenuEvent *e, QWidget *w )
 {
+    CHECK_MAINWINDOW;
     switch ( currTool ) {
     case POINTER_TOOL: {
 	if ( !isMainContainer( w ) && qstrcmp( w->name(), "central widget" ) != 0 ) { // press on a child widget
@@ -510,8 +537,7 @@ void FormWindow::handleContextMenu( QContextMenuEvent *e, QWidget *w )
 		    ( WidgetFactory::layoutType( w->parentWidget()) != WidgetFactory::NoLayout ||
 		      !insertedWidgets.find(w) ) )
 		w = w->parentWidget();
-	    if ( mainContainer()->inherits( "QMainWindow" ) &&
-		 ( (QMainWindow*)mainContainer() )->centralWidget() == realWidget ) {
+	    if ( ::qt_cast<QMainWindow*>(mainContainer()) && ((QMainWindow*)mainContainer())->centralWidget() == realWidget ) {
 		e->accept();
 		mainwindow->popupFormWindowMenu( e->globalPos(), this );
 	    } else {
@@ -531,6 +557,7 @@ void FormWindow::handleContextMenu( QContextMenuEvent *e, QWidget *w )
 
 void FormWindow::handleMousePress( QMouseEvent *e, QWidget *w )
 {
+    CHECK_MAINWINDOW;
     checkedSelectionsForMove = FALSE;
     checkSelectionsTimer->stop();
     if ( !sizePreviewLabel ) {
@@ -617,16 +644,26 @@ void FormWindow::handleMousePress( QMouseEvent *e, QWidget *w )
 	}
 	break;
     case CONNECT_TOOL:
+    case BUDDY_TOOL:
 	if ( e->button() != LeftButton )
 	    break;
+	validForBuddy = FALSE;
+	if ( currTool == BUDDY_TOOL ) {
+	    if ( !::qt_cast<QLabel*>(w) )
+		break;
+	    clearSelection( FALSE );
+	    validForBuddy = TRUE;
+	    mainWindow()->statusBar()->message( tr( "Set buddy for '%1' to..." ).arg( w->name() ) );
+	} else {
+	    mainWindow()->statusBar()->message( tr( "Connect '%1' with..." ).arg( w->name() ) );
+	}
 	saveBackground();
-	mainWindow()->statusBar()->message( tr( "Connect '%1' with..." ).arg( w->name() ) );
-	connectStartPos = mapFromGlobal( e->globalPos() );
-	currentConnectPos = mapFromGlobal( e->globalPos() );
-	connectSender = designerWidget( w );
-	connectReceiver = connectableObject( designerWidget( w ), connectReceiver );
+	startPos = mapFromGlobal( e->globalPos() );
+	currentPos = startPos;
+	startWidget = designerWidget( w );
+	endWidget = startWidget;
 	beginUnclippedPainter( FALSE );
-	drawConnectLine();
+	drawConnectionLine();
 	break;
     case ORDER_TOOL:
 	if ( !isMainContainer( w ) ) { // press on a child widget
@@ -654,7 +691,7 @@ void FormWindow::handleMousePress( QMouseEvent *e, QWidget *w )
 		for (;;) {
 		    int id = WidgetDatabase::idFromClassName( WidgetFactory::classNameOf( wid ) );
 		    if ( ( WidgetDatabase::isContainer( id ) || wid == mainContainer() ) &&
-			 !wid->inherits( "QLayoutWidget" ) && !wid->inherits( "QSplitter" ) ) {
+			 !::qt_cast<QLayoutWidget*>(wid) && !::qt_cast<QSplitter*>(wid) ) {
 			insertParent = WidgetFactory::containerOfWidget( wid ); // found another parent, store it
 			break;
 		    } else {
@@ -672,6 +709,7 @@ void FormWindow::handleMousePress( QMouseEvent *e, QWidget *w )
 
 void FormWindow::handleMouseDblClick( QMouseEvent *, QWidget *w )
 {
+    CHECK_MAINWINDOW;
     switch ( currTool ) {
     case ORDER_TOOL:
 	if ( !isMainContainer( w ) ) { // press on a child widget
@@ -700,10 +738,11 @@ void FormWindow::handleMouseDblClick( QMouseEvent *, QWidget *w )
 
 void FormWindow::handleMouseMove( QMouseEvent *e, QWidget *w )
 {
+    CHECK_MAINWINDOW;
     if ( ( e->state() & LeftButton ) != LeftButton )
 	return;
 
-    QWidget *newReceiver = (QWidget*)connectReceiver, *oldReceiver = (QWidget*)connectReceiver, *wid;
+    QWidget *newendWidget = endWidget, *oldendWidget = endWidget, *wid;
     bool drawRecRect;
     switch ( currTool ) {
     case POINTER_TOOL:
@@ -762,8 +801,8 @@ void FormWindow::handleMouseMove( QMouseEvent *e, QWidget *w )
 		}
 
 		// check whether we would have to reparent the selection and highlight the possible new parent container
-		QMapConstIterator<ulong, QPoint> it = moving.begin();
-		QWidget* wa = containerAt( e->globalPos(), ( (QWidget*)it.key() ) );
+		QMapConstIterator<QWidget*, QPoint> it = moving.begin();
+		QWidget* wa = containerAt( e->globalPos(), it.key() );
 		if ( wa  && !isMainContainer( wa ) && !isCentralWidget( wa ) ) {
 		    wa = WidgetFactory::containerOfWidget( wa );
 		    // ok, looks like we moved onto a container
@@ -818,22 +857,51 @@ void FormWindow::handleMouseMove( QMouseEvent *e, QWidget *w )
 	if ( wid )
 	    wid = designerWidget( wid );
 	if ( wid && ( isMainContainer( wid ) || insertedWidgets.find( wid ) ) && wid->isVisibleTo( this ) )
-	    newReceiver = wid;
-	if ( newReceiver &&
-	     ( newReceiver->inherits( "QLayoutWidget" ) || newReceiver->inherits( "Spacer" ) ) )
-	    newReceiver = (QWidget*)connectReceiver;
-	drawRecRect = newReceiver != connectReceiver;
-	currentConnectPos = mapFromGlobal( e->globalPos() );
-	if ( newReceiver &&
-	     ( isMainContainer( newReceiver ) || insertedWidgets.find( newReceiver ) ) && !isCentralWidget( newReceiver ) )
-	    connectReceiver = connectableObject( newReceiver, connectReceiver );
-	mainWindow()->statusBar()->message( tr( "Connect '%1' to '%2'" ).arg( connectSender->name() ).
-					    arg( connectReceiver->name() ) );
+	    newendWidget = wid;
+	if ( ::qt_cast<QLayoutWidget*>(newendWidget) || ::qt_cast<Spacer*>(newendWidget) )
+	    newendWidget = (QWidget*)endWidget;
+	drawRecRect = newendWidget != endWidget;
+	if ( newendWidget &&
+	     ( isMainContainer( newendWidget ) || insertedWidgets.find( newendWidget ) ) && !isCentralWidget( newendWidget ) )
+	    endWidget = newendWidget;
+	mainWindow()->statusBar()->message( tr( "Connect '%1' to '%2'" ).arg( startWidget->name() ).
+					    arg( endWidget->name() ) );
+	currentPos = mapFromGlobal( e->globalPos() );
 	qApp->processEvents();
 	if ( drawRecRect )
-	    restoreRect( QRect( mapToForm(  ( (QWidget*)oldReceiver )->parentWidget(), ( (QWidget*)oldReceiver )->pos() ),
-				( (QWidget*)oldReceiver )->size() ) );
-	drawConnectLine();
+	    restoreRect( QRect( mapToForm( ( (QWidget*)oldendWidget )->parentWidget(), ( (QWidget*)oldendWidget )->pos() ),
+			 ( (QWidget*)oldendWidget )->size() ) );
+	drawConnectionLine();
+	break;
+    case BUDDY_TOOL:
+	if ( !validForBuddy )
+	    break;
+	restoreConnectionLine();
+	wid = qApp->widgetAt( e->globalPos(), TRUE );
+	if ( wid )
+	    wid = designerWidget( wid );
+	if ( wid && canBeBuddy( wid ) && wid->isVisibleTo( this ) )
+	    newendWidget = wid;
+	else
+	    newendWidget = 0;
+	if ( ::qt_cast<QLayoutWidget*>(newendWidget) || ::qt_cast<Spacer*>(newendWidget) )
+	    newendWidget = (QWidget*)endWidget;
+	drawRecRect = newendWidget != endWidget;
+	if ( !newendWidget )
+	    endWidget = newendWidget;
+	else if ( insertedWidgets.find( newendWidget ) && !isCentralWidget( newendWidget ) )
+	    endWidget = newendWidget;
+	if ( endWidget )
+	    mainWindow()->statusBar()->message( tr( "Set buddy '%1' to '%2'" ).arg( startWidget->name() ).
+						arg( endWidget->name() ) );
+	else
+	    mainWindow()->statusBar()->message( tr( "Set buddy '%1' to ..." ).arg( startWidget->name() ) );
+	currentPos = mapFromGlobal( e->globalPos() );
+	qApp->processEvents();
+	if ( drawRecRect && oldendWidget )
+	    restoreRect( QRect( mapToForm( ( (QWidget*)oldendWidget )->parentWidget(), ( (QWidget*)oldendWidget )->pos() ),
+			 ( (QWidget*)oldendWidget )->size() ) );
+	drawConnectionLine();
 	break;
     case ORDER_TOOL:
 	break;
@@ -846,6 +914,7 @@ void FormWindow::handleMouseMove( QMouseEvent *e, QWidget *w )
 
 void FormWindow::handleMouseRelease( QMouseEvent *e, QWidget *w )
 {
+    CHECK_MAINWINDOW;
     if ( e->button() != LeftButton )
 	return;
 
@@ -854,7 +923,7 @@ void FormWindow::handleMouseRelease( QMouseEvent *e, QWidget *w )
 	if ( widgetPressed && allowMove( w ) ) { // we moved the widget
 	    sizePreviewLabel->hide();
 
-	    if ( moving.isEmpty() || w->pos() == *moving.find( (ulong)w ) )
+	    if ( moving.isEmpty() || w->pos() == *moving.find(w) )
 		break;
 
 	    // restore targetContainer
@@ -869,17 +938,17 @@ void FormWindow::handleMouseRelease( QMouseEvent *e, QWidget *w )
 	    if ( propertyWidget && propertyWidget->isWidgetType() && !isMainContainer( propertyWidget ) )
 		emitUpdateProperties( propertyWidget );
 
-	    QMapConstIterator<ulong,QPoint> it = moving.begin();
-	    QWidget *oldParent = ( (QWidget*)it.key() )->parentWidget();
+	    QMapConstIterator<QWidget*,QPoint> it = moving.begin();
+	    QWidget *oldParent = it.key()->parentWidget();
 	    QWidget *newParent = oldParent;
 	    // check whether we have to reparent the selection
-	    QWidget* wa = containerAt( e->globalPos(), ( (QWidget*)it.key() ) );
+	    QWidget* wa = containerAt( e->globalPos(), it.key() );
 	    if ( wa ) {
 		wa = WidgetFactory::containerOfWidget( wa );
 		// ok, looks like we moved onto a container
 
 		// check whether we really have different parents.
-		if ( wa == ( (QWidget*)it.key() )->parentWidget() )
+		if ( wa == it.key()->parentWidget() )
 		    goto make_move_command;
 
 		// break layout if necessary
@@ -899,15 +968,14 @@ void FormWindow::handleMouseRelease( QMouseEvent *e, QWidget *w )
 
 		// doesn't need to be a command, the MoveCommand does reparenting too
 		bool emitSelChanged = FALSE;
-		for ( QMap<ulong, QPoint>::Iterator it = moving.begin(); it != moving.end(); ++it ) {
-		    QWidget *i = (QWidget*)it.key();
-		    if ( !emitSelChanged && i->inherits( "QButton" )  ) {
-			if ( i->parentWidget() && i->parentWidget()->inherits( "QButtonGroup" ) ||
-			     wa->inherits( "QButtonGroup" ) )
+		for ( QMap<QWidget*, QPoint>::Iterator it = moving.begin(); it != moving.end(); ++it ) {
+		    QWidget *i = it.key();
+		    if ( !emitSelChanged && ::qt_cast<QButton*>(i) ) {
+			if ( ::qt_cast<QButtonGroup*>(i->parentWidget()) || ::qt_cast<QButtonGroup*>(wa) )
 			    emitSelChanged = TRUE;
-			if ( !wa->inherits( "QButtonGroup" ) ) {
+			if ( !::qt_cast<QButtonGroup*>(wa) ) {
 			    MetaDataBase::setPropertyChanged( i, "buttonGroupId", FALSE );
-			    if ( i->parentWidget() && i->parentWidget()->inherits( "QButtonGroup" ) )
+			    if ( ::qt_cast<QButtonGroup*>(i->parentWidget()) )
 				( (QButtonGroup*)i->parentWidget() )->remove( (QButton*)i );
 			}
 		    }
@@ -930,9 +998,9 @@ void FormWindow::handleMouseRelease( QMouseEvent *e, QWidget *w )
 	    QWidgetList widgets; // collect the widgets and its old and new positions which have been moved
 	    QValueList<QPoint> oldPos, newPos;
 	    for ( it = moving.begin(); it != moving.end(); ++it ) {
-		widgets.append( (QWidget*)it.key() );
+		widgets.append( it.key() );
 		oldPos.append( *it );
-		newPos.append( ( (QWidget*)it.key() )->pos() );
+		newPos.append( it.key()->pos() );
 	    }
 	    // add move command, don't execute it, this is just a summary of the operations we did during the move-event handling
 	    commandHistory()->addCommand( new MoveCommand( tr( "Move" ),
@@ -949,19 +1017,40 @@ void FormWindow::handleMouseRelease( QMouseEvent *e, QWidget *w )
 	}
 	break;
     case CONNECT_TOOL:
+    case BUDDY_TOOL:
 	restoreConnectionLine();
-	if ( connectSender )
-	    restoreRect( QRect( mapToForm(  ( (QWidget*)connectSender )->parentWidget(),
-					    ( (QWidget*)connectSender )->pos() ),
-				( (QWidget*)connectSender )->size() ) );
-	if ( connectReceiver )
-	    restoreRect( QRect( mapToForm( ( (QWidget*)connectReceiver )->parentWidget(),
-					   ( (QWidget*)connectReceiver )->pos() ),
-				( (QWidget*)connectReceiver )->size() ) );
+	if ( startWidget )
+	    restoreRect( QRect( mapToForm( ( (QWidget*)startWidget )->parentWidget(),
+					   ( (QWidget*)startWidget )->pos() ),
+				((QWidget*)startWidget )->size() ) );
+	if ( endWidget )
+	    restoreRect( QRect( mapToForm( ( (QWidget*)endWidget )->parentWidget(),
+					   ( (QWidget*)endWidget )->pos() ),
+				( (QWidget*)endWidget )->size() ) );
 	endUnclippedPainter();
 	qApp->processEvents();
-	if ( connectSender && connectReceiver )
-	    editConnections();
+
+	if ( startWidget && endWidget ) {
+	    if ( currTool == CONNECT_TOOL )
+		editConnections();
+	    else if ( currTool == BUDDY_TOOL && validForBuddy && startWidget != endWidget ) {
+		QString oldBuddy = startWidget->property( "buddy" ).toString();
+		if ( oldBuddy.isNull() )
+		    oldBuddy = "";
+		SetPropertyCommand *cmd = new SetPropertyCommand( tr( "Set buddy for " + QString( startWidget->name() ) ),
+								  this, startWidget, mainWindow()->propertyeditor(),
+								  "buddy", startWidget->property( "buddy" ),
+								  endWidget->name(), endWidget->name(),
+								  oldBuddy );
+		commandHistory()->addCommand( cmd, TRUE );
+		cmd->execute();
+		emitUpdateProperties( startWidget );
+	    }
+	}
+	if ( !toolFixed )
+	    mainwindow->resetTool();
+	startWidget = endWidget = 0;
+	mainWindow()->statusBar()->clear();
 	break;
     case ORDER_TOOL:
 	break;
@@ -998,6 +1087,7 @@ void FormWindow::handleMouseRelease( QMouseEvent *e, QWidget *w )
 
 void FormWindow::handleKeyPress( QKeyEvent *e, QWidget *w )
 {
+    CHECK_MAINWINDOW;
     e->ignore();
     checkSelectionsTimer->stop();
     if ( !checkedSelectionsForMove &&
@@ -1080,6 +1170,7 @@ void FormWindow::handleKeyRelease( QKeyEvent *e, QWidget * )
 
 void FormWindow::selectWidget( QObject *o, bool select )
 {
+    CHECK_MAINWINDOW;
     if ( !o->isWidgetType() ) {
 	// ########### do QObject stuff
 	return;
@@ -1096,8 +1187,7 @@ void FormWindow::selectWidget( QObject *o, bool select )
 	return;
     }
 
-    if ( mainContainer()->inherits( "QMainWindow" ) &&
-	 w == ( (QMainWindow*)mainContainer() )->centralWidget() ) {
+    if ( ::qt_cast<QMainWindow*>(mainContainer()) && w == ( (QMainWindow*)mainContainer() )->centralWidget() ) {
 	QObject *opw = propertyWidget;
 	propertyWidget = mainContainer();
 	if ( opw->isWidgetType() )
@@ -1106,7 +1196,7 @@ void FormWindow::selectWidget( QObject *o, bool select )
 	return;
     }
 
-    if ( o->inherits( "QDesignerToolBar" ) || o->inherits( "QDesignerMenuBar" ) )
+    if ( ::qt_cast<QDesignerToolBar*>(o) )
 	return;
 
     if ( select ) {
@@ -1154,7 +1244,7 @@ void FormWindow::selectWidget( QObject *o, bool select )
 
 QPoint FormWindow::grid() const
 {
-    if ( !mainWindow()->snapGrid() )
+    if ( !mainWindow() || !mainWindow()->snapGrid() )
 	return QPoint( 1, 1 );
     return mainWindow()->grid();
 }
@@ -1368,7 +1458,7 @@ void FormWindow::checkSelectionsForMove( QWidget *w )
 		    sel->setWidget( 0 );
 	    } else {
 		if ( WidgetFactory::layoutType( sel->widget()->parentWidget() ) == WidgetFactory::NoLayout ) {
-		    moving.insert( (ulong)sel->widget(), sel->widget()->pos() );
+		    moving.insert( sel->widget(), sel->widget()->pos() );
 		    sel->widget()->raise();
 		    raiseChildSelections( sel->widget() );
 		    raiseSelection( sel->widget() );
@@ -1381,6 +1471,7 @@ void FormWindow::checkSelectionsForMove( QWidget *w )
 
 void FormWindow::deleteWidgets()
 {
+    CHECK_MAINWINDOW;
     QWidgetList widgets;
     QPtrDictIterator<WidgetSelection> it( usedSelections );
     for ( ; it.current(); ++it ) {
@@ -1483,8 +1574,10 @@ void FormWindow::checkPreviewGeometry( QRect &r )
     }
 }
 
-void FormWindow::focusInEvent( QFocusEvent * )
-{
+void FormWindow::focusInEvent( QFocusEvent * ){
+
+    if (ff)
+	ff->checkTimeStamp();
 }
 
 void FormWindow::focusOutEvent( QFocusEvent * )
@@ -1565,12 +1658,14 @@ void FormWindow::emitSelectionChanged()
 
 void FormWindow::updatePropertiesTimerDone()
 {
+    CHECK_MAINWINDOW;
     if ( propertyWidget && mainWindow()->formWindow() == this )
 	emit updateProperties( propertyWidget );
 }
 
 void FormWindow::showPropertiesTimerDone()
 {
+    CHECK_MAINWINDOW;
     if ( propertyWidget && mainWindow()->formWindow() == this )
 	emit showProperties( propertyWidget );
 }
@@ -1582,6 +1677,7 @@ void FormWindow::selectionChangedTimerDone()
 
 void FormWindow::currentToolChanged()
 {
+    CHECK_MAINWINDOW;
     toolFixed = FALSE;
     int t = mainwindow->currentTool();
     if ( currTool == t && t != ORDER_TOOL )
@@ -1593,15 +1689,16 @@ void FormWindow::currentToolChanged()
 	hideOrderIndicators();
 	break;
     case CONNECT_TOOL:
+    case BUDDY_TOOL:
 	restoreConnectionLine();
-	if ( connectSender )
-	    restoreRect( QRect( mapToForm(  ( (QWidget*)connectSender )->parentWidget(),
-					    ( (QWidget*)connectSender )->pos() ),
-				( (QWidget*)connectSender )->size() ) );
-	if ( connectReceiver )
-	    restoreRect( QRect( mapToForm( ( (QWidget*)connectReceiver )->parentWidget(),
-					   ( (QWidget*)connectReceiver )->pos() ),
-				( (QWidget*)connectReceiver )->size() ) );
+	if ( startWidget )
+	    restoreRect( QRect( mapToForm( ( (QWidget*)startWidget )->parentWidget(),
+					   ( (QWidget*)startWidget )->pos() ),
+				( (QWidget*)startWidget )->size() ) );
+	if ( endWidget )
+	    restoreRect( QRect( mapToForm( ( (QWidget*)endWidget )->parentWidget(),
+					   ( (QWidget*)endWidget )->pos() ),
+				( (QWidget*)endWidget )->size() ) );
 	endUnclippedPainter();
 	break;
     case POINTER_TOOL:
@@ -1612,7 +1709,7 @@ void FormWindow::currentToolChanged()
 	break;
     }
 
-    connectSender = connectReceiver = 0;
+    startWidget = endWidget = 0;
     widgetPressed = FALSE;
     drawRubber = FALSE;
     insertParent = 0;
@@ -1644,7 +1741,11 @@ void FormWindow::currentToolChanged()
 	}
 	break;
     case CONNECT_TOOL:
-	mainWindow()->statusBar()->message( tr( "Drag a line to create a connection...") );
+    case BUDDY_TOOL:
+	if ( currTool == CONNECT_TOOL )
+	    mainWindow()->statusBar()->message( tr( "Drag a line to create a connection...") );
+	else
+	    mainWindow()->statusBar()->message( tr( "Drag a line to set a buddy...") );
 	setCursorToAll( CrossCursor, this );
 	if ( mainWindow()->formWindow() == this )
 	    emitShowProperties( mainContainer() );
@@ -1668,7 +1769,7 @@ void FormWindow::showOrderIndicators()
 	int order = 1;
 	for ( QObject *o = l->first(); o; o = l->next() ) {
 	    QWidget* w = (QWidget*) o;
-	    if ( w->isVisibleTo( w->parentWidget() ) &&
+	    if ( w->isShown() &&
 		 insertedWidgets[ (void*)w ]  &&
 		 w->focusPolicy() != NoFocus ) {
 		OrderIndicator* ind = new OrderIndicator( order++, w, this );
@@ -1752,6 +1853,7 @@ int FormWindow::numSelectedWidgets() const
 
 QString FormWindow::copy()
 {
+    CHECK_MAINWINDOW_VALUE( QString::null );
     Resource resource( mainWindow() );
     resource.setWidget( this );
     return resource.copy();
@@ -1769,7 +1871,7 @@ void FormWindow::lowerWidgets()
     commandHistory()->addCommand( cmd );
 }
 
-static void find_accel( const QString &txt, QMap<QChar, QWidgetList > &accels, QWidget *w )
+void find_accel( const QString &txt, QMap<QChar, QWidgetList > &accels, QWidget *w )
 {
     int i = txt.find( "&" );
     if ( i == -1 )
@@ -1791,6 +1893,7 @@ static void find_accel( const QString &txt, QMap<QChar, QWidgetList > &accels, Q
 
 void FormWindow::checkAccels()
 {
+    CHECK_MAINWINDOW;
     QMap<QChar, QWidgetList > accels;
     QObjectList *l = mainContainer()->queryList( "QWidget" );
     if ( l ) {
@@ -1810,6 +1913,8 @@ void FormWindow::checkAccels()
 		    find_accel( w->property( "title" ).toString(), accels, w );
 		if ( pageTitle )
 		    find_accel( w->property( "pageTitle" ).toString(), accels, w );
+	    } else if ( ::qt_cast<MenuBarEditor*>(o) ) {
+		((MenuBarEditor *)o)->checkAccels( accels );
 	    }
 	}
 	delete l;
@@ -1824,7 +1929,7 @@ void FormWindow::checkAccels()
 					       tr( "Accelerator '%1' is used %2 times."
 						   ).arg( it.key().upper() ).arg( (*it).count() ),
 					       tr( "&Select" ),
-					       tr( "&Cancel" ), 0, 2 ) ) {
+					       tr( "&Cancel" ), QString::null, 2 ) ) {
 	    case 0: // select
 		clearSelection( FALSE );
 		for ( wid = (*it).first(); wid; wid = (*it).next() )
@@ -1855,6 +1960,7 @@ void FormWindow::raiseWidgets()
 
 void FormWindow::paste( const QString &cb, QWidget *parent )
 {
+    CHECK_MAINWINDOW;
     Resource resource( mainWindow() );
     resource.setWidget( this );
     resource.paste( cb, parent );
@@ -2016,7 +2122,7 @@ void FormWindow::breakLayout( QWidget *w )
 	    Command *cmd = breakLayoutCommand( w );
 	    if ( cmd )
 		commands.insert( 0, cmd );
-	    if ( !w->inherits( "QLayoutWidget" ) && !w->inherits( "QSplitter" ) )
+	    if ( !::qt_cast<QLayoutWidget*>(w) && !::qt_cast<QSplitter*>(w) )
 		break;
 	}
 	w = w->parentWidget();
@@ -2033,6 +2139,7 @@ void FormWindow::breakLayout( QWidget *w )
 
 BreakLayoutCommand *FormWindow::breakLayoutCommand( QWidget *w )
 {
+    CHECK_MAINWINDOW_VALUE( 0 );
     QObjectList *l = (QObjectList*)w->children();
     if ( !l )
 	return 0;
@@ -2099,17 +2206,16 @@ bool FormWindow::allowMove( QWidget *w )
 
 void FormWindow::editConnections()
 {
+    CHECK_MAINWINDOW;
     buffer = 0;
-    if ( !connectSender || !connectReceiver )
+    if ( !startWidget || !endWidget )
 	return;
-    mainWindow()->statusBar()->clear();
-    ConnectionEditor editor( mainwindow, connectSender, connectReceiver, this );
+
+    ConnectionDialog dlg( mainwindow );
     mainWindow()->statusBar()->message( tr( "Edit connections...") );
-    editor.exec();
-    mainWindow()->statusBar()->clear();
-    if ( !toolFixed )
-	mainwindow->resetTool();
-    connectSender = connectReceiver = 0;
+    dlg.addConnection( startWidget, endWidget, QString::null, QString::null );
+    QTimer::singleShot( 0, &dlg, SLOT(ensureConnectionVisible()) );
+    dlg.exec();
 }
 
 void FormWindow::saveBackground()
@@ -2122,12 +2228,12 @@ void FormWindow::saveBackground()
 
 void FormWindow::restoreConnectionLine()
 {
-    if (!unclippedPainter)
+    if (!unclippedPainter || !buffer) // designer will occasionally crash if buffer is not tested to be non-zero
 	return;
 
-    int a =QABS( connectStartPos.x() - currentConnectPos.x() );
-    int b = QABS( connectStartPos.y() - currentConnectPos.y() );
-    QRect r( connectStartPos, currentConnectPos );
+    int a =QABS( startPos.x() - currentPos.x() );
+    int b = QABS( startPos.y() - currentPos.y() );
+    QRect r( startPos, currentPos );
 
     if ( a < 32 || b < 32 ) { // special case: vertical or horizontal line
 	r = r.normalize();
@@ -2151,7 +2257,7 @@ void FormWindow::restoreConnectionLine()
 
     int dx = 2 * w / 3;
     int dy = 2 * h / 3;
-    QPoint p( connectStartPos );
+    QPoint p( startPos );
 
     if ( r.x() > r.right() ) {
 	dx = dx * -1;
@@ -2173,13 +2279,13 @@ void FormWindow::restoreConnectionLine()
 	p.setY( p.y() + dy );
     }
 
-    unclippedPainter->drawPixmap( connectStartPos.x() - 10, connectStartPos.y() - 10, *buffer,
-				  connectStartPos.x() - 10, connectStartPos.y() - 10, 20, 20 );
+    unclippedPainter->drawPixmap( startPos.x() - 10, startPos.y() - 10, *buffer,
+				  startPos.x() - 10, startPos.y() - 10, 20, 20 );
 }
 
 void FormWindow::restoreRect( const QRect &rect )
 {
-    if (!unclippedPainter)
+    if (!unclippedPainter || !buffer)
 	return;
 
     QRect r( rect );
@@ -2193,26 +2299,32 @@ void FormWindow::restoreRect( const QRect &rect )
     unclippedPainter->drawPixmap( r.x() + r.width() - 2, r.y(), *buffer, r.x() + r.width() - 2, r.y(), 4, r.height() + 4 );
 }
 
-void FormWindow::drawConnectLine()
+void FormWindow::drawConnectionLine()
 {
     if ( !unclippedPainter )
 	return;
 
     unclippedPainter->setPen( QPen( white, 2 ) );
-    unclippedPainter->drawLine( connectStartPos, currentConnectPos );
-    unclippedPainter->setPen( QPen( darkCyan, 1 ) );
-    unclippedPainter->drawLine( connectStartPos, currentConnectPos );
+    unclippedPainter->drawLine( startPos, currentPos );
+    if ( validForBuddy )
+	unclippedPainter->setPen( QPen( darkRed, 1 ) );
+    else
+	unclippedPainter->setPen( QPen( darkCyan, 1 ) );
+    unclippedPainter->drawLine( startPos, currentPos );
 
-    unclippedPainter->setPen( QPen( magenta, 1 ) );
-    if ( connectSender ) {
-	QWidget *w = (QWidget*)connectSender;
-	QPoint p = mapToForm( w, QPoint(0,0) );
-	unclippedPainter->drawRect( QRect( p + QPoint( 2, 2 ), w->size() - QSize( 4, 4 ) ) );
+    if ( validForBuddy )
+	unclippedPainter->setPen( QPen( darkGreen, 1 ) );
+    else
+	unclippedPainter->setPen( QPen( magenta, 1 ) );
+    if ( startWidget ) {
+	QWidget *s = (QWidget*)startWidget;
+	QPoint p = mapToForm( s, QPoint(0,0) );
+	unclippedPainter->drawRect( QRect( p + QPoint( 2, 2 ), s->size() - QSize( 4, 4 ) ) );
      }
-    if ( connectReceiver ) {
-	QWidget *w = (QWidget*)connectReceiver;
-	QPoint p = mapToForm( w, QPoint(0,0) );
-	unclippedPainter->drawRect( QRect( p + QPoint( 2, 2 ), w->size() - QSize( 4, 4 ) ) );
+    if ( endWidget ) {
+	QWidget *e = (QWidget*)endWidget;
+	QPoint p = mapToForm( e, QPoint(0,0) );
+	unclippedPainter->drawRect( QRect( p + QPoint( 2, 2 ), e->size() - QSize( 4, 4 ) ) );
      }
 }
 
@@ -2273,7 +2385,21 @@ bool FormWindow::unify( QObject *w, QString &s, bool changeIt )
 		}
 	    }
 	}
-	if ( mainContainer()->inherits( "QMainWindow" ) ) {
+	if ( ::qt_cast<QMainWindow*>(mainContainer()) && !found ) {
+	    QObjectList *l = mainContainer()->queryList( "PopupMenuEditor" );
+	    for ( QObject *o = l->first(); o; o = l->next() ) {
+		if ( o != w &&
+		     qstrcmp ( o->name(), s.latin1() ) == 0 ) {
+		    found = TRUE;
+		    if ( !changeIt )
+			break;
+		    s = orig + "_" + QString::number( ++num );
+		    o = l->first();
+		}
+	    }
+	    delete l;
+	}
+	if ( ::qt_cast<QMainWindow*>(mainContainer()) ) {
 	    if ( !found ) {
 		QObjectList *l = mainContainer()->queryList( "QDockWindow", 0, TRUE );
 		for ( QObject *o = l->first(); o; o = l->next() ) {
@@ -2288,20 +2414,6 @@ bool FormWindow::unify( QObject *w, QString &s, bool changeIt )
 		}
 		delete l;
 	    }
-	    if ( !found ) {
-		QObjectList *l = mainContainer()->queryList( "QDesignerPopupMenu", 0, TRUE );
-		for ( QObject *o = l->first(); o; o = l->next() ) {
-		    if ( o != w &&
-			 qstrcmp( o->name(), s.latin1() ) == 0 ) {
-			found = TRUE;
-			if ( !changeIt )
-			    break;
-			s = orig + "_" + QString::number( ++num );
-			o = l->first();
-		    }
-		}
-		delete l;
-	    }
 	}
     }
 
@@ -2411,9 +2523,8 @@ QWidget *FormWindow::containerAt( const QPoint &pos, QWidget *notParentOf )
     }
 
     for ( ; it.current(); ++it ) {
-	if ( it.current()->inherits( "QLayoutWidget" ) )
-	    continue;
-	if ( it.current()->inherits( "QSplitter" ) )
+	if ( ::qt_cast<QLayoutWidget*>(it.current())
+	  || ::qt_cast<QSplitter*>(it.current()) )
 	    continue;
 	if ( !it.current()->isVisibleTo( this ) )
 	    continue;
@@ -2473,11 +2584,18 @@ void FormWindow::setMainContainer( QWidget *w )
     }
     if ( project() ) {
 	LanguageInterface *iface = MetaDataBase::languageInterface( project()->language() );
-	if ( iface && !MetaDataBase::hasEventFunctions( mainContainer() ) ) {
-	    QMap<QString, QString> eventFuncs;
-	    iface->initEventFunctions( eventFuncs );
-	    for ( QMap<QString, QString>::Iterator it = eventFuncs.begin(); it != eventFuncs.end(); ++it )
-		MetaDataBase::setEventFunctions( mainContainer(), this, project()->language(), it.key(), QStringList( *it ) );
+	if ( iface && !project()->isCpp() && !isFake() ) {
+	    if ( !MetaDataBase::hasFunction( this, "init()" ) )
+		MetaDataBase::addFunction( this, "init()", "", "private", "function",
+					   project()->language(), "void" );
+	    if ( !MetaDataBase::hasFunction( this, "destroy()" ) )
+		MetaDataBase::addFunction( this, "destroy()", "", "private", "function",
+					   project()->language(), "void" );
+	    if ( !MetaDataBase::hasConnection( this, mainContainer(), "shown()", mainContainer(), "init" ) )
+		MetaDataBase::addConnection( this, mainContainer(), "shown()", mainContainer(), "init" );
+	    if ( !MetaDataBase::hasConnection( this, mainContainer(), "destroyed()", mainContainer(), "destroy"  ) )
+		MetaDataBase::addConnection( this, mainContainer(), "destroyed()",
+					     mainContainer(), "destroy" );
 	}
     }
 }
@@ -2563,21 +2681,11 @@ DesignerFormWindow *FormWindow::iFace()
 
 bool FormWindow::isCentralWidget( QObject *w ) const
 {
-    if ( !mainContainer()->inherits( "QMainWindow" ) )
+    if ( !::qt_cast<QMainWindow*>(mainContainer()) )
 	return FALSE;
     return w == ( (QMainWindow*)mainContainer() )->centralWidget();
 }
 
-QObject *FormWindow::connectableObject( QObject *w, QObject * )
-{
-    if ( !project() )
-	return w;
-    LanguageInterface *iface = MetaDataBase::languageInterface( project()->language() );
-    if ( !iface )
-	return w;
-    return w;
-}
-
 int FormWindow::layoutDefaultSpacing() const
 {
     return defSpacing;
@@ -2598,6 +2706,36 @@ void FormWindow::setLayoutDefaultMargin( int s )
     defMargin = s;
 }
 
+void FormWindow::setSpacingFunction( const QString &funct )
+{
+    spacFunction = funct;
+}
+
+QString FormWindow::spacingFunction() const
+{
+    return spacFunction;
+}
+
+void FormWindow::hasLayoutFunctions( bool b )
+{
+    hasLayoutFunc = b;
+}
+
+bool FormWindow::hasLayoutFunctions() const
+{
+    return hasLayoutFunc;
+}
+
+void FormWindow::setMarginFunction( const QString &funct )
+{
+    margFunction = funct;
+}
+
+QString FormWindow::marginFunction() const
+{
+    return margFunction;
+}
+
 FormFile *FormWindow::formFile() const
 {
     return ff;
@@ -2609,3 +2747,8 @@ void FormWindow::setFormFile( FormFile *f )
     if ( ff )
 	connect( this, SIGNAL( modificationChanged(bool, const QString&) ), ff, SLOT( formWindowChangedSomehow() ) );
 }
+
+bool FormWindow::canBeBuddy( const QWidget *w ) const
+{
+    return w->focusPolicy() != QWidget::NoFocus;
+}
