@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of Qt Designer.
 **
@@ -227,8 +227,10 @@ bool SourceFile::closeEvent()
 	load();
 	if ( ed )
 	    ed->editorInterface()->setText( txt );
-	if ( fileNameTemp )
+	if ( fileNameTemp ) {
 	    pro->removeSourceFile( this );
+	    return TRUE;
+	}
 	if ( MainWindow::self )
 	    MainWindow::self->workspace()->update();
 	break;
