@@ -1,6 +1,6 @@
 /**********************************************************************
 **
-** Copyright (C) 2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of Qt Designer.
 **
@@ -9,10 +9,16 @@
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.
 **
+** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
+** licenses may use this file in accordance with the Qt Commercial License
+** Agreement provided with the Software.
+**
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** See http://www.trolltech.com/gpl/ for GPL licensing information.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+**   information about Qt Commercial License Agreements.
 **
 ** Contact info@trolltech.com if any conditions of this licensing are
 ** not clear to you.
@@ -24,6 +30,7 @@
 #include <designerinterface.h>
 #include <qfile.h>
 #include "yyreg.h"
+#include <qmetaobject.h>
 
 LanguageInterfaceImpl::LanguageInterfaceImpl( QUnknownInterface *outer )
     : parent( outer ), ref( 0 )
@@ -81,7 +88,7 @@ void LanguageInterfaceImpl::functions( const QString &code, QValueList<Function>
 	func.name.remove( 0, (*it).returnType().length() );
 	if ( func.name.find( "::" ) == -1 )
 	    continue;
-	func.name.remove( 0, func.name.find( "::" ) + 2 );
+	func.name.remove( (uint)0, func.name.find( "::" ) + 2 );
 	func.body = (*it).body();
 	func.returnType = (*it).returnType();
 	func.start = (*it).functionStartLineNum();
@@ -90,7 +97,9 @@ void LanguageInterfaceImpl::functions( const QString &code, QValueList<Function>
     }
 }
 
-QString LanguageInterfaceImpl::createFunctionStart( const QString &className, const QString &func, const QString &returnType )
+QString LanguageInterfaceImpl::createFunctionStart( const QString &className, const QString &func,
+						    const QString &returnType,
+						    const QString & )
 {
     return returnType + " " + className + "::" + func;
 }
@@ -98,7 +107,7 @@ QString LanguageInterfaceImpl::createFunctionStart( const QString &className, co
 QStringList LanguageInterfaceImpl::definitions() const
 {
     QStringList lst;
-    lst << "Includes (in Implementation)" <<  "Class Variables" << "Includes (in Declaration)" << "Forward Declarations" << "Signals";
+    lst << "Includes (in Implementation)" << "Includes (in Declaration)" << "Forward Declarations" << "Signals";
     return lst;
 }
 
@@ -118,8 +127,6 @@ QStringList LanguageInterfaceImpl::definitionEntries( const QString &definition,
 	lst = fw->declarationIncludes();
     } else if ( definition == "Forward Declarations" ) {
 	lst = fw->forwardDeclarations();
-    } else if ( definition == "Class Variables" ) {
-	lst = fw->variables();
     } else if ( definition == "Signals" ) {
 	lst = fw->signalList();
     }
@@ -142,28 +149,15 @@ void LanguageInterfaceImpl::setDefinitionEntries( const QString &definition, con
 	fw->setDeclarationIncludes( entries );
     } else if ( definition == "Forward Declarations" ) {
 	fw->setForwardDeclarations( entries );
-    } else if ( definition == "Class Variables" ) {
-	fw->setVariables( entries );
     } else if ( definition == "Signals" ) {
 	fw->setSignalList( entries );
     }
     iface->release();
 }
 
-QString LanguageInterfaceImpl::createArguments( const QStringList &args )
-{
-    QString s;
-    for ( QStringList::ConstIterator it = args.begin(); it != args.end(); ++it ) {
-	if ( it != args.begin() )
-	    s += ",";
-	s += *it;
-    }
-    return s;
-}
-
 QString LanguageInterfaceImpl::createEmptyFunction()
 {
-    return "{\n\n}";
+    return "{\n\n}\n";
 }
 
 bool LanguageInterfaceImpl::supports( Support s ) const
@@ -172,10 +166,6 @@ bool LanguageInterfaceImpl::supports( Support s ) const
 	return TRUE;
     if ( s == ConnectionsToCustomSlots )
 	return TRUE;
-    if ( s == AdditionalFiles )
-	return TRUE;
-    if ( s == StoreFormCodeSeperate )
-	return TRUE;
     return FALSE;
 }
 
@@ -195,8 +185,13 @@ QStringList LanguageInterfaceImpl::fileExtensionList() const
 
 QString LanguageInterfaceImpl::projectKeyForExtension( const QString &extension ) const
 {
-    if ( extension[ 0 ] == 'c' || extension[ 0 ] == 'C' )
-	return "SOURCES";
+    // also handle something like foo.ut.cpp
+    QString ext = extension;
+    int i = ext.findRev('.');
+    if ( i > -1 && i < (int)(ext.length()-1) )
+        ext = ext.mid( i + 1 );
+    if ( ext[ 0 ] == 'c' || ext[ 0 ] == 'C' )
+	    return "SOURCES";
     return "HEADERS";
 }
 
@@ -219,39 +214,9 @@ bool LanguageInterfaceImpl::canConnect( const QString &signal, const QString &sl
     return o.checkConnectArgs( signal.latin1(), slot.latin1() );
 }
 
-void LanguageInterfaceImpl::saveFormCode( const QString &form, const QString &filename,
-					       const QValueList<Function> &functions,
-					       const QStringList &,
-					       const QStringList &,
-					       const QStringList &,
-					       const QStringList &,
-					       const QValueList<Connection> & )
-{
-    QFile f( filename );
-    if ( !f.open( IO_WriteOnly | IO_Translate ) )
-	return;
-    QTextStream ts( &f );
-
-    if ( !functions.isEmpty() ) {
-	for ( QValueList<Function>::ConstIterator it = functions.begin();
-	      it != functions.end(); ++it ) {
-	    if ( (*it).returnType.isEmpty() )
-		ts << "void ";
-	    else
-		ts << (*it).returnType << " ";
-	    ts << form << "::" << (*it).name << endl;
-	    ts <<  (*it).body;
-	    ts << endl << endl;
-	}
-    }
-}
-
 void LanguageInterfaceImpl::loadFormCode( const QString &, const QString &filename,
 					       QValueList<Function> &functions,
 					       QStringList &,
-					       QStringList &,
-					       QStringList &,
-					       QStringList &,
 					       QValueList<Connection> & )
 {
     QFile f( filename );
@@ -267,3 +232,11 @@ void LanguageInterfaceImpl::preferedExtensions( QMap<QString, QString> &extensio
     extensionMap.insert( "cpp", "C++ Source File" );
     extensionMap.insert( "h", "C++ Header File" );
 }
+
+QStrList LanguageInterfaceImpl::signalNames( QObject *obj ) const
+{
+    QStrList sigs;
+    sigs = obj->metaObject()->signalNames( TRUE );
+    sigs.remove( "destroyed()" );
+    return sigs;
+}
