@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of Qt Designer.
 **
@@ -8,10 +8,16 @@
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.
 **
+** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
+** licenses may use this file in accordance with the Qt Commercial License
+** Agreement provided with the Software.
+**
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** See http://www.trolltech.com/gpl/ for GPL licensing information.
+** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
+**   information about Qt Commercial License Agreements.
 **
 ** Contact info@trolltech.com if any conditions of this licensing are
 ** not clear to you.
@@ -20,14 +26,14 @@
 
 #include "uic.h"
 #include "parser.h"
-#include "widgetdatabase.h"
 #include "domtool.h"
 #include <qregexp.h>
 #include <qsizepolicy.h>
 #include <qstringlist.h>
 #define NO_STATIC_COLORS
 #include <globaldefs.h>
-#include <zlib.h>
+#include "../interfaces/widgetinterface.h"
+#include "../shared/widgetdatabase.h"
 
 /*!
   Creates a declaration for the object given in \a e.
@@ -58,6 +64,8 @@ void Uic::createObjectDecl( const QDomElement& e )
 	objName = registerObject( objName );
 	if ( objClass == "Line" )
 	    objClass = "QFrame";
+	else if (objClass == "Spacer")
+	    objClass = "QSpacerItem";
 	out << "    " << objClass << "* " << objName << ";" << endl;
     }
 }
@@ -86,6 +94,9 @@ QString Uic::createObjectImpl( const QDomElement &e, const QString& parentClass,
     }
     QDomElement n;
     QString objClass, objName;
+    int numItems = 0;
+    int numColumns = 0;
+    int numRows = 0;
 
     if ( layouts.contains( e.tagName() ) )
 	return createLayoutImpl( e, parentClass, parent, layout );
@@ -133,6 +144,17 @@ QString Uic::createObjectImpl( const QDomElement &e, const QString& parentClass,
 	out << objName << " = new " << createObjectInstance( objClass, parent, objName ) << ";" << endl;
     }
 
+    if ( objClass == "QAxWidget" ) {
+	QString controlId;
+	for ( n = e.firstChild().toElement(); !n.isNull(); n = n.nextSibling().toElement() ) {
+	    if ( n.tagName() == "property" && n.attribute( "name" ) == "control" ) {
+		controlId = n.firstChild().toElement().text();
+	    }
+	}
+	out << "    ";
+	out << objName << "->setControl(\"" << controlId << "\");" << endl;
+    }
+
     lastItem = "0";
     // set the properties and insert items
     bool hadFrameShadow = FALSE;
@@ -141,7 +163,9 @@ QString Uic::createObjectImpl( const QDomElement &e, const QString& parentClass,
 	    bool stdset = stdsetdef;
 	    if ( n.hasAttribute( "stdset" ) )
 		stdset = toBool( n.attribute( "stdset" ) );
-	    QString prop = n.attribute("name");
+	    QString prop = n.attribute( "name" );
+	    if ( prop == "database" )
+		continue;
 	    QString value = setObjectProperty( objClass, objName, prop, n.firstChild().toElement(), stdset );
 	    if ( value.isEmpty() )
 		continue;
@@ -149,7 +173,7 @@ QString Uic::createObjectImpl( const QDomElement &e, const QString& parentClass,
 		continue;
 	    if ( isLine && prop == "frameShadow" )
 		hadFrameShadow = TRUE;
-	    if ( prop == "buddy" && value[0] == '\"' && value[(int)value.length()-1] == '\"' ) {
+	    if ( prop == "buddy" && value.startsWith("\"") && value.endsWith("\"") ) {
 		buddies << Buddy( objName, value.mid(1, value.length() - 2 ) );
 		continue;
 	    }
@@ -174,41 +198,81 @@ QString Uic::createObjectImpl( const QDomElement &e, const QString& parentClass,
 	    if ( objClass == "QMultiLineEdit" &&
 		 QRegExp("echoMode|hMargin|maxLength|maxLines|undoEnabled").exactMatch(prop) )
 		continue;
-	    if ( prop == "geometry" ) {
-		out << indent << objName << "->setGeometry( " << value << " ); " << endl;
+
+	    QString call = objName + "->";
+	    if ( stdset ) {
+		call += mkStdSet( prop ) + "( ";
 	    } else {
-		if ( stdset )
-		    out << indent << objName << "->" << mkStdSet( prop ) << "( " << value << " );" << endl;
-		else
-		    out << indent << objName << "->setProperty( \"" << prop << "\", " << value << " );" << endl;
+		call += "setProperty( \"" + prop + "\", ";
+	    }
+	    if ( prop == "accel" )
+		call += "QKeySequence( " + value + " ) );";
+	    else
+		call += value + " );";
+
+	    if ( n.firstChild().toElement().tagName() == "string" ||
+		 prop == "currentItem" ) {
+		trout << indent << call << endl;
+	    } else {
+		out << indent << call << endl;
 	    }
 	} else if ( n.tagName() == "item" ) {
-	    if ( objClass.mid( 1 ) == "ListBox" ) {
-		QString s = createListBoxItemImpl( n, objName );
-		if ( !s.isEmpty() )
-		    out << indent << s << endl;
-	    } else if ( objClass.mid( 1 ) == "ComboBox" ) {
-		QString s = createListBoxItemImpl( n, objName );
-		if ( !s.isEmpty() )
-		    out << indent << s << endl;
-	    } else if ( objClass.mid( 1 ) == "IconView" ) {
-		QString s = createIconViewItemImpl( n, objName );
-		if ( !s.isEmpty() )
-		    out << indent << s << endl;
-	    } else if ( objClass.mid( 1 ) == "ListView" ) {
-		QString s = createListViewItemImpl( n, objName, QString::null );
-		if ( !s.isEmpty() )
-		    out << s << endl;
+	    QString call;
+	    QString value;
+
+	    if ( objClass.contains( "ListBox" ) ) {
+		call = createListBoxItemImpl( n, objName );
+		if ( !call.isEmpty() ) {
+		    if ( numItems == 0 )
+			trout << indent << objName << "->clear();" << endl;
+		    trout << indent << call << endl;
+		}
+	    } else if ( objClass.contains( "ComboBox" ) ) {
+		call = createListBoxItemImpl( n, objName, &value );
+		if ( !call.isEmpty() ) {
+		    if ( numItems == 0 )
+			trout << indent << objName << "->clear();" << endl;
+		    trout << indent << call << endl;
+		}
+	    } else if ( objClass.contains( "IconView" ) ) {
+		call = createIconViewItemImpl( n, objName );
+		if ( !call.isEmpty() ) {
+		    if ( numItems == 0 )
+			trout << indent << objName << "->clear();" << endl;
+		    trout << indent << call << endl;
+		}
+	    } else if ( objClass.contains( "ListView" ) ) {
+		call = createListViewItemImpl( n, objName, QString::null );
+		if ( !call.isEmpty() ) {
+		    if ( numItems == 0 )
+			trout << indent << objName << "->clear();" << endl;
+		    trout << call << endl;
+		}
 	    }
+	    if ( !call.isEmpty() )
+		numItems++;
 	} else if ( n.tagName() == "column" || n.tagName() == "row" ) {
-	    if ( objClass.mid( 1 ) == "ListView" ) {
-		QString s = createListViewColumnImpl( n, objName );
-		if ( !s.isEmpty() )
-		    out << s;
+	    QString call;
+	    QString value;
+
+	    if ( objClass.contains( "ListView" ) ) {
+		call = createListViewColumnImpl( n, objName, &value );
+		if ( !call.isEmpty() ) {
+		    out << call;
+		    trout << indent << objName << "->header()->setLabel( "
+			  << numColumns++ << ", " << value << " );\n";
+		}
 	    } else if ( objClass ==  "QTable" || objClass == "QDataTable" ) {
-		QString s = createTableRowColumnImpl( n, objName );
-		if ( !s.isEmpty() )
-		    out << s;
+		bool isCols = ( n.tagName() == "column" );
+		call = createTableRowColumnImpl( n, objName, &value );
+		if ( !call.isEmpty() ) {
+		    out << call;
+		    trout << indent << objName << "->"
+			  << ( isCols ? "horizontalHeader" : "verticalHeader" )
+			  << "()->setLabel( "
+			  << ( isCols ? numColumns++ : numRows++ )
+			  << ", " << value << " );\n";
+		}
 	    }
 	}
     }
@@ -219,21 +283,85 @@ QString Uic::createObjectImpl( const QDomElement &e, const QString& parentClass,
 	for ( n = e.firstChild().toElement(); !n.isNull(); n = n.nextSibling().toElement() ) {
 	    if ( tags.contains( n.tagName()  ) ) {
 		QString page = createObjectImpl( n, objClass, objName );
-		QString label = DomTool::readAttribute( n, "title", "" ).toString();
-		out << indent << objName << "->insertTab( " << page << ", " << trcall( label ) << " );" << endl;
+		QString comment;
+		QString label = DomTool::readAttribute( n, "title", "", comment ).toString();
+		out << indent << objName << "->insertTab( " << page << ", QString::fromLatin1(\"\") );" << endl;
+		trout << indent << objName << "->changeTab( " << page << ", "
+		      << trcall( label, comment ) << " );" << endl;
+	    }
+	}
+    } else if ( objClass == "QWidgetStack" ) {
+	for ( n = e.firstChild().toElement(); !n.isNull(); n = n.nextSibling().toElement() ) {
+	    if ( tags.contains( n.tagName()  ) ) {
+		QString page = createObjectImpl( n, objClass, objName );
+		int id = DomTool::readAttribute( n, "id", "" ).toInt();
+		out << indent << objName << "->addWidget( " << page << ", " << id << " );" << endl;
+	    }
+	}
+    } else if ( objClass == "QToolBox" ) {
+	for ( n = e.firstChild().toElement(); !n.isNull(); n = n.nextSibling().toElement() ) {
+	    if ( tags.contains( n.tagName()  ) ) {
+		QString page = createObjectImpl( n, objClass, objName );
+		QString comment;
+		QString label = DomTool::readAttribute( n, "label", "", comment ).toString();
+		out << indent << objName << "->addItem( " << page << ", QString::fromLatin1(\"\") );" << endl;
+		trout << indent << objName << "->setItemLabel( " << objName 
+		      << "->indexOf(" << page << "), " << trcall( label, comment ) 
+		      << " );" << endl;
 	    }
 	}
      } else if ( objClass != "QToolBar" && objClass != "QMenuBar" ) { // standard widgets
-	for ( n = e.firstChild().toElement(); !n.isNull(); n = n.nextSibling().toElement() ) {
-	    if ( tags.contains( n.tagName() ) )
-		createObjectImpl( n, objClass, objName );
-	}
-    }
+	 WidgetInterface *iface = 0;
+	 widgetManager()->queryInterface( objClass, &iface );
+#ifdef QT_CONTAINER_CUSTOM_WIDGETS
+	 int id = WidgetDatabase::idFromClassName( objClass );
+	 if ( WidgetDatabase::isContainer( id ) && WidgetDatabase::isCustomPluginWidget( id ) && iface ) {
+	     QWidgetContainerInterfacePrivate *iface2 = 0;
+	     iface->queryInterface( IID_QWidgetContainer, (QUnknownInterface**)&iface2 );
+	     if ( iface2 ) {
+		 bool supportsPages = iface2->supportsPages( objClass );
+		 for ( n = e.firstChild().toElement(); !n.isNull(); n = n.nextSibling().toElement() ) {
+		     if ( tags.contains( n.tagName()  ) ) {
+			 if ( supportsPages ) {
+			     QString page = createObjectImpl( n, objClass, objName );
+			     QString comment;
+			     QString label = DomTool::readAttribute( n, "label", "", comment ).toString();
+			     out << indent << iface2->createCode( objClass, objName, page, label ) << endl;
+			 } else {
+			     createObjectImpl( n, objClass, objName );
+			 }
+		     }
+		 }
+		 iface2->release();
+	     }
+	     iface->release();
+	 } else {
+#endif
+	     for ( n = e.firstChild().toElement(); !n.isNull(); n = n.nextSibling().toElement() ) {
+		 if ( tags.contains( n.tagName() ) )
+		     createObjectImpl( n, objClass, objName );
+	     }
+#ifdef QT_CONTAINER_CUSTOM_WIDGETS
+	 }
+#endif
+     }
 
     return objName;
 }
 
+/*!
+  Creates declarations for spacer items that are children of \a e.
 
+  \sa createObjectDecl()
+*/
+
+void Uic::createSpacerDecl( const QDomElement &e )
+{
+    for ( QDomElement n = e.firstChild().toElement();
+	  !n.isNull(); n = n.nextSibling().toElement() )
+	if ( n.tagName() == "spacer" )
+	    out << "    QSpacerItem* " << registerObject(getObjectName(n)) << ";" << endl;
+}
 
 /*!
   Creates a set-call for property \a exclusiveProp of the object
@@ -252,20 +380,23 @@ void Uic::createExclusiveProperty( const QDomElement & e, const QString& exclusi
     if ( objClass.isEmpty() )
 	return;
     QString objName = getObjectName( e );
-    if ( objClass.isEmpty() )
+#if 0 // it's not clear whether this check should be here or not
+    if ( objName.isEmpty() )
 	return;
+#endif
     for ( n = e.firstChild().toElement(); !n.isNull(); n = n.nextSibling().toElement() ) {
 	if ( n.tagName() == "property" ) {
 	    bool stdset = stdsetdef;
 	    if ( n.hasAttribute( "stdset" ) )
 		stdset = toBool( n.attribute( "stdset" ) );
-	    QString prop = n.attribute("name");
+	    QString prop = n.attribute( "name" );
 	    if ( prop != exclusiveProp )
 		continue;
 	    QString value = setObjectProperty( objClass, objName, prop, n.firstChild().toElement(), stdset );
 	    if ( value.isEmpty() )
 		continue;
-	    out << indent << indent << objName << "->setProperty( \"" << prop << "\", " << value << " );" << endl;
+	    // we assume the property isn't of type 'string'
+	    out << '\t' << objName << "->setProperty( \"" << prop << "\", " << value << " );" << endl;
 	}
     }
 }
@@ -321,7 +452,7 @@ QString Uic::setObjectProperty( const QString& objClass, const QString& obj, con
 	v = v.arg(w).arg(h);
     } else if ( e.tagName() == "color" ) {
 	QDomElement n3 = e.firstChild().toElement();
-	int r= 0, g = 0, b = 0;
+	int r = 0, g = 0, b = 0;
 	while ( !n3.isNull() ) {
 	    if ( n3.tagName() == "red" )
 		r = n3.firstChild().toText().data().toInt();
@@ -335,9 +466,10 @@ QString Uic::setObjectProperty( const QString& objClass, const QString& obj, con
 	v = v.arg(r).arg(g).arg(b);
     } else if ( e.tagName() == "font" ) {
 	QDomElement n3 = e.firstChild().toElement();
+	QString attrname = e.parentNode().toElement().attribute( "name", "font" );
 	QString fontname;
 	if ( !obj.isEmpty() ) {
-	    fontname = obj + "_font";
+	    fontname = registerObject( obj + "_" + attrname );
 	    out << indent << "QFont "  << fontname << "(  " << obj << "->font() );" << endl;
 	} else {
 	    fontname = registerObject( "f" );
@@ -373,14 +505,18 @@ QString Uic::setObjectProperty( const QString& objClass, const QString& obj, con
 
 	if ( prop == "toolTip" && objClass != "QAction" && objClass != "QActionGroup" ) {
 	    if ( !obj.isEmpty() )
-		out << indent << "QToolTip::add( " << obj << ", " + trcall( txt, com ) << " );" << endl;
+		trout << indent << "QToolTip::add( " << obj << ", "
+		      << trcall( txt, com ) << " );" << endl;
 	    else
-		out << indent << "QToolTip::add( this, " << trcall( txt, com ) << " );" << endl;
+		trout << indent << "QToolTip::add( this, "
+		      << trcall( txt, com ) << " );" << endl;
 	} else if ( prop == "whatsThis" && objClass != "QAction" && objClass != "QActionGroup" ) {
 	    if ( !obj.isEmpty() )
-		out << indent << "QWhatsThis::add( " << obj << ", " << trcall( txt, com ) << " );" << endl;
+		trout << indent << "QWhatsThis::add( " << obj << ", "
+		      << trcall( txt, com ) << " );" << endl;
 	    else
-		out << indent << "QWhatsThis::add( this, " << trcall( txt, com ) << " );" << endl;
+		trout << indent << "QWhatsThis::add( this, "
+		      << trcall( txt, com ) << " );" << endl;
 	} else {
 	    v = trcall( txt, com );
 	}
@@ -416,27 +552,25 @@ QString Uic::setObjectProperty( const QString& objClass, const QString& obj, con
 	if ( stdset )
 	    v = "%1::%2";
 	else
-	    v = "(int)%1::%2";
+	    v = "\"%1\"";
 	QString oc = objClass;
 	QString ev = e.firstChild().toText().data();
-	if ( oc == "QListView" && ev == "Manual" ) // #### workaround, rename QListView::Manual of WithMode enum in 3.0
+	if ( oc == "QListView" && ev == "Manual" ) // #### workaround, rename QListView::Manual in 4.0
 	    oc = "QScrollView";
-	v = v.arg( oc ).arg( ev );
+	if ( stdset )
+	    v = v.arg( oc ).arg( ev );
+	else
+	    v = v.arg( ev );
     } else if ( e.tagName() == "set" ) {
 	QString keys( e.firstChild().toText().data() );
 	QStringList lst = QStringList::split( '|', keys );
 	v = "int( ";
-#if defined(Q_CC_EDG)
-	// workaround for EDG bug reproduced with MIPSpro C++ 7.3.?
-	// and KAI C++ 4.0e that will be fixed in KAI C++ 4.0f
 	QStringList::Iterator it = lst.begin();
-	for ( ; it != lst.end(); ++it ) {
-#else
-	for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it ) {
-#endif
+	while ( it != lst.end() ) {
 	    v += objClass + "::" + *it;
 	    if ( it != lst.fromLast() )
 		v += " | ";
+	    ++it;
 	}
 	v += " )";
     } else if ( e.tagName() == "sizepolicy" ) {
@@ -576,6 +710,7 @@ QString Uic::setObjectProperty( const QString& objClass, const QString& obj, con
 	    listname = registerObject( listname );
 	    out << indent << "QStringList "  << listname << ";" << endl;
 	} else {
+	    listname = registerObject( listname );
 	    out << indent << "QStringList "  << listname << ";" << endl;
 	}
 	while ( !n3.isNull() ) {
@@ -583,14 +718,11 @@ QString Uic::setObjectProperty( const QString& objClass, const QString& obj, con
 		out << indent << listname << " << \"" << n3.firstChild().toText().data().simplifyWhiteSpace() << "\";" << endl;
 	    n3 = n3.nextSibling().toElement();
 	}
-	v = listname;	
+	v = listname;
     }
     return v;
 }
 
-
-
-
 /*! Extracts a named object property from \a e.
  */
 QDomElement Uic::getObjectProperty( const QDomElement& e, const QString& name )
@@ -604,4 +736,3 @@ QDomElement Uic::getObjectProperty( const QDomElement& e, const QString& name )
     }
     return n;
 }
-
