@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2000 Trolltech AS.  All rights reserved.
+** Copyright (C) 2005 Trolltech AS.  All rights reserved.
 **
 ** This file is part of Qt Designer.
 **
@@ -77,8 +77,15 @@ QString Uic::fixString( const QString &str, bool encode )
     } else {
 	QCString utf8 = str.utf8();
 	const int l = utf8.length();
-	for ( int i = 0; i < l; ++i )
-	    s += "\\x" + QString::number( (uchar)utf8[i], 16 );
+
+        for ( int i = 0; i < l; ++i ) {
+            uchar c = (uchar)utf8[i];
+            if (c != 0x0d) // skip \r
+               s += "\\x" + QString::number(c, 16);
+
+            if (i && (i % 20) == 0)
+                s += "\"\n    \"";
+        }
     }
 
     return "\"" + s + "\"";
@@ -145,6 +152,7 @@ Uic::Uic( const QString &fn, const char *outputFn, QTextStream &outStream,
     pixmapLoaderFunction = getPixmapLoaderFunction( doc.firstChild().toElement() );
     nameOfClass = getFormClassName( doc.firstChild().toElement() );
 
+    uiFileVersion = doc.firstChild().toElement().attribute("version");
     stdsetdef = toBool( doc.firstChild().toElement().attribute("stdsetdef") );
 
     if ( doc.firstChild().isNull() || doc.firstChild().firstChild().isNull() )
@@ -314,8 +322,12 @@ QString Uic::getDatabaseInfo( const QDomElement& e, const QString& tag )
 
 void Uic::registerLayouts( const QDomElement &e )
 {
-    if ( layouts.contains(e.tagName()) )
+    if ( layouts.contains(e.tagName())) {
 	createObjectDecl(e);
+	QString t = e.tagName();
+	if ( t == "vbox" || t == "hbox" || t == "grid" )
+	    createSpacerDecl( e );
+    }
 
     QDomNodeList nl = e.childNodes();
     for ( int i = 0; i < (int) nl.length(); ++i )
@@ -358,8 +370,24 @@ void Uic::createToolbarDecl( const QDomElement &e )
 
 void Uic::createMenuBarDecl( const QDomElement &e )
 {
-    if ( e.tagName() == "item" )
+    if ( e.tagName() == "item" ) {
 	out << "    " << "QPopupMenu *" << e.attribute( "name" ) << ";" << endl;
+	createPopupMenuDecl( e );
+    }
+}
+
+void Uic::createPopupMenuDecl( const QDomElement &e )
+{
+    for ( QDomElement n = e.firstChild().toElement(); !n.isNull(); n = n.nextSibling().toElement() ) {
+	if ( n.tagName() == "action" || n.tagName() == "actiongroup" ) {
+	    QDomElement n2 = n.nextSibling().toElement();
+	    if ( n2.tagName() == "item" ) { // the action has a sub menu
+		out << "    " << "QPopupMenu *" << n2.attribute( "name" ) << ";" << endl;
+		createPopupMenuDecl( n2 );
+		n = n2;
+	    }
+	}
+    }
 }
 
 void Uic::createActionImpl( const QDomElement &n, const QString &parent )
@@ -373,6 +401,8 @@ void Uic::createActionImpl( const QDomElement &n, const QString &parent )
 	else
 	    continue;
 	bool subActionsDone = FALSE;
+	bool hasMenuText = FALSE;
+	QString actionText;
 	for ( QDomElement n2 = ae.firstChild().toElement(); !n2.isNull(); n2 = n2.nextSibling().toElement() ) {
 	    if ( n2.tagName() == "property" ) {
 		bool stdset = stdsetdef;
@@ -393,6 +423,11 @@ void Uic::createActionImpl( const QDomElement &n, const QString &parent )
 		}
 		call += value + " );";
 
+		if (prop == "menuText")
+		    hasMenuText = TRUE;
+		else if (prop == "text")
+		    actionText = value;
+
 		if ( n2.firstChild().toElement().tagName() == "string" ) {
 		    trout << indent << call << endl;
 		} else {
@@ -403,6 +438,9 @@ void Uic::createActionImpl( const QDomElement &n, const QString &parent )
 		subActionsDone = TRUE;
 	    }
 	}
+	// workaround for loading pre-3.3 files expecting bogus QAction behavior
+	if (!hasMenuText && !actionText.isEmpty() && uiFileVersion < "3.3")
+	    trout << indent << objName << "->setMenuText(" << actionText << ");" << endl;
     }
 }
 
@@ -443,7 +481,7 @@ void Uic::createToolbarImpl( const QDomElement &n, const QString &parentClass, c
 		if ( n2.attribute( "class" ) != "Spacer" ) {
 		    createObjectImpl( n2, "QToolBar", objName );
 		} else {
-		    QString child = createSpacerImpl( n, parentClass, parent, objName );
+		    QString child = createSpacerImpl( n2, parentClass, parent, objName );
 		    out << indent << "QApplication::sendPostedEvents( " << objName
 			<< ", QEvent::ChildInserted );" << endl;
 		    out << indent << objName << "->boxLayout()->addItem( " << child << " );" << endl;
@@ -467,7 +505,10 @@ void Uic::createMenuBarImpl( const QDomElement &n, const QString &parentClass, c
 	    out << indent << itemName << " = new QPopupMenu( this );" << endl;
 	    createPopupMenuImpl( c, parentClass, itemName );
 	    out << indent << objName << "->insertItem( QString(\"\"), " << itemName << ", " << i << " );" << endl;
-	    trout << indent << objName << "->findItem( " << i << " )->setText( " << trcall( c.attribute( "text" ) ) << " );" << endl;
+	    QString findItem(objName + "->findItem(%1)");
+	    findItem = findItem.arg(i);
+	    trout << indent << "if (" << findItem << ")" << endl;
+	    trout << indent << indent << findItem << "->setText( " << trcall( c.attribute( "text" ) ) << " );" << endl;
 	} else if ( c.tagName() == "separator" ) {
 	    out << endl;
 	    out << indent << objName << "->insertSeparator( " << i << " );" << endl;
@@ -479,16 +520,18 @@ void Uic::createMenuBarImpl( const QDomElement &n, const QString &parentClass, c
 
 void Uic::createPopupMenuImpl( const QDomElement &e, const QString &parentClass, const QString &parent )
 {
+    int i = 0;
     for ( QDomElement n = e.firstChild().toElement(); !n.isNull(); n = n.nextSibling().toElement() ) {
-	if ( n.tagName() == "action" ) {
+	if ( n.tagName() == "action" || n.tagName() == "actiongroup" ) {
 	    QDomElement n2 = n.nextSibling().toElement();
 	    if ( n2.tagName() == "item" ) { // the action has a sub menu
 		QString itemName = n2.attribute( "name" );
 		QString itemText = n2.attribute( "text" );
-		out << indent << "QPopupMenu *" << itemName << " = new QPopupMenu( this );" << endl;
-		out << indent << parent << "->setAccel( tr( \"" << n2.attribute( "accel" ) << "\" ), " << endl;
-		out << indent << indent << parent << "->insertItem( " << n.attribute( "name" ) << "->iconSet(),";
-		out << trcall( itemText ) << ", " << itemName << " ) );" << endl;
+		out << indent << itemName << " = new QPopupMenu( this );" << endl;
+		out << indent << parent << "->insertItem( "  << n.attribute( "name" ) << "->iconSet(), ";
+		out << trcall( itemText ) << ", " << itemName << " );" << endl;
+		trout << indent << parent << "->changeItem( " << parent << "->idAt( " << i << " ), ";
+		trout << trcall( itemText ) << " );" << endl;
 		createPopupMenuImpl( n2, parentClass, itemName );
 		n = n2;
 	    } else {
@@ -497,6 +540,7 @@ void Uic::createPopupMenuImpl( const QDomElement &e, const QString &parentClass,
 	} else if ( n.tagName() == "separator" ) {
 	    out << indent << parent << "->insertSeparator();" << endl;
 	}
+	++i;
     }
 }
 
@@ -856,7 +900,7 @@ QString Uic::createSpacerImpl( const QDomElement &e, const QString& /*parentClas
     QDomElement n;
     QString objClass, objName;
     objClass = e.tagName();
-    objName = registerObject( "spacer" );
+    objName = registerObject( getObjectName( e ) );
 
     QSize size = DomTool::readProperty( e, "sizeHint", QSize( 0, 0 ) ).toSize();
     QString sizeType = DomTool::readProperty( e, "sizeType", "Expanding" ).toString();
@@ -869,11 +913,11 @@ QString Uic::createSpacerImpl( const QDomElement &e, const QString& /*parentClas
     }
 
     if ( isVspacer )
-	out << "    QSpacerItem* " << objName << " = new QSpacerItem( "
+	out << "    " << objName << " = new QSpacerItem( "
 	    << size.width() << ", " << size.height()
 	    << ", QSizePolicy::Minimum, QSizePolicy::" << sizeType << " );" << endl;
     else
-	out << "    QSpacerItem* " << objName << " = new QSpacerItem( "
+	out << "    " << objName << " = new QSpacerItem( "
 	    << size.width() << ", " << size.height()
 	    << ", QSizePolicy::" << sizeType << ", QSizePolicy::Minimum );" << endl;
 
